@@ -167,12 +167,11 @@ void WpeXColorGet(XrmDatabase xresdb, XrmQuark *name_list,
  depth = DisplayPlanes(WpeXInfo.display, WpeXInfo.screen);
  visual = DefaultVisual(WpeXInfo.display, WpeXInfo.screen);
 
- if(depth == 1)
+ if (depth == 1)
  {
-  /* Old code since haven't changed h_error */
-  extern struct EXT h_error;
+  /* Old code */
   /* Error message for depth of one removed */
-  e_X_sw_color(h_error.cn->fb);
+  e_X_sw_color();
   WpeXInfo.colors[0] = BlackPixel(WpeXInfo.display, WpeXInfo.screen);
   WpeXInfo.colors[1] = WhitePixel(WpeXInfo.display, WpeXInfo.screen);
   return ;
@@ -389,7 +388,7 @@ void WpeXInit(int *argc, char **argv)
  XrmInitialize();
  if (WpeIsProg())
  {
-  window_name = "Window Programming Enviroment";
+  window_name = "Window Programming Environment";
   class_hint.res_name = "xwpe";
  }
  else
