@@ -6,10 +6,10 @@
 /* modify it under the terms of the                       */
 /* GNU General Public License, see the file COPYING.      */
 /*
-                     Header Datei fuer  FK - Editor
+                     Header file for FK-editor
 */
 
-#define VERSION "1.5.22a"
+#define VERSION "1.5.26a"
 
 #include "Xwpe.h"
 #include "WeString.h"
@@ -21,8 +21,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <limits.h>
 
-#ifdef GPM
+#ifdef HAVE_LIBGPM
 #include <gpm.h>
 #endif
 
@@ -42,6 +43,9 @@ extern int MAXSLNS, MAXSCOL, MENOPT;
 #define MAXLINES 10
 #define MAXCOLUM 120
 
+#define WPE_NOBACKUP 1
+#define WPE_BACKUP   0
+
 #if  MOUSE
 struct mouse {
  int x;
@@ -50,10 +54,16 @@ struct mouse {
 };
 #endif
 
-struct EXT {
- char sw;
- struct CNT *cn;
-};
+#define DTMD_NORMAL        'n' /* Normal text file */
+#define DTMD_MSDOS         'm' /* MS-DOS text file */
+#define DTMD_HELP          'h' /* Help window */
+#define DTMD_DATA          'D' /* Data/project windows */
+#define DTMD_FILEMANAGER   'F' /* File manager */
+/* File/directory dropdown of previous files/directories on the file manager */
+#define DTMD_FILEDROPDOWN  'M'
+
+#define DTMD_ISTEXT(x)     (x > 'Z')
+#define DTMD_ISMARKABLE(x) (x > DTMD_HELP) /* Means end marks can be shown*/
 
 struct dirfile {
  int anz;      /* number elements in the list */
@@ -80,7 +90,7 @@ typedef struct PICSTR {
 typedef struct FND {
  char search[80], replace[80];
  char file[80];   /* filename or pattern to search/open */
- char dirct[128];
+ char *dirct;
  int sn;
  int rn;
  unsigned int sw;
@@ -140,7 +150,9 @@ typedef struct undo {
 
 typedef struct STR {
  unsigned char *s;
- int len, nrc;
+ int len; /* Length of string not counting '\n' at the end */
+ int nrc;
+ /*int size;*/ /* Memory allocated for the string */
 } STRING;
 
 typedef struct BFF {
@@ -210,16 +222,14 @@ typedef struct FNST {
  char *datnam;    /* window header text */
  int winnum;      /* ID number in parents structure ??? */
  char ins;
- char dtmd; /* 'D'=DATA/PRJ, 'F'=file 'M'=messages? 'Z'=??? */
+ char dtmd; /* (See DTMD_* defines) */
  int save;
  char *hlp_str;
  WOPT *blst;      /* status line text */
  int nblst;       /* no of options in the status line */
-#ifdef UNIX
  int filemode, flg;
  int *c_sw;
  struct wpeSyntaxRule *c_st;
-#endif
  struct CNT *ed;  /* parent control structure ??? */
  struct BFF *b;
  struct SCHRM *s;
@@ -332,9 +342,12 @@ typedef struct wpeOptionSection {
 
 
 /*   we_main.c   */
+extern struct CNT *WpeEditor;
+
 void e_ini_desk(ECNT *cn);
-void e_ini_farbe(FARBE *fb, int sw);
+FARBE *e_ini_farbe();
 int e_switch_blst(ECNT *cn);
+void e_free_find(FIND *fd);
 
 /*   we_block.c   */
 int e_blck_del(FENSTER *f);
@@ -413,9 +426,9 @@ int e_su_lblk(int xa, char *s);
 int e_su_rblk(int xa, char *s);
 void e_zlsplt(FENSTER *f);
 void WpeFilenameToPathFile(char *filename, char **path, char **file);
-int e_Lst_zeichen(int x, int y, int n, int sw, int frb, int max, int iold,
+int e_lst_zeichen(int x, int y, int n, int sw, int frb, int max, int iold,
   int new);
-void e_mouse_Leiste(int x, int y, int n, int sw, int frb);
+void e_mouse_bar(int x, int y, int n, int sw, int frb);
 int e_chr_sp(int x, BUFFER *b, FENSTER *f);
 Undo *e_remove_undo(Undo *ud, int sw);
 int e_add_undo(int sw, BUFFER *b, int x, int y, int n);
@@ -498,15 +511,15 @@ struct dirfile *WpeGraphicalDirTree(struct dirfile *cd, struct dirfile *dd,
 
 /*   we_fl_fkt.c   */
 char *e_mkfilename(char *dr, char *fn);
-POINT e_readin(int i, int j, FILE *fp, BUFFER *b, int sw);
+POINT e_readin(int i, int j, FILE *fp, BUFFER *b, char *sw);
 int e_new(FENSTER *f);
 int e_m_save(FENSTER *f);
 int e_save(FENSTER *f);
 int e_saveall(FENSTER *f);
 int e_quit(FENSTER *f);
-int e_write(int xa, int ya, int xe, int ye, FENSTER *f);
+int e_write(int xa, int ya, int xe, int ye, FENSTER *f, int backup);
 char *e_new_qual(char *s, char *ns, char *sb);
-char *e_bakfilename(char *s, char *sb);
+char *e_bakfilename(char *s);
 int freedf(struct dirfile *df);
 int e_file_window(int sw, FLWND *fw, int ft, int fz);
 int e_pr_file_window(FLWND *fw, int c, int sw, int ft, int fz, int fs);
@@ -519,13 +532,18 @@ int e_help_ret(FENSTER *f);
 int e_topic_search(FENSTER *f);
 
 /*   we_hfkt.c   */
-int e_toupper(int c);
 int e_strstr(int x, int n, unsigned char *s, unsigned char *f);
 int e_ustrstr(int x, int n, unsigned char *s, unsigned char *f);
 int e_urstrstr(int x, int n, unsigned char *s, unsigned char *f, int *nn);
 int e_rstrstr(int x, int n, unsigned char *s, unsigned char *f, int *nn);
 int e_str_len(unsigned char *s);
+#if 0
 int e_str_nrc(unsigned char *s);
+int e_toupper(int c);
+#else
+#define e_str_nrc(s) strlen(s)
+#define e_toupper(c) toupper(c)
+#endif
 int e_num_kst(char *s, int num, int max, FENSTER *f, int n, int sw);
 COLOR e_s_x_clr(int f, int b);
 COLOR e_n_x_clr(int fb);
@@ -697,7 +715,7 @@ extern int (*e_u_ini_size)(void);
 extern int (*e_u_s_sys_end)(void);
 extern int (*e_u_s_sys_ini)(void);
 extern void (*e_u_setlastpic)(PIC *pic);
-extern int (*e_u_system)(char *exe);
+extern int (*e_u_system)(const char *exe);
 extern int (*e_u_kbhit)(void);
 extern void (*WpeDisplayEnd)(void);
 
@@ -769,7 +787,7 @@ int e_d_reinit_watches(FENSTER *f,char * prj);
 #endif
 
 /* we_gpm.c */
-#ifdef GPM
+#ifdef HAVE_LIBGPM
 int WpeGpmInit(void);
 int WpeGpmMouse(int *g);
 #endif
