@@ -40,7 +40,6 @@ int e_d_nbrpts = 0, e_d_nwtchs = 0, e_d_zwtchs = 0, *e_d_ybrpts, *e_d_nrbrpts, *
 int e_d_nstack;
 char e_d_file[128], **e_d_sbrpts, **e_d_swtchs;
 char *e_d_out = NULL;
-int e_d_out_cur = 0;
 int e_deb_type = 0, e_deb_mode = 0;
 char e_d_out_str[SVLINES][256];
 char *e_d_sp[SVLINES];
@@ -76,22 +75,19 @@ char *e_d_msg[] = {  "Ctrl C pressed\nQuit Debugger ?",
 			"Program received signal",
 			"Unknown Break\nQuit Debugger ?",
 			"Can\'t find file %s",
-			"Terminal Name not found",  /*   Number 15   */
-			"Can\'t create named pipe",
+			"Can\'t create named pipe",  /*   Number 15   */
 			"Breakpoint",
-			"Process terminated",
 			"(sig ",
-			"program exited",      /* Number 20  */
+			"program exited",
 			"signal",
-			"interrupt",
+			"interrupt",      /* Number 20  */
 			"stopped in",
 			"BREAKPOINT",
-			"Child process terminated normally",  /* Nr 25  */
+			"Child process terminated normally",
 			"software termination",
-			"Quit Debugger ?",
-			"Continuing.\n",
+			"Continuing.\n",  /* Number 25  */
 			"No stack.",
-			"no process"    	/*	Nr 30	*/
+			"no process"
 		  };
 
 extern char *e_p_msg[];
@@ -175,7 +171,7 @@ int e_deb_inp(FENSTER *f)
 
 int e_d_q_quit(FENSTER *f)
 {
- int ret = e_message(1, e_d_msg[1], f);
+ int ret = e_message(1, e_d_msg[ERR_QUITDEBUG], f);
 
  if (ret == 'Y')
   e_d_quit(f);
@@ -340,36 +336,43 @@ int e_d_dum_read()
 /* Output Routines */
 int e_d_p_exec(FENSTER *f)
 {
-   ECNT *cn = f->ed;
-   BUFFER *b;
-   SCHIRM *s;
-   int ret, i, is, j;
-   char str[512];
+ ECNT *cn = f->ed;
+ BUFFER *b;
+ SCHIRM *s;
+ int ret, i, is, j;
+ char str[512];
 
-   for(i = cn->mxedt; i > 0 && strcmp(cn->f[i]->datnam, "Messages"); i--);
-   if(i <= 0) {  e_edit(cn, "Messages");  i = cn->mxedt;  }
-   f = cn->f[i];
-   b = cn->f[i]->b;
-   s = cn->f[i]->s;
-   if(b->bf[b->mxlines-1].len != 0)  e_new_line(b->mxlines, b);
-   for(j = 0, i = is = b->mxlines-1;
-      (ret = e_d_line_read(wfildes[0], str, 512, 0, 0)) == 0; )
-   {  if(ret == -1) return(ret);
-       print_to_end_of_buffer(b, str, b->mx.x);
-   }
-   if(ret == 1)
-   {  e_new_line(i, b);
-      for(j = 0; j < NUM_COLS_ON_SCREEN_SAFE - 2 && str[j] != '\n'
-	 && str[j] != '\0'; j++) *(b->bf[i].s+j) = str[j];
-      b->b.y = i;
-      b->b.x = b->bf[i].len = j;
-      b->bf[i].nrc = j;
-   }
-   b->b.y = b->mxlines-1;
-   b->b.x = 0;
-
-   e_rep_win_tree(cn);
+ for (i = cn->mxedt; i > 0 && strcmp(cn->f[i]->datnam, "Messages"); i--)
+  ;
+ if (i <= 0)
+ {  e_edit(cn, "Messages");  i = cn->mxedt;  }
+ f = cn->f[i];
+ b = cn->f[i]->b;
+ s = cn->f[i]->s;
+ if (b->bf[b->mxlines-1].len != 0)
+  e_new_line(b->mxlines, b);
+ for (j = 0, i = is = b->mxlines-1;
+   (ret = e_d_line_read(wfildes[0], str, 512, 0, 0)) == 0; )
+ {
+  if (ret == -1)
    return(ret);
+  print_to_end_of_buffer(b, str, b->mx.x);
+ }
+ if (ret == 1)
+ {
+  e_new_line(i, b);
+  for (j = 0; j < NUM_COLS_ON_SCREEN_SAFE - 2 && str[j] != '\n' &&
+    str[j] != '\0'; j++)
+   *(b->bf[i].s+j) = str[j];
+  b->b.y = i;
+  b->b.x = b->bf[i].len = j;
+  b->bf[i].nrc = j;
+ }
+ b->b.y = b->mxlines-1;
+ b->b.x = 0;
+
+ e_rep_win_tree(cn);
+ return(ret);
 }
 
 /*    Help Routines   */
@@ -394,14 +397,13 @@ int e_d_getchar()
  {
   if (c == CTRLC)
   {
-   extern struct EXT h_error;
 /*	 if (WpeIsXwin() || e_deb_mode)    */
    fcntl(fd, F_SETFL, kbdflgs & ~O_NONBLOCK);
    e_d_switch_out(0);
-   i = e_message(1, e_d_msg[0], h_error.cn->f[h_error.cn->mxedt]);
+   i = e_message(1, e_d_msg[ERR_CTRLCPRESS], WpeEditor->f[WpeEditor->mxedt]);
    if (i == 'Y')
    {
-    e_d_quit(h_error.cn->f[h_error.cn->mxedt]);
+    e_d_quit(WpeEditor->f[WpeEditor->mxedt]);
     return(D_CBREAK);
    }
    else
@@ -454,7 +456,6 @@ int e_d_quit(FENSTER *f)
   if (e_d_out)
    FREE(e_d_out);
   e_d_out = NULL;
-  e_d_out_cur = 0;
  }
  if (rfildes[1] >= 0)
   close(rfildes[1]);
@@ -489,7 +490,7 @@ int e_d_quit(FENSTER *f)
    e_d_switch_out(0);
   }
  }
- e_d_p_message(e_d_msg[2], f, 1);
+ e_d_p_message(e_d_msg[ERR_ENDDEBUG], f, 1);
  WpeMouseChangeShape(WpeEditingShape);
  e_d_delbreak(f);
  for (i = cn->mxedt; i > 0; i--)
@@ -690,7 +691,7 @@ int e_d_p_watches(FENSTER *f, int sw)
   /* If no debugger or no response, give message of no symbol in context */
   if (!e_d_swtch || (ret = e_d_line_read(wfildes[0], str1, 256, 0, 0)) == 1)
   {
-   strcpy(str1, e_d_msg[4]);
+   strcpy(str1, e_d_msg[ERR_NOSYMBOL]);
    k = 0;
   }
   else /* Debugger successfully returned a value */
@@ -724,7 +725,7 @@ int e_d_p_watches(FENSTER *f, int sw)
     {
      if (str != str1) FREE(str);
      str = str1;
-     strcpy(str, e_d_msg[4]);
+     strcpy(str, e_d_msg[ERR_NOSYMBOL]);
      k = 0;
     }
     for(k++; str[k] != '\0' && isspace(str[k]); k++);
@@ -782,44 +783,44 @@ int e_p_show_watches(FENSTER *f)
 /***  reinitialize watches from prj  ***/
 int e_d_reinit_watches(FENSTER * f,char * prj)
 {
-   int i,e,g,q,y,r;
-   char * prj2;
-   
-   for(i = f->ed->mxedt; i > 0; i--)
-   {
-      if(!strcmp(f->ed->f[i]->datnam, "Watches"))
-      {  
-         e_remove_all_watches(f->ed->f[f->ed->edt[i]]);
-	 break; 
-      }
-   }
-   g=strlen(prj);
-   prj2=MALLOC(sizeof(char)*(g+1));
-   strcpy(prj2,prj);
-   q=0;
-   y=0;
-   r=0;
-   while(q<g) 
-   {
-     e=q;
-     while(prj2[e]!=';' && e<g) e++;
-     prj2[e]='\0';
-     q=e+1;
-     r++;
-   } 
-   e_d_nwtchs=r;
-   e_d_swtchs = (char **) MALLOC(e_d_nwtchs * sizeof(char *));
-   e_d_nrwtchs =(int *) MALLOC(e_d_nwtchs * sizeof(int));   
+ int i,e,g,q,y,r;
+ char * prj2;
 
-   for(e=0,q=0;e<r;e++)
-   {
-     e_d_swtchs[e] = MALLOC(strlen(prj2+q)+1);
-     strcpy(e_d_swtchs[e], prj2+q); 
-     q+=strlen(prj2+q)+1;
-   } 
-   FREE(prj2);
-   e_d_p_watches(f, 1);   
-   return 0;
+ for(i = f->ed->mxedt; i > 0; i--)
+ {
+  if (!strcmp(f->ed->f[i]->datnam, "Watches"))
+  {  
+   e_remove_all_watches(f->ed->f[f->ed->edt[i]]);
+   break; 
+  }
+ }
+ g=strlen(prj);
+ prj2=MALLOC(sizeof(char)*(g+1));
+ strcpy(prj2,prj);
+ q=0;
+ y=0;
+ r=0;
+ while(q<g) 
+ {
+  e=q;
+  while(prj2[e]!=';' && e<g) e++;
+  prj2[e]='\0';
+  q=e+1;
+  r++;
+ } 
+ e_d_nwtchs=r;
+ e_d_swtchs = (char **) MALLOC(e_d_nwtchs * sizeof(char *));
+ e_d_nrwtchs =(int *) MALLOC(e_d_nwtchs * sizeof(int));   
+
+ for(e=0,q=0;e<r;e++)
+ {
+  e_d_swtchs[e] = MALLOC(strlen(prj2+q)+1);
+  strcpy(e_d_swtchs[e], prj2+q); 
+  q+=strlen(prj2+q)+1;
+ } 
+ FREE(prj2);
+ e_d_p_watches(f, 1);   
+ return 0;
 }
 /***************************************/
 
@@ -839,7 +840,7 @@ int e_d_p_stack(FENSTER *f, int sw)
  char str[256];
 
  if (e_d_swtch < 3)
-  return(e_error(e_d_msg[3], 0, f->fb));
+  return(e_error(e_d_msg[ERR_NOTRUNNING], 0, f->fb));
  for (i = 0; i < SVLINES; i++)
   e_d_out_str[i][0] = '\0';
  for (is = cn->mxedt; is > 0 && strcmp(cn->f[is]->datnam, "Stack"); is--)
@@ -871,7 +872,7 @@ int e_d_p_stack(FENSTER *f, int sw)
  i = j = 0;
  if (ret == 1)
  {
-  e_d_error(e_d_msg[10]);
+  e_d_error(e_d_msg[ERR_PROGEXIT]);
   return(e_d_quit(f));
  }
  while(ret != 1)
@@ -984,9 +985,9 @@ int e_make_stack(FENSTER *f)
       }
 
       if(e_deb_type == 3 && (line = e_make_line_num2(tmpstr, file)) < 0)
-	 return(e_error(e_d_msg[5], 0, f->fb));
+	 return(e_error(e_d_msg[ERR_NOSOURCE], 0, f->fb));
       else if(e_deb_type != 3 && (line = e_make_line_num(tmpstr, file)) < 0)
-	 return(e_error(e_d_msg[5], 0, f->fb));
+	 return(e_error(e_d_msg[ERR_NOSOURCE], 0, f->fb));
       if(dif > e_d_nstack)
 	 sprintf(str, "%s %d\n",
 	 e_deb_type != 3 ? "up" : "down", dif - e_d_nstack);
@@ -1017,24 +1018,24 @@ int e_brk_schirm(FENSTER *f)
  int i;
  int n;
 
-   SCHIRM *s = f->s;
-   s->brp= REALLOC(s->brp, sizeof(int));
-   s->brp[0]=0;
-   for(i=0;i<e_d_nbrpts;i++)
+ SCHIRM *s = f->s;
+ s->brp= REALLOC(s->brp, sizeof(int));
+ s->brp[0]=0;
+ for(i=0;i<e_d_nbrpts;i++)
+ {
+  if(!strcmp(f->datnam,e_d_sbrpts[i]))
+  {
+   for(n=1;n<= (s->brp[0]);n++) if(e_d_ybrpts[i]==(s->brp[n])) break;
+   if(n>s->brp[0]) 
    {
-     if(!strcmp(f->datnam,e_d_sbrpts[i]))
-     {
-       for(n=1;n<= (s->brp[0]);n++) if(e_d_ybrpts[i]==(s->brp[n])) break;
-       if(n>s->brp[0]) 
-       {
-       /****  New break, not in schirm  ****/   
-	 (s->brp[0])++;
-	 s->brp = REALLOC(s->brp, (s->brp[0]+1) * sizeof(int));
-         s->brp[s->brp[0]] = e_d_ybrpts[i]-1; 
-       }
-     }
+    /****  New break, not in schirm  ****/   
+    (s->brp[0])++;
+    s->brp = REALLOC(s->brp, (s->brp[0]+1) * sizeof(int));
+    s->brp[s->brp[0]] = e_d_ybrpts[i]-1; 
    }
-   return 0;
+  }
+ }
+ return 0;
 }
 /*****************************************/
 
@@ -1187,7 +1188,7 @@ int e_remove_breakpoints(FENSTER *f)
  for (i = 0; i < e_d_nbrpts; i++)
   FREE(e_d_sbrpts[i]);
  for (i = cn->mxedt; i >= 0; i--)
-  if(cn->f[i]->dtmd > 'Z')
+  if (DTMD_ISTEXT(cn->f[i]->dtmd))
    cn->f[i]->s->brp[0] = 0;
  e_d_nbrpts = 0;
  if (e_d_sbrpts)
@@ -1548,7 +1549,7 @@ int e_exec_deb(FENSTER *f, char *prog)
     mkfifo(npipe[3], S_IRUSR | S_IWUSR) < 0 ||
     mkfifo(npipe[4], S_IRUSR | S_IWUSR) < 0)
   {
-   e_error(e_d_msg[16], 0, f->fb);
+   e_error(e_d_msg[ERR_CANTPIPE], 0, f->fb);
    return(0);
   }
  }
@@ -1761,7 +1762,7 @@ int e_start_debug(FENSTER *f)
   }
  }
  e_sys_end();
- e_d_p_message(e_d_msg[6], f, 1);
+ e_d_p_message(e_d_msg[ERR_STARTDEBUG], f, 1);
  WpeMouseChangeShape(WpeDebuggingShape);
  if (cn->mxedt > 1)
   e_switch_window(cn->edt[cn->mxedt - 1], cn->f[cn->mxedt]);
@@ -1804,7 +1805,7 @@ int e_deb_run(FENSTER *f)
  {
   e_d_quit(f);
   if (ret == -1) {  e_show_error(0, f);  return(ret);  }
-  return(e_error(e_d_msg[7], 0, f->fb));
+  return(e_error(e_d_msg[ERR_CANTDEBUG], 0, f->fb));
  }
  for (ret = 0; isspace(e_d_tty[ret]); ret++)
   ;
@@ -1853,7 +1854,9 @@ int e_deb_run(FENSTER *f)
  {
   while((ret = e_d_line_read(wfildes[0], eing, 256, 0, 0)) == 2 ||
     !eing[0] || (!e_deb_type && prsw && ((len = (strlen(eing)-12)) < 0 ||
-    strcmp(eing + len, e_d_msg[28]))))
+    strcmp(eing + len, e_d_msg[ERR_CONTINUE]))) ||
+    (e_d_swtch < 3 && ((!e_deb_type && strncmp(e_d_msg[ERR_STARTPROG],eing, 17)) ||
+    (e_deb_type == 2 && strncmp("Running:",eing, 8)))))
   {
    if (ret == 2)
     e_d_error(eing);
@@ -1863,11 +1866,11 @@ int e_deb_run(FENSTER *f)
  }
  if (!prsw)
   e_d_p_message(eing, f, 0);
- if (e_d_swtch < 3 && ((!e_deb_type && strncmp(e_d_msg[8],eing, 17)) ||
+ if (e_d_swtch < 3 && ((!e_deb_type && strncmp(e_d_msg[ERR_STARTPROG],eing, 17)) ||
    (e_deb_type == 2 && strncmp("Running:",eing, 8))))
  {
   e_d_quit(f);
-  return(e_error(e_d_msg[9], 0, f->fb));
+  return(e_error(e_d_msg[ERR_CANTPROG], 0, f->fb));
  }
  e_d_swtch = 3;
  return(e_read_output(f));
@@ -1892,7 +1895,7 @@ int e_d_step_next(FENSTER *f, int sw)
   e_d_quit(f);
   if (ret == -1)
   {  e_show_error(0, f);  return(ret);  }
-  return(e_error(e_d_msg[7], 0, f->fb));
+  return(e_error(e_d_msg[ERR_CANTDEBUG], 0, f->fb));
  }
  if (e_d_swtch < 3)
  {
@@ -1926,7 +1929,7 @@ int e_d_goto_func(FENSTER *f, int flag)
   e_d_quit(f);
   if (ret == -1)
   {  e_show_error(0, f);  return(ret);  }
-  return(e_error(e_d_msg[7], 0, f->fb));
+  return(e_error(e_d_msg[ERR_CANTDEBUG], 0, f->fb));
  }
  if (e_d_swtch < 3)
  {
@@ -1977,37 +1980,37 @@ int e_d_fst_check(FENSTER *f)
  e_d_switch_out(0);
  for (i = 0; i < SVLINES - 1; i++)
  {
-  if ((e_deb_type != 2 && !strncmp(e_d_sp[i], e_d_msg[10], 14)) ||
-    ((e_deb_type == 2 && !strncmp(e_d_sp[i], e_d_msg[20], 14)) ||
-    (e_deb_type == 3 && !strncmp(e_d_sp[i], e_d_msg[25], strlen(e_d_msg[25])))))
+  if ((e_deb_type != 2 && !strncmp(e_d_sp[i], e_d_msg[ERR_PROGEXIT], 14)) ||
+    ((e_deb_type == 2 && !strncmp(e_d_sp[i], e_d_msg[ERR_PROGEXIT2], 14)) ||
+    (e_deb_type == 3 && !strncmp(e_d_sp[i], e_d_msg[ERR_NORMALTERM], strlen(e_d_msg[ERR_NORMALTERM])))))
   {
    e_d_error(e_d_sp[i]);		/*  Program exited   */
    e_d_quit(f);
    return(i);
   }
-  else if ((e_deb_type == 0 || e_deb_type == 1) && !strncmp(e_d_sp[i], e_d_msg[11], 18))
+  else if ((e_deb_type == 0 || e_deb_type == 1) && !strncmp(e_d_sp[i], e_d_msg[ERR_PROGTERM], 18))
   {
-   e_error(e_d_msg[11], 0, f->fb);
+   e_error(e_d_msg[ERR_PROGTERM], 0, f->fb);
    e_d_quit(f);
    return(i);
   }
-  else if (e_deb_type == 0 && !strncmp(e_d_sp[i], e_d_msg[12], 23))
+  else if (e_deb_type == 0 && !strncmp(e_d_sp[i], e_d_msg[ERR_PROGSIGNAL], 23))
   {
    e_d_pr_sig(e_d_sp[i], f);
    return(i);
   }
-  else if (e_deb_type == 3 && !strncmp(e_d_sp[i], e_d_msg[26], strlen(e_d_msg[26])))
+  else if (e_deb_type == 3 && !strncmp(e_d_sp[i], e_d_msg[ERR_SOFTTERM], strlen(e_d_msg[ERR_SOFTTERM])))
   {
    e_d_pr_sig(e_d_sp[i], f);
    return(i);
   }
-  else if (e_deb_type == 2 && (!strncmp(e_d_sp[i], e_d_msg[21], 6) ||
-    !strncmp(e_d_sp[i], e_d_msg[22], 9)))
+  else if (e_deb_type == 2 && (!strncmp(e_d_sp[i], e_d_msg[ERR_SIGNAL], 6) ||
+    !strncmp(e_d_sp[i], e_d_msg[ERR_INTERRUPT], 9)))
   {
    e_d_pr_sig(e_d_sp[i], f);
    return(i);
   }
-  else if (e_deb_type == 1 && strstr(e_d_sp[i], e_d_msg[19]))
+  else if (e_deb_type == 1 && strstr(e_d_sp[i], e_d_msg[ERR_SIGNAL]))
   {
    e_d_pr_sig(e_d_sp[i], f);
    return(i);
@@ -2017,8 +2020,8 @@ int e_d_fst_check(FENSTER *f)
    e_d_pr_sig(e_d_sp[i-1], f);
    return(i-1);
   }
-  else if (!strncmp(e_d_sp[i], e_d_msg[17], 10) ||
-    (e_deb_type == 1 && !strncmp(e_d_sp[i], e_d_msg[24], 10)))
+  else if (!strncmp(e_d_sp[i], e_d_msg[ERR_BREAKPOINT], 10) ||
+    (e_deb_type == 1 && !strncmp(e_d_sp[i], e_d_msg[ERR_BREAKPOINT2], 10)))
   {
    if (!e_deb_type)
    {
@@ -2058,7 +2061,7 @@ int e_d_fst_check(FENSTER *f)
     }
    }
   }
-  else if (e_deb_type == 2 && !strncmp(e_d_sp[i], e_d_msg[23], 10))
+  else if (e_deb_type == 2 && !strncmp(e_d_sp[i], e_d_msg[ERR_STOPPEDIN], 10))
   {
    for (j = i + 1; j < SVLINES - 1; j++)		      /*  Breakpoint   */
    {
@@ -2090,78 +2093,107 @@ int e_d_fst_check(FENSTER *f)
  return(-2);
 }
 
-int e_d_snd_check(f)
-     FENSTER *f;
+int e_d_snd_check(FENSTER *f)
 {
-   int i, j, k, ret;
-   e_d_switch_out(0);
-   for(i = SVLINES - 2; i >= 0; i--)
-   {  if(!e_deb_type && (ret = atoi(e_d_sp[i])) > 0)
-      {  for(k = 0; e_d_sp[i][k] && isdigit(e_d_sp[i][k]); k++);
-	 if(e_d_sp[i][k] != '\t') continue;
-	 if(i > 0)
-	 {  for(k = strlen(e_d_sp[i-1]); k >= 0 && e_d_sp[i-1][k] != ':'; k--);
-	    if(k >= 0 && atoi(e_d_sp[i-1]+k+1) == ret)
-	    {  i--;
-	       if(e_make_line_num(e_d_sp[i], e_d_sp[SVLINES-1]) >= 0)
-		  strcpy(e_d_file, e_d_sp[SVLINES-1]);
-	       do
-	       {  for(; k >= 0 && e_d_sp[i][k] != ')'; k--);
-		  if(k < 0)
-		  {  i--;  k = strlen(e_d_sp[i]);  }
-		  else break;
-	       }  while (i >= 0);
-	       do
-	       {  for(j = 1, k--; k >= 0 && j > 0; k--)
-		  {  if(e_d_sp[i][k] == ')') j++;
-		     else if(e_d_sp[i][k] == '(') j--;
-		  }
-		  if(k < 0)
-		  {  i--;  k = strlen(e_d_sp[i]);  }
-	       }  while (i >= 0 && j > 0);
-	       if(k == 0 && i > 0)
-	       {  i--;  k = strlen(e_d_sp[i]);  }
-	       for(k--; k >= 0 && isspace(e_d_sp[i][k]); k--);
-	       if(k < 0 && i > 0) i--;
-	    }
-	 }
-	 if(e_d_p_watches(f, 0) == -1) return(-1);
-	 e_d_goto_break(e_d_file, ret, f);
-	 return(i);
-      }
-      else if(e_deb_type == 1 && (ret = atoi(e_d_sp[i])) > 0)
-      {  for(; i > 0; i--)
-	 {  for(j = strlen(e_d_sp[i-1])-1; j >= 0
-	       && isspace(e_d_sp[i-1][j]); j--);
-	    if(j < 0) {  i--;  continue;  }
-	    for(j--; j >= 0 && !isspace(e_d_sp[i-1][j]); j--);
-	    if(j < 0) i--;
-	    for(j--; j >= 0 && isspace(e_d_sp[i-1][j]); j--);
-	    if(j < 0) i--;
-	    for(j--; j >= 0 && !isspace(e_d_sp[i-1][j]); j--);
-	    if(!strncmp(e_d_sp[i-1]+j+1, "in ", 3))
-	    {  strcpy(e_d_file, e_d_sp[i-1]+j+4);
-	       for(k = i+2; !e_d_file[0]; k++)
-		  strcpy(e_d_file, e_d_sp[i-1]+j+4);
-	       for(k = strlen(e_d_file)-1; k >= 0
-		  && isspace(e_d_file[k]); k--);
-	       e_d_file[k+1] = '\0';
-	       if(e_d_p_watches(f, 0) == -1) return(-1);
-	       e_d_goto_break(e_d_file, ret, f);
-	       return(i);
-	    }
-	 }
-	 return(-2);
-      }
-      else if(e_deb_type == 3 && i == SVLINES-2
-	 && (ret = e_make_line_num(e_d_sp[i], e_d_sp[SVLINES-1])) >= 0)
-      {  strcpy(e_d_file, e_d_sp[SVLINES-1]);
-	 if(e_d_p_watches(f, 0) == -1) return(-1);
-	 e_d_goto_break(e_d_file, ret, f);
-	 return(i);
+ int i, j, k, ret;
+
+ e_d_switch_out(0);
+ for (i = SVLINES - 2; i >= 0; i--)
+ {
+  if (!e_deb_type && (ret = atoi(e_d_sp[i])) > 0)
+  {
+   for (k = 0; e_d_sp[i][k] && isdigit(e_d_sp[i][k]); k++)
+    ;
+   if (e_d_sp[i][k] != '\t')
+    continue;
+   if (i > 0)
+   {
+    for (k = strlen(e_d_sp[i-1]); k >= 0 && e_d_sp[i-1][k] != ':'; k--)
+     ;
+    if (k >= 0 && atoi(e_d_sp[i-1]+k+1) == ret)
+    {
+     i--;
+     if (e_make_line_num(e_d_sp[i], e_d_sp[SVLINES-1]) >= 0)
+      strcpy(e_d_file, e_d_sp[SVLINES-1]);
+     do
+     {
+      for (; k >= 0 && e_d_sp[i][k] != ')'; k--)
+       ;
+      if (k < 0)
+      {  i--;  k = strlen(e_d_sp[i]);  }
+      else
+       break;
+     }  while (i >= 0);
+     do
+     {
+      for (j = 1, k--; k >= 0 && j > 0; k--)
+      {
+       if (e_d_sp[i][k] == ')')
+        j++;
+       else if (e_d_sp[i][k] == '(')
+        j--;
       }
+      if (k < 0)
+      {  i--;  k = strlen(e_d_sp[i]);  }
+     } while (i >= 0 && j > 0);
+     if (k == 0 && i > 0)
+     {  i--;  k = strlen(e_d_sp[i]);  }
+     for (k--; k >= 0 && isspace(e_d_sp[i][k]); k--)
+      ;
+     if (k < 0 && i > 0)
+      i--;
+    }
+   }
+   if (e_d_p_watches(f, 0) == -1)
+    return(-1);
+   e_d_goto_break(e_d_file, ret, f);
+   return(i);
+  }
+  else if (e_deb_type == 1 && (ret = atoi(e_d_sp[i])) > 0)
+  {
+   for (; i > 0; i--)
+   {
+    for (j = strlen(e_d_sp[i-1])-1; j >= 0 && isspace(e_d_sp[i-1][j]); j--)
+     ;
+    if (j < 0)
+    {  i--;  continue;  }
+    for (j--; j >= 0 && !isspace(e_d_sp[i-1][j]); j--)
+     ;
+    if (j < 0)
+     i--;
+    for (j--; j >= 0 && isspace(e_d_sp[i-1][j]); j--)
+     ;
+    if (j < 0)
+     i--;
+    for (j--; j >= 0 && !isspace(e_d_sp[i-1][j]); j--)
+     ;
+    if (!strncmp(e_d_sp[i-1]+j+1, "in ", 3))
+    {
+     strcpy(e_d_file, e_d_sp[i-1]+j+4);
+     for (k = i+2; !e_d_file[0]; k++)
+      strcpy(e_d_file, e_d_sp[i-1]+j+4);
+     for (k = strlen(e_d_file)-1; k >= 0 && isspace(e_d_file[k]); k--)
+      ;
+     e_d_file[k+1] = '\0';
+     if (e_d_p_watches(f, 0) == -1)
+      return(-1);
+     e_d_goto_break(e_d_file, ret, f);
+     return(i);
+    }
    }
    return(-2);
+  }
+  else if (e_deb_type == 3 && i == SVLINES-2 &&
+    (ret = e_make_line_num(e_d_sp[i], e_d_sp[SVLINES-1])) >= 0)
+  {
+   strcpy(e_d_file, e_d_sp[SVLINES-1]);
+   if (e_d_p_watches(f, 0) == -1)
+    return(-1);
+   e_d_goto_break(e_d_file, ret, f);
+   return(i);
+  }
+ }
+ return(-2);
 }
 
 int e_d_trd_check(FENSTER *f)
@@ -2172,7 +2204,7 @@ int e_d_trd_check(FENSTER *f)
  str[0] = '\0';
  e_d_switch_out(0);
  if ((ret = e_d_pr_sig(str, f)) == -1) return(-1);
- else if (ret == -2) e_d_error(e_d_msg[5]);
+ else if (ret == -2) e_d_error(e_d_msg[ERR_NOSOURCE]);
  return(0);
 }
 
@@ -2206,133 +2238,151 @@ int e_read_output(FENSTER *f)
  if (i < 0)
  {
   e_d_switch_out(0);
-  i = e_message(1, e_d_msg[13], f);
+  i = e_message(1, e_d_msg[ERR_UNKNOWNBRK], f);
   if (i == 'Y') return(e_d_quit(f));
  }
  return(0);
 }
 
-int e_d_pr_sig(str, f)
-     char *str;
-     FENSTER *f;
+int e_d_pr_sig(char *str, FENSTER *f)
 {
-   int i, line = -1, ret = 0;
-   char file[128], str2[256];
-   if(str && str[0]) e_d_error(str);
-   for(i = 0; i < SVLINES; i++) e_d_out_str[i][0] = '\0';
-   if(e_deb_type != 1 && e_deb_type != 3)
-   {  write(rfildes[1], "where\n", 6);
-      for(i = 0; ((ret = e_d_line_read(wfildes[0], str, 256, 0, 1)) == 0
-	 && (line = e_make_line_num(str, file)) < 0) || ret == 2; i++)
-      {  if(!strncmp(str, e_d_msg[29], 9))
-	 {  e_error(e_d_msg[10], 0, f->fb);
-	    while(ret == 0 || ret == 2)
-	       if((ret = e_d_line_read(wfildes[0], str2, 256, 0, 0)) == 2)
-	       e_d_error(str2);
-	    e_d_quit(f);
-	    return(0);
-	 }
-	 else if(ret == 2) e_d_error(str);
-      }
-   }
-   else
-   {  write(rfildes[1], "t\n", 2);
-      for(i = 0; ((ret = e_d_line_read(wfildes[0], str, 256, 0, 1)) == 0
-	 && (line = e_make_line_num2(str, file)) < 0) || ret == 2; i++)
-      {  if(!strncmp(str, e_d_msg[30], 10))
-	 {  e_error(e_d_msg[10], 0, f->fb);
-	    while(ret == 0 || ret == 2)
-	       if((ret = e_d_line_read(wfildes[0], str2, 256, 0, 0)) == 2)
-	       e_d_error(str2);
-	    e_d_quit(f);
-	    return(0);
-	 }
-	 else if(ret == 2) e_d_error(str);
-      }
-   }
-   if(ret == 1 && i == 0)
-   {  e_d_error(e_d_msg[10]);
-      return(e_d_quit(f));
-   }
-   while(ret == 0 || ret == 2) if((ret = e_d_line_read(wfildes[0], str2, 256, 0, 0)) == 2)
+ int i, line = -1, ret = 0;
+ char file[128], str2[256];
+
+ if (str && str[0])
+  e_d_error(str);
+ for (i = 0; i < SVLINES; i++)
+  e_d_out_str[i][0] = '\0';
+ if (e_deb_type != 1 && e_deb_type != 3)
+ {
+  write(rfildes[1], "where\n", 6);
+  for (i = 0; ((ret = e_d_line_read(wfildes[0], str, 256, 0, 1)) == 0 &&
+    (line = e_make_line_num(str, file)) < 0) || ret == 2; i++)
+  {
+   if (!strncmp(str, e_d_msg[ERR_NOSTACK], 9))
+   {
+    e_error(e_d_msg[ERR_PROGEXIT], 0, f->fb);
+    while (ret == 0 || ret == 2)
+     if ((ret = e_d_line_read(wfildes[0], str2, 256, 0, 0)) == 2)
       e_d_error(str2);
-   if(ret == -1) return(ret);
-   if(line >= 0)
-   {  strcpy(e_d_file, file);
-      e_d_goto_break(file, line, f);
-      return(0);
+    e_d_quit(f);
+    return(0);
    }
-   else return(-2);
+   else if (ret == 2)
+    e_d_error(str);
+  }
+ }
+ else
+ {
+  write(rfildes[1], "t\n", 2);
+  for (i = 0; ((ret = e_d_line_read(wfildes[0], str, 256, 0, 1)) == 0 &&
+    (line = e_make_line_num2(str, file)) < 0) || ret == 2; i++)
+  {
+   if (!strncmp(str, e_d_msg[ERR_NOPROCESS], 10))
+   {
+    e_error(e_d_msg[ERR_PROGEXIT], 0, f->fb);
+    while (ret == 0 || ret == 2)
+     if ((ret = e_d_line_read(wfildes[0], str2, 256, 0, 0)) == 2)
+      e_d_error(str2);
+    e_d_quit(f);
+    return(0);
+   }
+   else if (ret == 2)
+    e_d_error(str);
+  }
+ }
+ if (ret == 1 && i == 0)
+ {
+  e_d_error(e_d_msg[ERR_PROGEXIT]);
+  return(e_d_quit(f));
+ }
+ while (ret == 0 || ret == 2)
+  if ((ret = e_d_line_read(wfildes[0], str2, 256, 0, 0)) == 2)
+   e_d_error(str2);
+ if (ret == -1)
+  return(ret);
+ if (line >= 0)
+ {
+  strcpy(e_d_file, file);
+  e_d_goto_break(file, line, f);
+  return(0);
+ }
+ else
+  return(-2);
 }
 
-int e_make_line_num(str, file)
-     char *str;
-     char *file;
+int e_make_line_num(char *str, char *file)
 {
-   char *sp;
-   int i, n, num;
-/*
-      if(e_deb_type == 0)
-      {  if(!(sp = strstr(str, " at "))) return(-1);
-      for(sp += 4; isspace(*sp); sp++);
-      for(n = 0; sp[n] != ':' && sp[n] != '\0'; n++);
-      if(sp[n] == '\0') return(-1);
-      for(; n >= 0 && !isspace(sp[n]); n--);
-      for(n++, i = 0; (file[i] = sp[n+i]) != ':' && file[i] != '\0'; i++);
-      file[i] = '\0';
-      return(atoi(sp+n+i+1));
-      }
-*/
-   if(e_deb_type == 0)
-   {  for(n = strlen(str); n >= 0 && str[n] != ':'; n--);
-      if(n < 0) return(-1);
-      for(i = n-1; i >= 0 && !isspace(str[i]); i--);
-      for(n = i+1; str[n] != ':'; n++) file[n-i-1] = str[n];
-      file[n-i-1] = '\0';
-      return(atoi(str+n+1));
-   }
-   else if(e_deb_type == 2)
-   {  if(!(sp = strstr(str, " line "))) return(-1);
-/*
-	 {	if(!(sp = strstr(str, " line ")))
-	 {   for(i = 0; str[i] && isspace(str[i]); i++);
-	 if(!str[i] || !(num = atoi(str+i))) return(-1);
-	 strcpy(file, e_d_file);
-	 return(num);
-	 }
-*/
-      if(!(num = atoi(sp+6))) return(-1);
-      for(i = 6;  sp[i] != '\"'; i++);
-      sp += (i+1);
-      for(i = 0; (file[i] = sp[i]) != '\"' && file[i] != '\0'; i++);
-      if(file[i] == '\0') return(-1);
-      file[i] = '\0';
-      return(num);
-   }
-   else if(e_deb_type == 3)
-   {  for(sp = str, i = 0; (file[i] = sp[i]) && sp[i] != ':'; i++);
-      if(!sp[i]) return(-1);
-      file[i] = '\0';
-      for(i++; sp[i] && sp[i] != ':'; i++);
-      if(!sp[i]) return(-1);
-      for(i++; sp[i] && isspace(sp[i]); i++);
-      if(!isdigit(sp[i])) return(-1);
-      sp += i;
-      return(atoi(sp));
-   }
-   else
-   {  for(i = 0; str[i] != '\0' && str[i] != ':'; i++);
-      if((!str[i]) || (num = atoi(str+i+1)) < 0) return(-1);
-      write(rfildes[1], "e\n", 2);
-      while((i = e_d_line_read(wfildes[0], str, 256, 0, 0)) ==  2)
-	 e_d_error(str);
-      if(i < 0) return(-1);
-      for(i = 0; str[i] != '\0' && str[i] != '\"'; i++);
-      for(sp = str + i + 1, i = 0; (file[i] = sp[i]) && file[i] != '\"'; i++);
-      file[i] = '\0';
-      if(e_d_dum_read() == -1) return(-1);
-      return(num);
-   }
+ char *sp;
+ int i, n, num;
+
+ if (e_deb_type == 0)
+ {
+  for (n = strlen(str); n >= 0 && str[n] != ':'; n--)
+   ;
+  if (n < 0)
+   return(-1);
+  for (i = n-1; i >= 0 && !isspace(str[i]); i--)
+   ;
+  for (n = i+1; str[n] != ':'; n++)
+   file[n-i-1] = str[n];
+  file[n-i-1] = '\0';
+  return(atoi(str+n+1));
+ }
+ else if (e_deb_type == 2)
+ {
+  if (!(sp = strstr(str, " line ")))
+   return(-1);
+  if (!(num = atoi(sp+6)))
+   return(-1);
+  for (i = 6;  sp[i] != '\"'; i++)
+   ;
+  sp += (i+1);
+  for (i = 0; (file[i] = sp[i]) != '\"' && file[i] != '\0'; i++)
+   ;
+  if (file[i] == '\0')
+   return(-1);
+  file[i] = '\0';
+  return(num);
+ }
+ else if (e_deb_type == 3)
+ {
+  for (sp = str, i = 0; (file[i] = sp[i]) && sp[i] != ':'; i++)
+   ;
+  if (!sp[i])
+   return(-1);
+  file[i] = '\0';
+  for (i++; sp[i] && sp[i] != ':'; i++)
+   ;
+  if (!sp[i])
+   return(-1);
+  for (i++; sp[i] && isspace(sp[i]); i++)
+   ;
+  if (!isdigit(sp[i]))
+   return(-1);
+  sp += i;
+  return(atoi(sp));
+ }
+ else
+ {
+  for (i = 0; str[i] != '\0' && str[i] != ':'; i++)
+   ;
+  if ((!str[i]) || (num = atoi(str+i+1)) < 0)
+   return(-1);
+  write(rfildes[1], "e\n", 2);
+  while ((i = e_d_line_read(wfildes[0], str, 256, 0, 0)) ==  2)
+   e_d_error(str);
+  if (i < 0)
+   return(-1);
+  for (i = 0; str[i] != '\0' && str[i] != '\"'; i++)
+   ;
+  for (sp = str + i + 1, i = 0; (file[i] = sp[i]) && file[i] != '\"'; i++)
+   ;
+  file[i] = '\0';
+  if (e_d_dum_read() == -1)
+   return(-1);
+  return(num);
+ }
 }
 
 int e_make_line_num2(char *str, char *file)
@@ -2352,55 +2402,58 @@ int e_make_line_num2(char *str, char *file)
  return(atoi(sp+i));
 }
 
-int e_d_goto_break(file, line, f)
-     char *file;
-     int line;
-     FENSTER *f;
+int e_d_goto_break(char *file, int line, FENSTER *f)
 {
-   ECNT *cn = f->ed;
-   BUFFER *b;
-   SCHIRM *s;
-   FENSTER ftmp;
-   int i;
-   char str[120];
+ ECNT *cn = f->ed;
+ BUFFER *b;
+ SCHIRM *s;
+ FENSTER ftmp;
+ int i;
+ char str[120];
+
 /*   if(schirm != e_d_save_schirm) e_d_switch_out(0);  */
-   e_d_switch_out(0);
-   ftmp.ed = cn;
-   ftmp.fb = f->fb;
-   WpeFilenameToPathFile(file, &ftmp.dirct, &ftmp.datnam);
-   for(i = 0; i < SVLINES; i++) e_d_out_str[i][0] = '\0';
-   for(i = cn->mxedt; i > 0; i--)
-      if(!strcmp(cn->f[i]->datnam, ftmp.datnam) &&
-      !strcmp(cn->f[i]->dirct, ftmp.dirct))
-   {  /*  for(j = 0; j <= cn->mxedt; j++)
+ e_d_switch_out(0);
+ ftmp.ed = cn;
+ ftmp.fb = f->fb;
+ WpeFilenameToPathFile(file, &ftmp.dirct, &ftmp.datnam);
+ for (i = 0; i < SVLINES; i++)
+  e_d_out_str[i][0] = '\0';
+ for (i = cn->mxedt; i > 0; i--)
+  if (!strcmp(cn->f[i]->datnam, ftmp.datnam) &&
+    !strcmp(cn->f[i]->dirct, ftmp.dirct))
+  {
+   /*  for(j = 0; j <= cn->mxedt; j++)
 	 if(!strcmp(cn->f[j]->datnam, "Stack"))
 	 {  if(cn->f[i]->e.x > 2*MAXSCOL/3-1) cn->f[i]->e.x = 2*MAXSCOL/3-1;
 	 break;
 	 }  */
-      e_switch_window(cn->edt[i], cn->f[cn->mxedt]);
-      break;
-   }
-   f = cn->f[cn->mxedt];
-   FREE(ftmp.dirct);
-   FREE(ftmp.datnam);
-   if(i <= 0)
-   {  if(access(file, 0))
-      {  sprintf(str, e_d_msg[14], file);
-	 return(e_error(str, 0, f->fb));
-      }
-      if(e_edit(cn, file)) return(WPE_ESC);
-      b = cn->f[cn->mxedt]->b;
-      s = cn->f[cn->mxedt]->s;
-   }
-   f = cn->f[cn->mxedt];
-   b = cn->f[cn->mxedt]->b;
-   s = cn->f[cn->mxedt]->s;
-   s->da.y = b->b.y = line - 1;
-   s->da.x = b->b.x = 0;
-   s->de.x = MAXSCOL;
-   e_schirm(f, 1);
-   e_cursor(f, 1);
-   return(0);
+   e_switch_window(cn->edt[i], cn->f[cn->mxedt]);
+   break;
+  }
+ f = cn->f[cn->mxedt];
+ FREE(ftmp.dirct);
+ FREE(ftmp.datnam);
+ if (i <= 0)
+ {
+  if (access(file, 0))
+  {
+   sprintf(str, e_d_msg[ERR_CANTFILE], file);
+   return(e_error(str, 0, f->fb));
+  }
+  if (e_edit(cn, file))
+   return(WPE_ESC);
+  b = cn->f[cn->mxedt]->b;
+  s = cn->f[cn->mxedt]->s;
+ }
+ f = cn->f[cn->mxedt];
+ b = cn->f[cn->mxedt]->b;
+ s = cn->f[cn->mxedt]->s;
+ s->da.y = b->b.y = line - 1;
+ s->da.x = b->b.x = 0;
+ s->de.x = MAXSCOL;
+ e_schirm(f, 1);
+ e_cursor(f, 1);
+ return(0);
 }
 
 int e_d_delbreak(FENSTER *f)
@@ -2409,7 +2462,7 @@ int e_d_delbreak(FENSTER *f)
  int i;
 
  for (i = cn->mxedt; i >= 0; i--)
-  if (cn->f[i]->dtmd > 'Z')
+  if (DTMD_ISTEXT(cn->f[i]->dtmd))
    cn->f[i]->s->da.y = -1;
  e_rep_win_tree(cn);
  e_refresh();
@@ -2418,13 +2471,12 @@ int e_d_delbreak(FENSTER *f)
 
 int e_d_error(char *s)
 {
- extern struct EXT h_error;
  int len;
 
  e_d_switch_out(0);
  if (s[(len = strlen(s) - 1)] == '\n')
   s[len] = '\0';
- return(e_error(s, 0, h_error.cn->fb));
+ return(e_error(s, 0, WpeEditor->fb));
 }
 
 int e_d_putchar(int c)
