@@ -8,10 +8,8 @@
 #include "edit.h"
 #include "makro.h"
 
-#ifdef UNIX
-#include<sys/types.h> /*  included fuer digital station  */
+#include<sys/types.h> /*  included for digital station  */
 #include<sys/stat.h>
-#endif
 
 int e_undo_sw = 0, e_redo_sw = 0;
 
@@ -50,7 +48,7 @@ int e_edit(ECNT *cn, char *filename)
   else
   {
    for (i = cn->mxedt; i > 0 &&
-     (cn->f[i]->dtmd != 'D' || cn->f[i]->ins != 4); i--)
+     (cn->f[i]->dtmd != DTMD_DATA || cn->f[i]->ins != 4); i--)
     ;
    if (i > 0)
    {
@@ -110,7 +108,7 @@ int e_edit(ECNT *cn, char *filename)
    
  f->fb = cn->fb;
  cn->f[cn->mxedt] = f;
-   
+
  if ((f->b = (BUFFER *)MALLOC(sizeof(BUFFER))) == NULL)
   e_error(e_msg[ERR_LOWMEM], 1, f->fb);
  if ((f->s = (SCHIRM*)MALLOC(sizeof(SCHIRM))) == NULL)
@@ -119,7 +117,7 @@ int e_edit(ECNT *cn, char *filename)
   e_error(e_msg[ERR_LOWMEM], 1, f->fb);
 #ifdef PROG
  for (i = cn->mxedt-1;
-   i > 0 && (!strcmp(cn->f[i]->datnam, "Messages") || cn->f[i]->dtmd <= 'Z' ||
+   i > 0 && (!strcmp(cn->f[i]->datnam, "Messages") || !DTMD_ISTEXT(cn->f[i]->dtmd) ||
      !strcmp(cn->f[i]->datnam, "Watches") ||
      !strcmp(cn->f[i]->datnam, "Stack")); i--)
   ;
@@ -127,7 +125,7 @@ int e_edit(ECNT *cn, char *filename)
   if (!strcmp(cn->f[j]->datnam, "Stack"))
    st = 1;
 #else
- for(i = cn->mxedt-1; i > 0 && cn->f[i]->dtmd <= 'Z'; i--)
+ for(i = cn->mxedt-1; i > 0 && !DTMD_ISTEXT(cn->f[i]->dtmd); i--)
   ;
 #endif
 #ifdef PROG
@@ -195,6 +193,7 @@ int e_edit(ECNT *cn, char *filename)
  f->b->cn = cn;
  f->b->ud = NULL;
  f->b->rd = NULL;
+ f->fd.dirct = NULL;
  if (WpeIsProg())
   e_add_synt_tl(filename, f);
  else
@@ -254,7 +253,7 @@ int e_edit(ECNT *cn, char *filename)
  if (strcmp(filename, "Help") == 0)
  {
   complete_fname = e_mkfilename(LIBRARY_DIR, HELP_FILE);
-  f->dtmd = 'h';
+  f->dtmd = DTMD_HELP;
   f->ins = 8;
   f->hlp_str = e_hlp_str[25];
   f->nblst = 7;
@@ -309,11 +308,11 @@ int e_edit(ECNT *cn, char *filename)
 
  if (fp != NULL && ftype != 1)
  {
-  e_readin(0, 0, fp, f->b, f->dtmd);
+  e_readin(0, 0, fp, f->b, &f->dtmd);
   if (fclose(fp) != 0)
    e_error(e_msg[ERR_FCLOSE], 0, cn->fb);
-  if (cn->dtmd == 'm' || cn->dtmd == 'h')
-   cn->dtmd = 'n';
+  if (cn->dtmd == DTMD_HELP)
+   cn->dtmd = DTMD_NORMAL;
 #ifdef PROG
   if (WpeIsProg())
   {
@@ -388,103 +387,102 @@ int e_edit(ECNT *cn, char *filename)
 /*   keyboard output routine */
 int e_eingabe(ECNT *e)
 {
-   BUFFER *b = e->f[e->mxedt]->b;
-   SCHIRM *s = e->f[e->mxedt]->s;
-   FENSTER *f = e->f[e->mxedt];
-   int ret, c = 0;
-#ifdef UNIX
-   unsigned char cc;
-#endif
-   fk_cursor(1);
-   while (c != WPE_ESC)
-   {  if(e->mxedt < 1) c = WpeHandleMainmenu(-1, f);
-      else if(f->dtmd <= 'Z') return(0);
-      else
-      {
-	 if(f->save > f->ed->maxchg) e_autosave(f);
+ BUFFER *b = e->f[e->mxedt]->b;
+ SCHIRM *s = e->f[e->mxedt]->s;
+ FENSTER *f = e->f[e->mxedt];
+ int ret, c = 0;
+ unsigned char cc;
+
+ fk_cursor(1);
+ while (c != WPE_ESC)
+ {
+  if (e->mxedt < 1) c = WpeHandleMainmenu(-1, f);
+  else if (!DTMD_ISTEXT(f->dtmd)) return(0);
+  else
+  {
+   if (f->save > f->ed->maxchg) e_autosave(f);
 #if  MOUSE
-#ifdef UNIX
-	 if( (c = e_getch()) < 0) cc = c = e_edt_mouse(c, f);
-	 else cc = c;
+   if ((c = e_getch()) < 0) cc = c = e_edt_mouse(c, f);
+   else cc = c;
 #else
-	 else if( (c = e_getch()) < 0) c = e_edt_mouse(c, f);
+   cc = c = e_getch();
 #endif
-#else
-#ifdef UNIX
-	 else cc = c = e_getch();
-#else
-	 else c = e_getch();
-#endif
-#endif
-      }
-      if((c > 31 || (c == WPE_TAB && !(f->flg & 1))
-		|| (f->ins > 1 && f->ins != 8)) && c < 255)
-      {  if(f->ins == 8) continue;
-	 if (f->ins == 0 || f->ins == 2)  e_put_char(c, b, s);
-#ifdef UNIX
-	 else  e_ins_nchar(b, s, &cc, b->b.x, b->b.y, 1);
-#else
-	 else  e_ins_nchar(b, s, (unsigned char *)&c,
-                                                        b->b.x, b->b.y, 1);
-#endif
-	 e_schirm(f, 1);
-      }
-      else if(c == WPE_DC)
-      {  if(f->ins == 8)
-	 {  if(f->dtmd == 'h') e_help_last(f);
-	    continue;
-	 }
-	 if( b->b.y > 0 || b->b.x > 0)
-	 {  if(b->bf[b->b.y].len == 0)
-	    {  e_del_line(b->b.y, b, s);
-	       b->b.y--;
-	       b->b.x = b->bf[b->b.y].len;
-	       if(*(b->bf[b->b.y].s+b->b.x) == '\0') b->b.x--;
-	    }
-	    else
-	    {  if(b->b.x > 0) b->b.x--;
-	       else
-	       {  b->b.y--;
-		  b->b.x = b->bf[b->b.y].len;
-		  if(*(b->bf[b->b.y].s+b->b.x) == '\0') b->b.x--;
-	       }
-	       if(f->flg & 1) e_del_a_ind(b,  s);
-	       else e_del_nchar(b, s, b->b.x, b->b.y, 1);
-	    }
-	    e_schirm(f, 1);
-	 }
-      }
-      else if(c == ENTF || c == 4)
-      {  if(f->ins == 8) {
+  }
+  if ((c > 31 || (c == WPE_TAB && !(f->flg & 1)) ||
+    (f->ins > 1 && f->ins != 8)) && c < 255)
+  {
+   if (f->ins == 8) continue;
+   if (f->ins == 0 || f->ins == 2)  e_put_char(c, b, s);
+   else e_ins_nchar(b, s, &cc, b->b.x, b->b.y, 1);
+   e_schirm(f, 1);
+  }
+  else if (c == WPE_DC)
+  {
+   if (f->ins == 8)
+   {
+    if (f->dtmd == DTMD_HELP) e_help_last(f);
+    continue;
+   }
+   if (b->b.y > 0 || b->b.x > 0)
+   {
+    if (b->bf[b->b.y].len == 0)
+    {
+     e_del_line(b->b.y, b, s);
+     b->b.y--;
+     b->b.x = b->bf[b->b.y].len;
+     if (*(b->bf[b->b.y].s+b->b.x) == '\0') b->b.x--;
+    }
+    else
+    {
+     if (b->b.x > 0) b->b.x--;
+     else
+     {
+      b->b.y--;
+      b->b.x = b->bf[b->b.y].len;
+      if (*(b->bf[b->b.y].s+b->b.x) == '\0') b->b.x--;
+     }
+     if (f->flg & 1) e_del_a_ind(b,  s);
+     else e_del_nchar(b, s, b->b.x, b->b.y, 1);
+    }
+    e_schirm(f, 1);
+   }
+  }
+  else if (c == ENTF || c == 4)
+  {
+   if (f->ins == 8)
+   {
 #ifdef DEBUGGER
-	    if (WpeIsProg()) e_d_is_watch(c, f);
+    if (WpeIsProg()) e_d_is_watch(c, f);
 #endif
-	    continue;  }
-	 if( *(b->bf[b->b.y].s + b->b.x) != '\0'
-		  && (b->b.y < b->mxlines-1
-			|| *(b->bf[b->b.y].s + b->b.x) != WPE_WR))
-	 {  e_del_nchar(b, s, b->b.x, b->b.y, 1);
-	    e_schirm(f, 1);
-	 }
-      }
-      else if(c == WPE_CR)
-      {
+    continue;
+   }
+   if (*(b->bf[b->b.y].s + b->b.x) != '\0' &&
+     (b->b.y < b->mxlines-1 || *(b->bf[b->b.y].s + b->b.x) != WPE_WR))
+   {
+    e_del_nchar(b, s, b->b.x, b->b.y, 1);
+    e_schirm(f, 1);
+   }
+  }
+  else if (c == WPE_CR)
+  {
 #ifdef PROG
-	 if(f->ins == 8)
-	 {  if(f->dtmd == 'h') {  e_help_ret(f);  goto weiter;  }
-	    if (WpeIsProg())  {  e_d_car_ret(f);  goto weiter;  }
-	    else continue;
-	 }
+   if (f->ins == 8)
+   {
+    if (f->dtmd == DTMD_HELP) {  e_help_ret(f);  goto weiter;  }
+    if (WpeIsProg())  {  e_d_car_ret(f);  goto weiter;  }
+    else continue;
+   }
 #else
-	 if(f->ins == 8) continue;
+   if (f->ins == 8) continue;
 #endif
-	 e_car_ret(b, s);
-	 e_schirm(f, 1);
-      }
-      else if(c == WPE_TAB)
-      {  e_tab_a_ind(b, s);
-	 e_schirm(f, 1);
-      }
+   e_car_ret(b, s);
+   e_schirm(f, 1);
+  }
+  else if (c == WPE_TAB)
+  {
+   e_tab_a_ind(b, s);
+   e_schirm(f, 1);
+  }
 /*
           else if(c == WPE_TAB)
           {    if(f->ins == 8) continue;
@@ -497,26 +495,28 @@ int e_eingabe(ECNT *e)
                e_schirm(b, s, f, 1);
           }
 */
-      else
-      {  ret = e_tst_cur(c, e); /*up/down arrows go this way*/
-	 if(ret != 0) ret = e_tst_fkt(c, e);
-      }
-      weiter:
-      f = e->f[e->mxedt];
-      if(e->mxedt > 0 && f->dtmd > 'Z')
-      {  b = f->b;
-         s = f->s;
-         s->ks.x = b->b.x;  s->ks.y = b->b.y;
-         e_cursor(f, 1);
-	 if((c & 511) != CUP && (c & 511) != CDO) b->clsv = b->cl;
-	 e_zlsplt(f);
-      }
-   }
-   return(c);
+  else
+  {
+   ret = e_tst_cur(c, e); /*up/down arrows go this way*/
+   if (ret != 0) ret = e_tst_fkt(c, e);
+  }
+weiter:
+  f = e->f[e->mxedt];
+  if (e->mxedt > 0 && DTMD_ISTEXT(f->dtmd))
+  {
+   b = f->b;
+   s = f->s;
+   s->ks.x = b->b.x;  s->ks.y = b->b.y;
+   e_cursor(f, 1);
+   if ((c & 511) != CUP && (c & 511) != CDO) b->clsv = b->cl;
+   e_zlsplt(f);
+  }
+ }
+ return(c);
 }
 
-/*              Interpretation der Cursor Tasten
-                     ( Primitiv Editor )                    */
+/*              Interpretation of cursor keys
+                     ( Primitive editor )                    */
 int e_tst_cur(int c, ECNT *e)
 {
    FENSTER *f = e->f[e->mxedt];
@@ -672,12 +672,12 @@ int e_tst_cur(int c, ECNT *e)
 	 break;
       case AF7:
       case AltV:
-	 if(f->dtmd != 'h' || f->ins != 8) return(c);
+	 if(f->dtmd != DTMD_HELP || f->ins != 8) return(c);
          e_help_next(f, 0);
 	 break;
       case AF8:
       case AltT:
-	 if(f->dtmd != 'h' || f->ins != 8) return(c);
+	 if(f->dtmd != DTMD_HELP || f->ins != 8) return(c);
          e_help_next(f, 1);
 	 break;
       default:
@@ -735,9 +735,7 @@ int e_tst_fkt(int c, ECNT *e)
       case AltG:
 	 e_goto_line(f);
 	 break;
-#ifndef UNIX
       case CF10:
-#endif
       case CtrlW:
 	 e_show_clipboard(f);
 	 break;
@@ -765,7 +763,7 @@ int e_tst_fkt(int c, ECNT *e)
       case CtrlV:
 	 e_edt_einf(f);
 	 break;
-#if !defined(NO_XWINDOWS) && !defined(DJGPP)
+#if !defined(NO_XWINDOWS)
       case AltEin:
 	 e_copy_X_buffer(f);
 	 break;
@@ -1054,7 +1052,7 @@ int e_tst_dfkt(FENSTER *f, int c)
   case HELP:
    e_help_loc(f, 0);
    break;
-#if defined(PROG) && !defined(DJGPP)
+#if defined(PROG)
   case CF1:
    if (WpeIsProg())
     e_topic_search(f);
@@ -1173,7 +1171,7 @@ int e_chr_sp(int x, BUFFER *b, FENSTER *f)
   }
   else if (*(b->bf[b->b.y].s + i) < ' ')
    j++;
-  if (f->dtmd == 'h')
+  if (f->dtmd == DTMD_HELP)
   {
    if (b->bf[b->b.y].s[i] == HBG || b->bf[b->b.y].s[i] == HFB ||
      b->bf[b->b.y].s[i] == HED || b->bf[b->b.y].s[i] == HHD ||
@@ -1481,91 +1479,87 @@ int e_car_ret(BUFFER *b, SCHIRM *s)
 /*   cursor placement */
 void e_cursor(FENSTER *f, int sw)
 {
-   BUFFER *b = f->b;
-   SCHIRM *s = f->s;
-   static int iold = 0, jold = 0;
-   int i, j;
-   if(f->dtmd <= 'Z') return;
-   if(b->b.y < 0) b->b.y = 0;
-   if(b->b.y > b->mxlines-1) b->b.y = b->mxlines-1;
-   if(b->b.x < 0) b->b.x = 0;
-   if(b->b.x > b->bf[b->b.y].len) b->b.x = b->bf[b->b.y].len;
-   for(i = j = 0; i < b->b.x; i++)
-   {  if(*(b->bf[b->b.y].s + i) == WPE_TAB)
-      j += (f->ed->tabn - ((j + i) % f->ed->tabn) - 1);
-#ifdef UNIX
-      else if(!WpeIsXwin()
-		&& ((unsigned char) *(b->bf[b->b.y].s + i)) > 126)
-      {  j++;
-	 if(((unsigned char) *(b->bf[b->b.y].s + i)) < 128 + ' ') j++;
-      }
-      else if(*(b->bf[b->b.y].s + i) < ' ') j++;
-      if(f->dtmd == 'h')
-      {  if(b->bf[b->b.y].s[i] == HBG || b->bf[b->b.y].s[i] == HED
-		|| b->bf[b->b.y].s[i] == HHD || b->bf[b->b.y].s[i] == HFE
-		|| b->bf[b->b.y].s[i] == HFB || b->bf[b->b.y].s[i] == HBB
-                || b->bf[b->b.y].s[i] == HNF)
-	 j -= 2;
-      }
-#endif
-   }
-   if( b->b.y - s->c.y < 0 || b->b.y - s->c.y >= NUM_LINES_ON_SCREEN - 1
-         || s->c.y < 0 || s->c.y >= b->mxlines
-         || b->b.x + j - s->c.x < 0
-	 || b->b.x + j- s->c.x >= NUM_COLS_ON_SCREEN - 1 )
-   {
-#if defined(UNIX) && !defined(DJGPP)
+ BUFFER *b = f->b;
+ SCHIRM *s = f->s;
+ static int iold = 0, jold = 0;
+ int i, j;
+
+ if (!DTMD_ISTEXT(f->dtmd)) return;
+ if (b->b.y < 0) b->b.y = 0;
+ if (b->b.y > b->mxlines-1) b->b.y = b->mxlines-1;
+ if (b->b.x < 0) b->b.x = 0;
+ if (b->b.x > b->bf[b->b.y].len) b->b.x = b->bf[b->b.y].len;
+ for (i = j = 0; i < b->b.x; i++)
+ {
+  if (*(b->bf[b->b.y].s + i) == WPE_TAB)
+   j += (f->ed->tabn - ((j + i) % f->ed->tabn) - 1);
+  else if (!WpeIsXwin() && ((unsigned char) *(b->bf[b->b.y].s + i)) > 126)
+  {
+   j++;
+   if (((unsigned char) *(b->bf[b->b.y].s + i)) < 128 + ' ') j++;
+  }
+  else if (*(b->bf[b->b.y].s + i) < ' ') j++;
+  if (f->dtmd == DTMD_HELP)
+  {
+   if (b->bf[b->b.y].s[i] == HBG || b->bf[b->b.y].s[i] == HED ||
+     b->bf[b->b.y].s[i] == HHD || b->bf[b->b.y].s[i] == HFE ||
+     b->bf[b->b.y].s[i] == HFB || b->bf[b->b.y].s[i] == HBB ||
+     b->bf[b->b.y].s[i] == HNF)
+    j -= 2;
+  }
+ }
+ if (b->b.y - s->c.y < 0 || b->b.y - s->c.y >= NUM_LINES_ON_SCREEN - 1 ||
+   s->c.y < 0 || s->c.y >= b->mxlines ||
+   b->b.x + j - s->c.x < 0 ||
+   b->b.x + j- s->c.x >= NUM_COLS_ON_SCREEN - 1 )
+ {
+#if defined(UNIX)
       /*if(b->b.y - s->c.y < 0) s->c.y = b->b.y - (f->e.y - f->a.y)/2;
       else if(b->b.y - s->c.y >= f->e.y - f->a.y -1)
       s->c.y = b->b.y - (f->e.y - f->a.y)/2;*/
-	if (b->b.y - s->c.y < -1)
-        {
-         s->c.y = b->b.y - (NUM_LINES_ON_SCREEN)/2;
-        }
-      	else if (b->b.y - s->c.y == -1)
-        {
-         s->c.y -= 1;
-        }
-	else if (b->b.y - s->c.y > NUM_LINES_ON_SCREEN -1)
-      	{
-         s->c.y = b->b.y - (NUM_LINES_ON_SCREEN)/2;
-        }
-      	else if (b->b.y - s->c.y == NUM_LINES_ON_SCREEN -1)
-	{
-	 s->c.y += 1;
-      	}
+  if (b->b.y - s->c.y < -1)
+  {
+   s->c.y = b->b.y - (NUM_LINES_ON_SCREEN)/2;
+  }
+  else if (b->b.y - s->c.y == -1)
+  {
+   s->c.y -= 1;
+  }
+  else if (b->b.y - s->c.y > NUM_LINES_ON_SCREEN -1)
+  {
+   s->c.y = b->b.y - (NUM_LINES_ON_SCREEN)/2;
+  }
+  else if (b->b.y - s->c.y == NUM_LINES_ON_SCREEN -1)
+  {
+   s->c.y += 1;
+  }
 #else
-      if(b->b.y - s->c.y < 0) s->c.y = b->b.y;
-      else if(b->b.y - s->c.y >= NUM_LINES_ON_SCREEN -1)
-      (s->c.y) = b->b.y - f->e.y + f->a.y + 2 ;
+  if (b->b.y - s->c.y < 0) s->c.y = b->b.y;
+  else if (b->b.y - s->c.y >= NUM_LINES_ON_SCREEN -1)
+   (s->c.y) = b->b.y - f->e.y + f->a.y + 2;
 #endif
-      if(s->c.y >= b->mxlines - 1) s->c.y = b->mxlines - 2 ;
-      if(s->c.y < 0) s->c.y = 0 ;
-      
-#ifdef UNIX
-      if(b->b.x + j - s->c.x < 0)
-      (s->c.x) = b->b.x + j - (NUM_COLS_ON_SCREEN)/2 ;
-      else if(b->b.x + j - s->c.x >= NUM_COLS_ON_SCREEN - 1)
-      (s->c.x) = b->b.x + j - (NUM_COLS_ON_SCREEN)/2 ;
-#else
-      if(b->b.x + j - s->c.x < 0) (s->c.x) = b->b.x - 2 ;
-      else if(b->b.x + j - s->c.x >= NUM_COLS_ON_SCREEN - 1)
-      (s->c.x) = b->b.x + j - f->e.x + f->a.x+ 2 ;
-#endif
-      if(s->c.x < 0) s->c.x = 0 ;
-      else if(s->c.x >= b->bf[b->b.y].len + j) s->c.x = b->bf[b->b.y].len + j;
-      e_schirm(f, sw);
-   }
-   if(s->fa.y == -1) {  e_schirm(f, sw);  s->fa.y--;  }
-   else if(s->fa.y > -1) s->fa.y = -1;
-   if(sw != 0)
-   {  iold = e_lst_zeichen(f->e.x, f->a.y+1, f->e.y-f->a.y-1, 0,
-                             f->fb->em.fb, b->mxlines, iold, b->b.y);
-      jold = e_lst_zeichen(f->a.x+19, f->e.y, f->e.x-f->a.x-20, 1,
-                             f->fb->em.fb, b->mx.x, jold, b->b.x);
-   }
-   b->cl = b->b.x+j;
-   fk_locate(f->a.x+b->b.x-s->c.x+j+1, f->a.y+b->b.y-s->c.y+1);
+  if (s->c.y >= b->mxlines - 1) s->c.y = b->mxlines - 2 ;
+  if (s->c.y < 0) s->c.y = 0 ;
+
+  if (b->b.x + j - s->c.x < 0)
+   (s->c.x) = b->b.x + j - (NUM_COLS_ON_SCREEN)/2 ;
+  else if (b->b.x + j - s->c.x >= NUM_COLS_ON_SCREEN - 1)
+   (s->c.x) = b->b.x + j - (NUM_COLS_ON_SCREEN)/2 ;
+  if (s->c.x < 0) s->c.x = 0 ;
+  else if (s->c.x >= b->bf[b->b.y].len + j) s->c.x = b->bf[b->b.y].len + j;
+  e_schirm(f, sw);
+ }
+ if (s->fa.y == -1) {  e_schirm(f, sw);  s->fa.y--;  }
+ else if (s->fa.y > -1) s->fa.y = -1;
+ if (sw != 0)
+ {
+  iold = e_lst_zeichen(f->e.x, f->a.y+1, f->e.y-f->a.y-1, 0,
+    f->fb->em.fb, b->mxlines, iold, b->b.y);
+  jold = e_lst_zeichen(f->a.x+19, f->e.y, f->e.x-f->a.x-20, 1,
+    f->fb->em.fb, b->mx.x, jold, b->b.x);
+ }
+ b->cl = b->b.x+j;
+ fk_locate(f->a.x+b->b.x-s->c.x+j+1, f->a.y+b->b.y-s->c.y+1);
 }
 
 /*   delete one line */
@@ -1608,50 +1602,56 @@ int e_del_line(int yd, BUFFER *b, SCHIRM *s)
 /*   delete N chars from buffer */
 int e_del_nchar(BUFFER *b, SCHIRM *s, int x, int y, int n)
 {
-   extern struct EXT h_error;
-   FENSTER *f = h_error.cn->f[h_error.cn->mxedt];
-   int len, i, j;
-   (f->save) += n;
-   e_add_undo('r', b, x, y, n);
-   e_undo_sw++;
-   len = b->bf[y].len;
-   if( *(b->bf[y].s+len) == WPE_WR) len++;
-   for (j = x; j <= len - n; ++j) *(b->bf[y].s+j) = *(b->bf[y].s+j+n);
-   if(s->mark_begin.y == y && s->mark_begin.x > x)
-   s->mark_begin.x = (s->mark_begin.x-n > x) ? s->mark_begin.x-n : x;
-   if(s->mark_end.y == y && s->mark_end.x > x)
-   s->mark_end.x = (s->mark_end.x-n > x) ? s->mark_end.x-n : x;
-   
-   if(len <= n) e_del_line(y, b, s);
-   else if(y < b->mxlines - 1 && *(b->bf[y].s+len-n-1) != WPE_WR )
-   {  if(b->mx.x+n-len > b->bf[y+1].len)  i = b->bf[y+1].len;
-      else  i = e_su_rblk(b->mx.x+n-len, b->bf[y+1].s);
-      if(b->bf[y+1].s[i] == WPE_WR) i++;
-      if(i > 0)
-      {  for(j = 0; j < i; ++j) *(b->bf[y].s+len-n+j) = *(b->bf[y+1].s+j);
-	 *(b->bf[y].s+len-n+i) = '\0';
-	 if(s->mark_begin.y == y+1 && s->mark_begin.x <= i)
-	 {  s->mark_begin.y--;  s->mark_begin.x += (len-n);  }
-	 if(s->mark_end.y == y+1 && s->mark_end.x <= i)
-	 {  s->mark_end.y--;  s->mark_end.x += (len-n);  }
-	 e_del_nchar(b, s, 0, y+1, i);
-      }
-   }
-   if(y < b->mxlines)
-   {  b->bf[y].len = e_str_len(b->bf[y].s);
-      b->bf[y].nrc = e_str_nrc(b->bf[y].s);
-   }
-   e_undo_sw--;
-   sc_txt_4(y, b, 0);
-   return(x+n);
+ FENSTER *f = WpeEditor->f[WpeEditor->mxedt];
+ int len, i, j;
+
+ (f->save) += n;
+ e_add_undo('r', b, x, y, n);
+ e_undo_sw++;
+ len = b->bf[y].len;
+ if (*(b->bf[y].s+len) == WPE_WR) len++;
+ for (j = x; j <= len - n; ++j) *(b->bf[y].s+j) = *(b->bf[y].s+j+n)
+  ;
+ if (s->mark_begin.y == y && s->mark_begin.x > x)
+  s->mark_begin.x = (s->mark_begin.x-n > x) ? s->mark_begin.x-n : x;
+ if (s->mark_end.y == y && s->mark_end.x > x)
+  s->mark_end.x = (s->mark_end.x-n > x) ? s->mark_end.x-n : x;
+
+ if (len <= n) e_del_line(y, b, s);
+ else if (y < b->mxlines - 1 && *(b->bf[y].s+len-n-1) != WPE_WR )
+ {
+  if (b->mx.x+n-len > b->bf[y+1].len)
+   i = b->bf[y+1].len;
+  else
+   i = e_su_rblk(b->mx.x+n-len, b->bf[y+1].s);
+  if (b->bf[y+1].s[i] == WPE_WR) i++;
+  if (i > 0)
+  {
+   for (j = 0; j < i; ++j)
+    *(b->bf[y].s+len-n+j) = *(b->bf[y+1].s+j);
+   *(b->bf[y].s+len-n+i) = '\0';
+   if (s->mark_begin.y == y+1 && s->mark_begin.x <= i)
+   {  s->mark_begin.y--;  s->mark_begin.x += (len-n);  }
+   if (s->mark_end.y == y+1 && s->mark_end.x <= i)
+   {  s->mark_end.y--;  s->mark_end.x += (len-n);  }
+   e_del_nchar(b, s, 0, y+1, i);
+  }
+ }
+ if (y < b->mxlines)
+ {
+  b->bf[y].len = e_str_len(b->bf[y].s);
+  b->bf[y].nrc = e_str_nrc(b->bf[y].s);
+ }
+ e_undo_sw--;
+ sc_txt_4(y, b, 0);
+ return(x+n);
 }
 
 /*   insert N chars in buffer */
 int e_ins_nchar(BUFFER *b, SCHIRM *sch, unsigned char *s, int xa, int ya,
   int n)
 {
- extern struct EXT h_error;
- FENSTER *f = h_error.cn->f[h_error.cn->mxedt];
+ FENSTER *f = WpeEditor->f[WpeEditor->mxedt];
  int i, j;
 
  (f->save) += n;
@@ -1810,7 +1810,7 @@ int e_new_line(int yd, BUFFER *b)
  return(b->mxlines);
 }
 
-/*     ?berschreiben eines Buchstabens       */
+/*     Overwriting of a character       */
 int e_put_char(int c, BUFFER *b, SCHIRM *s)
 {
  unsigned char cc = c;
@@ -1841,7 +1841,7 @@ int e_su_lblk(int xa, char *s)
  return(xa);
 }
 
-/*     Suchen nach links (linkes Wortende)     */
+/*     Search left (left end of word)     */
 int e_su_rblk(int xa, char *s)
 {
  int len = strlen(s);
@@ -1859,13 +1859,12 @@ int e_su_rblk(int xa, char *s)
  return(!xa ? xa : xa + 1);
 }
 
-/*       Zeilen und Spaltenposition des Cursors schreiben        */
 /* Prints out the line number and column of cursor */
 void e_zlsplt(FENSTER *f)
 {
  char str[20];
 
- if (f->dtmd <= 'Z')
+ if (!DTMD_ISTEXT(f->dtmd))
   return;
  sprintf(str,"%5d:%-4d", f->b->b.y + 1, f->b->cl + 1);
  e_puts(str,f->a.x+5,f->e.y,f->fb->er.fb);
@@ -1885,7 +1884,7 @@ void e_zlsplt(FENSTER *f)
 void WpeFilenameToPathFile(char *filename, char **path, char **file)
 {
  char *tmp;
- char cur_dir[121];
+ char *cur_dir;
  int len;
 
  *path = NULL;
@@ -1900,28 +1899,21 @@ void WpeFilenameToPathFile(char *filename, char **path, char **file)
  }
  if ((!tmp) || ((filename + 1 == tmp) && (*filename == '.')))
  {
-  /* Should use WpeGetCurrentDir but no ECNT */
-  if (getcwd(cur_dir, 120) == NULL)
-  {
-   WpeFree(*file);
-   *file = NULL;
-   return ;
-  }
-  if (cur_dir[strlen(cur_dir) - 1] != DIRC)
-   strcat(cur_dir, DIRS);
-  *path = WpeStrdup(cur_dir);
+  *path = WpeGetCurrentDir(WpeEditor);
  }
  else
  {
   if (*filename != DIRC)
   {
-   /* Should use WpeGetCurrentDir but no ECNT */
-   if (getcwd(cur_dir, 120) == NULL)
+   if ((cur_dir = WpeGetCurrentDir(WpeEditor)) == NULL)
    {
     WpeFree(*file);
     *file = NULL;
     return ;
    }
+   /* Delete the slash at the end. */
+   len = strlen(cur_dir);
+   cur_dir[len - 1] = 0;
    len = tmp - filename + 1;
    while (strncmp(filename, "../", 3) == 0)
    {
@@ -1950,6 +1942,7 @@ void WpeFilenameToPathFile(char *filename, char **path, char **file)
     (*path)[strlen(cur_dir) + len] = DIRC;
     (*path)[strlen(cur_dir) + len + 1] = 0;
    }
+   WpeFree(cur_dir);
   }
   else
   {
@@ -1962,7 +1955,7 @@ void WpeFilenameToPathFile(char *filename, char **path, char **file)
 }
 
 /*   draw mouse slider cursor */
-int e_Lst_zeichen(int x, int y, int n, int sw, int frb, int max, int iold,
+int e_lst_zeichen(int x, int y, int n, int sw, int frb, int max, int iold,
   int new)
 {
  int inew;
@@ -1997,7 +1990,7 @@ int e_Lst_zeichen(int x, int y, int n, int sw, int frb, int max, int iold,
 }
 
 /*   draw mouse slider bar */
-void e_mouse_Leiste(int x, int y, int n, int sw, int frb)
+void e_mouse_bar(int x, int y, int n, int sw, int frb)
 {
  int sv = n;
 
@@ -2032,14 +2025,17 @@ int e_autosave(FENSTER *f)
  if (!(f->ed->autosv & 2)) return(0);
  /* Check if file system could have an autosave or emergency save file
     >12 check is to eliminate dos file systems */
- if ((maxname = pathconf(f->dirct, _PC_NAME_MAX) >= strlen(f->datnam) + 4) &&
+ if (((maxname = pathconf(f->dirct, _PC_NAME_MAX)) >= strlen(f->datnam) + 4) &&
    (maxname > 12))
  {
   str = MALLOC(strlen(f->datnam) + 5);
   str = e_make_postf(str, f->datnam, ".ASV");
   tmp = f->datnam;
   f->datnam = str;
-  e_save(f);
+  /*e_save(f);*/
+  WpeMouseChangeShape(WpeWorkingShape);
+  e_write(0, 0, f->b->mx.x, f->b->mxlines-1, f, WPE_NOBACKUP);
+  WpeMouseRestoreShape();
   f->datnam = tmp;
   f->save = 1;
   FREE(str);
@@ -2049,12 +2045,10 @@ int e_autosave(FENSTER *f)
 
 Undo *e_remove_undo(Undo *ud, int sw)
 {
- extern struct EXT h_error;
-
  if (ud == NULL)
   return(ud);
  ud->next = e_remove_undo(ud->next, sw + 1);
- if (sw > h_error.cn->numundo)
+ if (sw > WpeEditor->numundo)
  {
   if(ud->type == 'l')
    FREE(ud->u.pt);
@@ -2085,12 +2079,11 @@ Undo *e_remove_undo(Undo *ud, int sw)
 
 int e_add_undo(int sw, BUFFER *b, int x, int y, int n)
 {
- extern struct EXT h_error;
  Undo *next;
 
  if (e_undo_sw) return(0);
  if (!e_redo_sw && b->rd)
-  b->rd = e_remove_undo(b->rd, h_error.cn->numundo+1);
+  b->rd = e_remove_undo(b->rd, WpeEditor->numundo+1);
  if ((next = MALLOC(sizeof(Undo))) == NULL)
  {
   e_error(e_msg[ERR_LOWMEM], 0, b->fb);
@@ -2147,6 +2140,7 @@ int e_add_undo(int sw, BUFFER *b, int x, int y, int n)
   fn->c_sw = NULL;
   fn->c_st = NULL;
   fn->ed = NULL; /* Quick fix so that breakpoints aren't recalculated */
+  fn->fd.dirct = NULL;
   bn->f = fn;
   bn->f->s = sn;
   bn->b = e_set_pnt(0, 0);
@@ -2193,7 +2187,7 @@ int e_make_rudo(FENSTER *f, int sw)
  Undo *ud;
  int i;
 
- for (i = f->ed->mxedt; i > 0 && f->ed->f[i]->dtmd <= 'Z'; i--);
+ for (i = f->ed->mxedt; i > 0 && !DTMD_ISTEXT(f->ed->f[i]->dtmd); i--);
  if (i <= 0) return(0);
  e_switch_window(f->ed->edt[i], f);
  f = f->ed->f[f->ed->mxedt];
