@@ -1,4 +1,4 @@
-/* we_fl_nw.c                                             */
+/* we_fl_fkt.c                                            */
 /* Copyright (C) 1993 Fred Kruse                          */
 /* This is free software; you can redistribute it and/or  */
 /* modify it under the terms of the                       */
@@ -11,6 +11,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #endif
+#ifdef HAS_LIBZ
+#include <zlib.h>
+#endif
 #ifdef DEFPGC
 #define putc(c, fp) fputc(c, fp)
 #endif
@@ -36,115 +39,122 @@ char *e_mkfilename(char *dr, char *fn)
  }
  fl = MALLOC(strlen(dr) + strlen(fn) + 2);
  strcpy(fl, dr);
-#ifdef DJGPP
- if (dr[0] != '\0' && dr[strlen(dr)-1] != DIRC &&
-   dr[strlen(dr)-1] != '/')
-  strcat(fl, DIRS);
-#else
  if (dr[0] != '\0' && dr[strlen(dr)-1] != DIRC)
   strcat(fl, DIRS);
-#endif
  strcat(fl, fn);
  return(fl);
 }
 
 /*   read file routine     */
-POINT e_readin(int i, int j, FILE *fp, BUFFER *b, int sw)
+POINT e_readin(int i, int j, FILE *fp, BUFFER *b, char *type)
 {
-   POINT pkt;
-   int ii, k, n = 1, hb = 0;
-   signed char cc, c = 0;
-   WpeMouseChangeShape(WpeWorkingShape);
-   if(i == 0) e_new_line(j, b);
-   while (n == 1 && c != EOF)
-   {  for(; i < b->mx.x; i++)
-      {  if(hb != 2) n = fread(&c, sizeof(char), 1, fp);
-	 else {  c = cc;  hb = 0;  }
-#if defined(DJGPP)
-	 if(n == 1 && c == WR && i == 0) {  i--; continue;  }
-#endif
-/*
-	     else if(sw == 'm' && c == CtrlH)
-	     {    i -= 2;   continue;   }
-*/
-	 if(sw == 'h' && c == CtrlH)
-	 {  if(!hb)
-	    {  *(b->bf[j].s+i-1) = HBB;
-	       hb = 1;
-	    }
-	    else i--;
-	    n = fread(&c, sizeof(char), 1, fp);
-	 }
-	 else if(sw == 'h' && hb == 1)
-	 {  n = fread(&cc, sizeof(char), 1, fp);
-	    if(cc != HBS)
-	    {  *(b->bf[j].s+i) = HED;
-	       i++;
-	       hb = 2;
-	    }
-	    else n = fread(&c, sizeof(char), 1, fp);
-	 }
-	 if(n != 1)
-	 {  *(b->bf[j].s+i) = '\0';
-	    break;
-	 }
-	 *(b->bf[j].s+i) = c;
-#ifdef UNIX
-	 if(c == EOF || c== WPE_CR || c == WPE_WR) break;
-#else
-	 if(c == EOF || c== WPE_CR || c == WPE_WR || c == '\0') break;
-#endif
-      }
-      if (c == EOF || n != 1)
-      {  if(i == 0 && j > 0)
-	 {  FREE(b->bf[j].s);
-	    for(k = j; k < b->mxlines; k++) b->bf[k] = b->bf[k+1];
-	    (b->mxlines)--;
-	 }
-	 else
-	 {  *(b->bf[j].s+i) = WPE_WR;
-	    *(b->bf[j].s+i+1) = '\0';
-	    j++;
-	 }
-      }
-      else if(i >= b->mx.x)
-      {  for(ii = i-1; *(b->bf[j].s+ii) != ' ' && *(b->bf[j].s+ii) != '-' && ii > 0; ii--);
-	 if(ii <= 1) ii = i - 1;
-	 e_new_line(j+1, b);
-	 for(k = ii+1; k <= i; k++)
-	 {  if(*(b->bf[j].s+k) != '\0') *(b->bf[j+1].s+k-ii-1) = *(b->bf[j].s+k);
-	    else *(b->bf[j+1].s+k-ii-1) = ' ';
-	 }
-	 i = i - ii - 1;
-	 *(b->bf[j].s+ii+1) = '\0';
-	 j++;
-	 if(i < 0) i = 0;
-      }
-      else if(c == WPE_CR || c == WPE_WR)
-      {  *(b->bf[j].s+i) = WPE_WR;
-	 *(b->bf[j].s+i+1) = '\0';
-	 e_new_line(++j, b);
-/*
-#if defined(DJGPP)
-             c = getc(fp);
-#endif
-*/
-	 i = 0;
-      }
-      else
-      {  *(b->bf[j].s+i) = ' ';
-	 i++;
-	 *(b->bf[j].s+i) = '\0';
-      }
-      b->bf[j-1].len = e_str_len(b->bf[j-1].s);
-      b->bf[j-1].nrc = e_str_nrc(b->bf[j-1].s);
+ POINT pkt;
+ int ii, k, n = 1, hb = 0;
+ signed char cc, c = 0;
+
+ WpeMouseChangeShape(WpeWorkingShape);
+ if (i == 0) e_new_line(j, b);
+ while (n == 1)
+ {
+  for (; i < b->mx.x; i++)
+  {
+   if (hb != 2)
+    n = fread(&c, sizeof(char), 1, fp);
+   else
+   {
+    c = cc;
+    hb = 0;
    }
-   pkt.x = i;  pkt.y = j;
-   WpeMouseRestoreShape();
-   return(pkt);
+   if (n == 1 && c == WPE_CR)
+   {
+    if (DTMD_NORMAL == *type)
+    {
+     *type = DTMD_MSDOS;
+    }
+    i--;
+    continue;
+   }
+   if ((DTMD_HELP == *type) && (c == CtrlH))
+   {
+    if (!hb)
+    {
+     *(b->bf[j].s+i-1) = HBB;
+     hb = 1;
+    }
+    else i--;
+    n = fread(&c, sizeof(char), 1, fp);
+   }
+   else if ((DTMD_HELP == *type) && (hb == 1))
+   {
+    n = fread(&cc, sizeof(char), 1, fp);
+    if (cc != HBS)
+    {
+     *(b->bf[j].s+i) = HED;
+     i++;
+     hb = 2;
+    }
+    else n = fread(&c, sizeof(char), 1, fp);
+   }
+   if (n != 1)
+   {
+    *(b->bf[j].s+i) = '\0';
+    break;
+   }
+   *(b->bf[j].s+i) = c;
+   if (c == WPE_WR) break;
+  }
+  if (n != 1)
+  {
+   if (i == 0 && j > 0)
+   {
+    FREE(b->bf[j].s);
+    for (k = j; k < b->mxlines; k++) b->bf[k] = b->bf[k+1];
+    (b->mxlines)--;
+   }
+   else
+   {
+    *(b->bf[j].s+i) = WPE_WR;
+    *(b->bf[j].s+i+1) = '\0';
+    j++;
+   }
+  }
+  else if (i >= b->mx.x)
+  {
+   for (ii = i-1; *(b->bf[j].s+ii) != ' ' && *(b->bf[j].s+ii) != '-' && ii > 0; ii--);
+   if (ii <= 1) ii = i - 1;
+   e_new_line(j+1, b);
+   for (k = ii+1; k <= i; k++)
+   {
+    if (*(b->bf[j].s+k) != '\0') *(b->bf[j+1].s+k-ii-1) = *(b->bf[j].s+k);
+    else *(b->bf[j+1].s+k-ii-1) = ' ';
+   }
+   i = i - ii - 1;
+   *(b->bf[j].s+ii+1) = '\0';
+   j++;
+   if (i < 0) i = 0;
+  }
+  else if (c == WPE_WR)
+  {
+   *(b->bf[j].s+i) = WPE_WR;
+   *(b->bf[j].s+i+1) = '\0';
+   e_new_line(++j, b);
+   i = 0;
+  }
+  else
+  {
+   *(b->bf[j].s+i) = ' ';
+   i++;
+   *(b->bf[j].s+i) = '\0';
+  }
+  b->bf[j-1].len = e_str_len(b->bf[j-1].s);
+  b->bf[j-1].nrc = e_str_nrc(b->bf[j-1].s);
+ }
+ pkt.x = i;  pkt.y = j;
+ WpeMouseRestoreShape();
+ return(pkt);
 }
 
-/*   Neues Edit-Fenster oeffnen   */
+/*   Open new edit window   */
 int e_new(FENSTER *f)
 {
  e_edit(f->ed, "");
@@ -162,10 +172,10 @@ int e_m_save(FENSTER *f)
 
 int e_save(FENSTER *f)
 {
- BUFFER *b = f->ed->f[f->ed->mxedt]->b;
+ BUFFER *b;
  int ret;
 
- if (f->dtmd <= 'Z')
+ if (!DTMD_ISTEXT(f->dtmd))
   return(0);
  for (ret = f->ed->mxedt; ret > 0 && f->ed->f[ret] != f; ret--)
   ;
@@ -173,7 +183,7 @@ int e_save(FENSTER *f)
   return(0);
  WpeMouseChangeShape(WpeWorkingShape);
  b = f->ed->f[ret]->b;
- ret = e_write(0, 0, b->mx.x, b->mxlines-1, f);
+ ret = e_write(0, 0, b->mx.x, b->mxlines-1, f, WPE_BACKUP);
  if (ret != WPE_ESC)
   f->save = 0;
  WpeMouseRestoreShape();
@@ -220,7 +230,7 @@ int e_quit(FENSTER *f)
   if (e_close_window(cn->f[cn->mxedt]) == WPE_ESC)
    return(WPE_ESC);
  }
-  if (cn->autosv != 0)
+ if (cn->autosv & 1)
   e_save_opt(cn->f[0]);
 #ifdef UNIX
  if (WpeQuitWastebasket(cn->f[0]))
@@ -241,11 +251,11 @@ int e_quit(FENSTER *f)
 }
 
 /*	write file to disk */
-int e_write(int xa, int ya, int xe, int ye, FENSTER *f)
+int e_write(int xa, int ya, int xe, int ye, FENSTER *f, int backup)
 {
  BUFFER *b;
  int i = xa, j;
- char tmp[80], *ptmp;
+ char *tmp, *ptmp;
  FILE *fp;
 
  for (j = f->ed->mxedt; j > 0 && f->ed->f[j] != f; j--)
@@ -254,11 +264,13 @@ int e_write(int xa, int ya, int xe, int ye, FENSTER *f)
  if (f->ins == 8)
   return(WPE_ESC);
  ptmp = e_mkfilename(f->dirct, f->datnam);
- if (access(ptmp, 0) == 0)
+ if ((backup == WPE_BACKUP) && (access(ptmp, 0) == 0))
  {
-  if (access(e_bakfilename(ptmp, tmp), 0) == 0)
+  tmp = e_bakfilename(ptmp);
+  if (access(tmp, 0) == 0)
    remove(tmp);
   WpeRenameCopy(ptmp, tmp, f, 1);
+  WpeFree(tmp);
  }
  if ((fp = fopen(ptmp, "wb")) == NULL)
  {
@@ -266,10 +278,8 @@ int e_write(int xa, int ya, int xe, int ye, FENSTER *f)
   FREE(ptmp);
   return(WPE_ESC);
  }
-#ifdef UNIX
  if (f->filemode >= 0)
   chmod(ptmp, f->filemode);
-#endif
  for (j = ya; j < ye && j < b->mxlines; j++)
  {
   if (b->bf[j].s != NULL)
@@ -277,9 +287,10 @@ int e_write(int xa, int ya, int xe, int ye, FENSTER *f)
    {
     if (*(b->bf[j].s+i) == WPE_WR)
     {
-#if defined(DJGPP)
-     putc(WPE_CR, fp);
-#endif
+     if (DTMD_MSDOS == f->dtmd)
+     {
+      putc(WPE_CR, fp);
+     }
      putc(WPE_WR, fp);
      break;
     }
@@ -293,9 +304,10 @@ int e_write(int xa, int ya, int xe, int ye, FENSTER *f)
   {
    if (*(b->bf[j].s+i) == WPE_WR)
    {
-#if defined(DJGPP)
-    putc(WPE_CR, fp);
-#endif
+    if (DTMD_MSDOS == f->dtmd)
+    {
+     putc(WPE_CR, fp);
+    }
     putc(WPE_WR, fp);
     break;
    }
@@ -332,30 +344,33 @@ char *e_new_qual(char *s, char *ns, char *sb)
 }
 
 /*     make up the bak-file name */
-char *e_bakfilename(char* s, char* sb)
+char *e_bakfilename(char* s)
 {
-  /* "" is the special code for TurboC replace-extension style */
-# ifndef SIMPLE_BACKUP_SUFFIX
-# define SIMPLE_BACKUP_SUFFIX ""
-# endif
+ /* "" is the special code for TurboC replace-extension style */
+#ifndef SIMPLE_BACKUP_SUFFIX
+#define SIMPLE_BACKUP_SUFFIX ""
+#endif
 
-  static char* bak = NULL;
-  if (!bak) {
-    bak = getenv("SIMPLE_BACKUP_SUFFIX");
-    if (!bak) bak = SIMPLE_BACKUP_SUFFIX;
-  }
-  
-  if (!*bak)
-    return e_new_qual (s, "bak", sb); /* TurboC style */
-  else
-  {
-    strcpy (sb, s);
-    strcat (sb, bak);
-    return (sb);
-  }
+ static char* bak = NULL;
+ char *result;
+ if (!bak)
+ {
+  bak = getenv("SIMPLE_BACKUP_SUFFIX");
+  if (!bak) bak = SIMPLE_BACKUP_SUFFIX;
+ }
+
+ result = WpeMalloc(strlen(s) + strlen(bak) + 5);
+ if (!*bak)
+  return e_new_qual(s, "bak", result); /* TurboC style */
+ else
+ {
+  strcpy(result, s);
+  strcat(result, bak);
+  return result;
+ }
 }
 
-/*    File/Directory-Struktur freigeben   */
+/*    Clear file-/directory-structure   */
 int freedf(struct dirfile *df)
 {
  if (df == NULL) return(-1);
@@ -382,8 +397,8 @@ int e_file_window(int sw, FLWND *fw, int ft, int fz)
    {  if(fw->nf >= fw->df->anz) fw->nf = fw->df->anz-1;
       len = strlen(*(fw->df->name+fw->nf));
    }
-   e_mouse_leiste(fw->xe, fw->ya, fw->ye-fw->ya, 0, fw->f->fb->em.fb);
-   e_mouse_leiste(fw->xa, fw->ye, fw->xe-fw->xa, 1, fw->f->fb->em.fb);
+   e_mouse_bar(fw->xe, fw->ya, fw->ye-fw->ya, 0, fw->f->fb->em.fb);
+   e_mouse_bar(fw->xa, fw->ye, fw->xe-fw->xa, 1, fw->f->fb->em.fb);
    while (1)
    {  e_pr_file_window(fw, 1, 1, ft, fz, 0);
 #if  MOUSE
@@ -512,6 +527,12 @@ struct help_ud
  int x, y, sw;
 } *ud_help;
 
+#ifdef HAS_LIBZ
+typedef gzFile IFILE;
+
+#define e_i_fgets(s, n, p) gzgets(p, s, n)
+#define e_i_fclose(p) gzclose(p)
+#else
 typedef struct
 {
  FILE *fp;
@@ -519,6 +540,7 @@ typedef struct
 } IFILE;
 
 #define e_i_fgets(s, n, p) fgets(s, n, p->fp)
+#endif
 
 int e_mkdir_path(char *path)
 {
@@ -545,8 +567,58 @@ int e_mkdir_path(char *path)
 
 IFILE *e_i_fopen(char *path, char *stat)
 {
+ char *tmp2;
+#ifdef HAS_LIBZ
+ IFILE *fp;
+
+ if (!path) {  return(NULL);  }
+ if ((fp = gzopen(path, stat))) {  return(fp);  }
+ if (!(tmp2 = MALLOC((strlen(path)+11)*sizeof(char))))
+ {
+  FREE(fp);
+  return(NULL);
+ }
+ strcpy(tmp2, path);
+ strcat(tmp2, ".info");
+ if ((fp = gzopen(tmp2, stat)))
+ {
+  FREE(tmp2);
+  return(fp);
+ }
+ strcpy(tmp2, path);
+ strcat(tmp2, ".gz");
+ if ((fp = gzopen(tmp2, stat)))
+ {
+  FREE(tmp2);
+  return(fp);
+ }
+ strcpy(tmp2, path);
+ strcat(tmp2, ".Z");
+ if ((fp = gzopen(tmp2, stat)))
+ {
+  FREE(tmp2);
+  return(fp);
+ }
+ strcpy(tmp2, path);
+ strcat(tmp2, ".info.gz");
+ if ((fp = gzopen(tmp2, stat)))
+ {
+  FREE(tmp2);
+  return(fp);
+ }
+ strcpy(tmp2, path);
+ strcat(tmp2, ".info.Z");
+ if ((fp = gzopen(tmp2, stat)))
+ {
+  FREE(tmp2);
+  return(fp);
+ }
+ FREE(fp);
+ FREE(tmp2);
+ return(NULL);
+#else
  extern char *e_tmp_dir;
- char *tmp, *tmp2, *command;
+ char *tmp, *command;
  int len;
  IFILE *fp = MALLOC(sizeof(IFILE));
 
@@ -617,8 +689,10 @@ IFILE *e_i_fopen(char *path, char *stat)
  if (!fp->fp) {  FREE(fp);  return(NULL);  }
  fp->sw = 1;
  return(fp);  
+#endif
 }
 
+#ifndef HAS_LIBZ
 int e_i_fclose(IFILE *fp)
 {
  int ret = fclose(fp->fp);
@@ -626,6 +700,7 @@ int e_i_fclose(IFILE *fp)
  FREE(fp);
  return(ret);
 }
+#endif
 
 int e_read_help(char *str, FENSTER *f, int sw)
 {
@@ -704,10 +779,11 @@ int e_read_help(char *str, FENSTER *f, int sw)
 
 int e_help_ret(FENSTER *f)
 {
-   BUFFER *b = f->b;
-   int i, j;
-   char str[126];
-   struct help_ud *next;
+ BUFFER *b = f->b;
+ int i, j;
+ char str[126];
+ struct help_ud *next;
+
    for(i = b->b.x; i >= 0 && b->bf[b->b.y].s[i] != HED; i--)
    {  if(b->bf[b->b.y].s[i] == HBG)
       {  str[0] = HHD;
@@ -866,79 +942,90 @@ int e_help_free(FENSTER *f)
 
 int e_help_comp(FENSTER *f)
 {
-   BUFFER *b = f->b;
-   int i, j, k, hn = 0, sn = 0;
-   char **swtch = MALLOC(sizeof(char *));
-   char **hdrs = MALLOC(sizeof(char *));
-   char str[256];
-   if(f->dtmd != 'h') return(1);
-   for(j = 0; j < b->mxlines; j++)
-   {  for(i = 0; i < b->bf[j].len; i++)
-      {  if(b->bf[j].s[i] == HBG)
-	 {  for(k = i+1; k < b->bf[j].len
-		  && (str[k-i-1] = b->bf[j].s[k]) != HED
-		  && str[k-i-1] != HBG && str[k-i-1] != HHD
-		  && str[k-i-1] != HFE ; k++);
-	    if(str[k-i-1] != HED)
-	    {  e_error("Error in Switch!", 0, f->fb);
-	       b->b.x = k;  b->b.y = j;
-	       e_cursor(f, 1);
-	       return(2);
-	    }
-	    str[k-i-1] = '\0';
-	    sn++;
-	    swtch = REALLOC(swtch, sn * sizeof(char *));
-	    swtch[sn-1] = MALLOC((strlen(str) + 1) + sizeof(char));
-	    strcpy(swtch[sn-1], str);
-	 }
-      }
+ BUFFER *b = f->b;
+ int i, j, k, hn = 0, sn = 0;
+ char **swtch = MALLOC(sizeof(char *));
+ char **hdrs = MALLOC(sizeof(char *));
+ char str[256];
+
+ if (f->dtmd != DTMD_HELP) return(1);
+ for (j = 0; j < b->mxlines; j++)
+ {
+  for (i = 0; i < b->bf[j].len; i++)
+  {
+   if (b->bf[j].s[i] == HBG)
+   {
+    for (k = i+1; k < b->bf[j].len && (str[k-i-1] = b->bf[j].s[k]) != HED &&
+      str[k-i-1] != HBG && str[k-i-1] != HHD && str[k-i-1] != HFE ; k++)
+     ;
+    if (str[k-i-1] != HED)
+    {
+     e_error("Error in Switch!", 0, f->fb);
+     b->b.x = k;  b->b.y = j;
+     e_cursor(f, 1);
+     return(2);
+    }
+    str[k-i-1] = '\0';
+    sn++;
+    swtch = REALLOC(swtch, sn * sizeof(char *));
+    swtch[sn-1] = MALLOC((strlen(str) + 1) + sizeof(char));
+    strcpy(swtch[sn-1], str);
    }
-   for(j = 0; j < b->mxlines; j++)
-   {  for(i = 0; i < b->bf[j].len; i++)
-      {  if(b->bf[j].s[i] == HHD)
-	 {  for(k = i+1; k < b->bf[j].len
-		  && (str[k-i-1] = b->bf[j].s[k]) != HED
-		  && str[k-i-1] != HBG && str[k-i-1] != HHD
-		  && str[k-i-1] != HFE ; k++);
-	    if(str[k-i-1] != HED)
-	    {  e_error("Error in Header!", 0, f->fb);
-	       b->b.x = k;  b->b.y = j;
-	       e_cursor(f, 1);
-	       return(2);
-	    }
-	    str[k-i-1] = '\0';
-	    hn++;
-	    hdrs = REALLOC(hdrs, hn * sizeof(char *));
-	    hdrs[hn-1] = MALLOC((strlen(str) + 1) + sizeof(char));
-	    strcpy(hdrs[hn-1], str);
-	 }
-      }
+  }
+ }
+ for (j = 0; j < b->mxlines; j++)
+ {
+  for (i = 0; i < b->bf[j].len; i++)
+  {
+   if (b->bf[j].s[i] == HHD)
+   {
+    for (k = i+1; k < b->bf[j].len && (str[k-i-1] = b->bf[j].s[k]) != HED &&
+      str[k-i-1] != HBG && str[k-i-1] != HHD && str[k-i-1] != HFE ; k++)
+     ;
+    if (str[k-i-1] != HED)
+    {
+     e_error("Error in Header!", 0, f->fb);
+     b->b.x = k;  b->b.y = j;
+     e_cursor(f, 1);
+     return(2);
+    }
+    str[k-i-1] = '\0';
+    hn++;
+    hdrs = REALLOC(hdrs, hn * sizeof(char *));
+    hdrs[hn-1] = MALLOC((strlen(str) + 1) + sizeof(char));
+    strcpy(hdrs[hn-1], str);
    }
-   for(j = 0; j < sn; j++)
-   {  for(i = 0; i < hn; i++) if(!WpeStrccmp(swtch[j], hdrs[i])) break;
-      if(i >= hn)
-      {  sprintf(str, "Switch \'%s\' has no Header!", swtch[j]);
-	 e_error(str, 0, f->fb);
-	 goto ende;
-      }
-   }
-   for(j = 0; j < hn; j++)
-   {  for(i = 0; i < sn; i++) if(!WpeStrccmp(swtch[i], hdrs[j])) break;
-      if(i >= sn)
-      {  sprintf(str, "No Jump to Header \'%s\'!", hdrs[j]);
-	 e_error(str, 0, f->fb);
-	 goto ende;
-      }
-   }
-   ende:
-    for(i = 0; i < sn; i++) FREE(swtch[i]);
-   for(i = 0; i < hn; i++) FREE(hdrs[i]);
-   FREE(swtch);
-   FREE(hdrs);
-   return(0);
+  }
+ }
+ for (j = 0; j < sn; j++)
+ {
+  for (i = 0; i < hn; i++) if(!WpeStrccmp(swtch[j], hdrs[i])) break;
+  if (i >= hn)
+  {
+   sprintf(str, "Switch \'%s\' has no Header!", swtch[j]);
+   e_error(str, 0, f->fb);
+   goto ende;
+  }
+ }
+ for (j = 0; j < hn; j++)
+ {
+  for (i = 0; i < sn; i++) if(!WpeStrccmp(swtch[i], hdrs[j])) break;
+  if (i >= sn)
+  {
+   sprintf(str, "No Jump to Header \'%s\'!", hdrs[j]);
+   e_error(str, 0, f->fb);
+   goto ende;
+  }
+ }
+ende:
+ for (i = 0; i < sn; i++) FREE(swtch[i]);
+ for (i = 0; i < hn; i++) FREE(hdrs[i]);
+ FREE(swtch);
+ FREE(hdrs);
+ return(0);
 }
 
-/*          Hilfe-Fenster    */
+/*          Help window    */
 int e_help(FENSTER *f)
 {
  extern char *e_hlp;
@@ -959,33 +1046,43 @@ int e_info(FENSTER *f)
 
 int e_mk_info_button(char *str)
 {
-   int i, bg, nd, len = strlen(str);
-   if(str[0] == '\n' || str[0] == '\0') return(1);
-   for(bg = 0; str[bg] && isspace(str[bg]); bg++);
-   for(nd = bg; str[nd] && (str[nd] != ':' || 
-      (!isspace(str[nd+1]) && (str[nd+1] != '(') && (str[nd+1] != ':' || 
-       (!isspace(str[nd+2]) && str[nd+2] != '.')))); nd++);
-   if(!str[nd]) return(-1);
-   if(str[nd+1] != ':')
-   {  for(nd++; str[nd] && isspace(str[nd]); nd++);
-      if(str[nd] == '(')
-      {  for(i = len; i >= bg; i--) str[i+1] = str[i];
-         str[bg] = HNF;
-         str[nd+1] = HED;
-         for(nd++; str[nd] && str[nd] && str[nd] != ')'; nd++);
-         if(str[nd]) str[nd] = HED;
-         return(0);
-      }
-      for(; str[nd] && (str[nd] != '.' || isalnum1(str[nd+1])); nd++);
-      if(!str[nd]) return(-1);
-      for(bg = nd; str[bg] != ':' || !isspace(str[bg+1]); bg--);
-      for(bg++; isspace(str[bg]); bg++);
-   }
-   for(i = len; i >= nd; i--) str[i+2] = str[i];
+ int i, bg, nd, len = strlen(str);
+
+ if (str[0] == '\n' || str[0] == '\0')
+  return(1);
+ for (bg = 0; str[bg] && isspace(str[bg]); bg++)
+  ;
+ for (nd = bg; str[nd] && (str[nd] != ':' || 
+   (!isspace(str[nd+1]) && (str[nd+1] != '(') && (str[nd+1] != ':' || 
+   (!isspace(str[nd+2]) && str[nd+2] != '.')))); nd++)
+  ;
+ if (!str[nd]) return(-1);
+ if (str[nd+1] != ':')
+ {
+  for (nd++; str[nd] && isspace(str[nd]); nd++);
+  if (str[nd] == '(')
+  {
+   for (i = len; i >= bg; i--)
+    str[i+1] = str[i];
+   str[bg] = HNF;
    str[nd+1] = HED;
-   for(; i >= bg; i--) str[i+1] = str[i];
-   str[bg] = HBG;
+   for (nd++; str[nd] && str[nd] && str[nd] != ')'; nd++);
+   if (str[nd])
+    str[nd] = HED;
    return(0);
+  }
+  for (; str[nd] && (str[nd] != '.' || isalnum1(str[nd+1])); nd++);
+  if (!str[nd]) return(-1);
+  for (bg = nd; str[bg] != ':' || !isspace(str[bg+1]); bg--);
+  for (bg++; isspace(str[bg]); bg++);
+ }
+ for (i = len; i >= nd; i--)
+  str[i+2] = str[i];
+ str[nd+1] = HED;
+ for (; i >= bg; i--)
+  str[i+1] = str[i];
+ str[bg] = HBG;
+ return(0);
 }
 
 int e_mk_info_mrk(char *str)
@@ -1009,65 +1106,69 @@ int e_mk_info_mrk(char *str)
 
 IFILE *e_info_jump(char *str, char **path, IFILE *fp)
 {
-   IFILE *fpn;
-   int i, j, n, anz = 0;
-   char *ptmp, *fstr, tstr[256], nfl[128];
-   struct FL_INFO{  char *name; int line;  } **files = MALLOC(1);
-   while(e_i_fgets(tstr, 256, fp) && tstr[0] != IFE)
-   {  for(i = 0; (nfl[i] = tstr[i]) && tstr[i] != ':'; i++);
-      if(!tstr[i]) continue;
-      nfl[i] = '\0';
-      anz++;
-      files = REALLOC(files, anz * sizeof(struct FL_INFO *));
-      files[anz-1] = MALLOC(sizeof(struct FL_INFO));
-      files[anz-1]->name = MALLOC((strlen(nfl)+1)*sizeof(char));
-      strcpy(files[anz-1]->name, nfl);
-      files[anz-1]->line = atoi(tstr+i+1);
-   }
-   i = (strlen(str) + 6);
-   fstr = MALLOC((i+1) * sizeof(char));
-   strcat(strcpy(fstr, "Node: "), str);
-   if(fstr[n = strlen(fstr)-1] == HED) {  fstr[n] = '\0';  i--;  }
-   while((ptmp = e_i_fgets(tstr, 256, fp)) && strncmp(tstr, fstr, i));
-   if(ptmp)
-   {  n = atoi(tstr+i+1);
-      for(i = 1; i < anz && n >= files[i]->line; i++);
-#ifdef DJGPP
-      if(files[i-1]->name[0] == DIRC || files[i-1]->name[0] == '/')
-#else
-      if(files[i-1]->name[0] == DIRC)
-#endif
-      {  ptmp = MALLOC((strlen(files[i-1]->name)+1)*sizeof(char));
-         strcpy(ptmp, files[i-1]->name);
-      }
-      else
-#ifdef DJGPP
-      {  for(n = strlen(*path)-1; n >= 0 && *(*path+n) != DIRC 
-            && *(*path+n) != '/'; n--);
-#else
-      {  for(n = strlen(*path)-1; n >= 0 && *(*path+n) != DIRC; n--);
-#endif
-         n++;
-         ptmp = MALLOC((strlen(files[i-1]->name)+n+1)*sizeof(char));
-         for(j = 0; j < n; j++) ptmp[j] = *(*path+j);
-         ptmp[j] = '\0';
-         strcat(ptmp, files[i-1]->name);
-      }
-      if((fpn = e_i_fopen(ptmp, "rb")) != NULL)
-      {  e_i_fclose(fp);
-         fp = fpn;
-         FREE(*path);
-         *path = ptmp;
-      }
-      else FREE(ptmp);
-   }
-   FREE(fstr);
-   for(i = 0; i < anz; i++)
-   {  FREE(files[i]->name);
-      FREE(files[i]);
-   }
-   FREE(files);
-   return(fp);
+ IFILE *fpn;
+ int i, j, n, anz = 0;
+ char *ptmp, *fstr, tstr[256], nfl[128];
+ struct FL_INFO{  char *name; int line;  } **files = MALLOC(1);
+
+ while (e_i_fgets(tstr, 256, fp) && tstr[0] != IFE)
+ {
+  for (i = 0; (nfl[i] = tstr[i]) && tstr[i] != ':'; i++)
+   ;
+  if (!tstr[i]) continue;
+  nfl[i] = '\0';
+  anz++;
+  files = REALLOC(files, anz * sizeof(struct FL_INFO *));
+  files[anz-1] = MALLOC(sizeof(struct FL_INFO));
+  files[anz-1]->name = MALLOC((strlen(nfl)+1)*sizeof(char));
+  strcpy(files[anz-1]->name, nfl);
+  files[anz-1]->line = atoi(tstr+i+1);
+ }
+ i = (strlen(str) + 6);
+ fstr = MALLOC((i+1) * sizeof(char));
+ strcat(strcpy(fstr, "Node: "), str);
+ if (fstr[n = strlen(fstr)-1] == HED) {  fstr[n] = '\0';  i--;  }
+ while ((ptmp = e_i_fgets(tstr, 256, fp)) && strncmp(tstr, fstr, i))
+  ;
+ if (ptmp)
+ {
+  n = atoi(tstr+i+1);
+  for (i = 1; i < anz && n >= files[i]->line; i++)
+   ;
+  if (files[i-1]->name[0] == DIRC)
+  {
+   ptmp = MALLOC((strlen(files[i-1]->name)+1)*sizeof(char));
+   strcpy(ptmp, files[i-1]->name);
+  }
+  else
+  {
+   for (n = strlen(*path)-1; n >= 0 && *(*path+n) != DIRC; n--)
+    ;
+   n++;
+   ptmp = MALLOC((strlen(files[i-1]->name)+n+1)*sizeof(char));
+   for (j = 0; j < n; j++)
+    ptmp[j] = *(*path+j);
+   ptmp[j] = '\0';
+   strcat(ptmp, files[i-1]->name);
+  }
+  if ((fpn = e_i_fopen(ptmp, "rb")) != NULL)
+  {
+   e_i_fclose(fp);
+   fp = fpn;
+   FREE(*path);
+   *path = ptmp;
+  }
+  else
+   FREE(ptmp);
+ }
+ FREE(fstr);
+ for (i = 0; i < anz; i++)
+ {
+  FREE(files[i]->name);
+  FREE(files[i]);
+ }
+ FREE(files);
+ return(fp);
 }
 
 char *e_mk_info_pt(char *str, char *node)
@@ -1098,34 +1199,33 @@ char *e_mk_info_pt(char *str, char *node)
 
 char *e_mk_info_path(char *path, char *file)
 {
-   int n;
-   char *tp;
-   if(!info_file || !*info_file) return(NULL);
-#ifdef DJGPP
-   if(file[0] == DIRC || file[0] == '/')
-#else
-   if(file[0] == DIRC)
-#endif
-   {  if(path && !strcmp(path, file)) {  FREE(path);  return(NULL);  }
-      else if(path) FREE(path);
-      if(!(path = MALLOC((strlen(file) + 1) * sizeof(char)))) return(NULL);
-      return(strcpy(path, file));
-   }
-   tp = info_file;
-   if(path)
-   {  n = strlen(path) - strlen(file);
-      if(n > 1) path[n - 1] = PTHD;
-      else if(n == 1) path[n++] = PTHD;
-      while(strncmp(tp, path, n) && (tp = strchr(tp, PTHD)) != NULL && *++tp);
-      if(tp == NULL || !*tp || !*(tp += n)) return(NULL);
-      FREE(path);
-   }
-   for(n = 0; tp[n] && tp[n] != PTHD; n++);
-   if(!(path = MALLOC((strlen(file) + n + 2) * sizeof(char)))) return(NULL);
-   strncpy(path, tp, n);
-   if(n > 1) path[n++] = DIRC;
-   strcpy(path + n, file);
-   return(path);
+ int n;
+ char *tp;
+
+ if (!info_file || !*info_file) return(NULL);
+ if (file[0] == DIRC)
+ {
+  if (path && !strcmp(path, file)) {  FREE(path);  return(NULL);  }
+  else if(path) FREE(path);
+  if (!(path = MALLOC((strlen(file) + 1) * sizeof(char)))) return(NULL);
+  return (strcpy(path, file));
+ }
+ tp = info_file;
+ if (path)
+ {
+  n = strlen(path) - strlen(file);
+  if (n > 1) path[n - 1] = PTHD;
+  else if (n == 1) path[n++] = PTHD;
+  while (strncmp(tp, path, n) && (tp = strchr(tp, PTHD)) != NULL && *++tp);
+  if (tp == NULL || !*tp || !*(tp += n)) return(NULL);
+  FREE(path);
+ }
+ for (n = 0; tp[n] && tp[n] != PTHD; n++);
+ if (!(path = MALLOC((strlen(file) + n + 2) * sizeof(char)))) return(NULL);
+ strncpy(path, tp, n);
+ if (n > 1) path[n++] = DIRC;
+ strcpy(path + n, file);
+ return(path);
 }
 
 int e_read_info(char *str, FENSTER *f, char *file)
@@ -1181,7 +1281,7 @@ int e_read_info(char *str, FENSTER *f, char *file)
    while(e_i_fgets(tstr, 256, fp))
    {  for(i = 0; tstr[i]; i++)
       if(tstr[i] == IFE)  {  e_i_fclose(fp);  return(0);  }
-      if(bsw == 1) 
+      if(bsw == 1)
       {  bsw = e_mk_info_button(tstr);
          for(ptmp = tstr; (ptmp = WpeStrcstr(ptmp, "*note")); ptmp += 5)
             bsw = e_mk_info_button(ptmp+5);
@@ -1353,11 +1453,14 @@ int e_hp_ret(FENSTER *f)
 /* Give a context-sensitive help for the identifier under cursor */
 int e_topic_search(FENSTER *f)
 {
- int x,y;
+ int x, y;
  char *s;
- BUFFER *b = f->ed->f[f->ed->mxedt]->b;
- char item[100],*ptr=item;
+ BUFFER *b;
+ char item[100], *ptr=item;
 
+ if (!DTMD_ISTEXT(f->ed->f[f->ed->mxedt]->dtmd))
+  return(0);
+ b = f->ed->f[f->ed->mxedt]->b;
  y=b->b.y;
  x=b->b.x;
  s=b->bf[y].s;
