@@ -34,8 +34,8 @@ extern int col_num;
 extern FARBE *u_fb, *x_fb;
 
 /* globals */
+struct CNT *WpeEditor;
 int fk__cursor = 0;
-struct EXT uhr, h_error;
 
 WOPT *eblst, *fblst, *mblst, *dblst, *xblst, *wblst, *rblst;
 WOPT *ablst, *sblst, *hblst, *gblst, *oblst;
@@ -44,10 +44,6 @@ char *e_hlp, *user_shell;
 WOPT *blst;
 int nblst = 7;
 
-#ifdef UNIX
-int e_term_def = 0;
-#endif
-
 #if MOUSE
 int fk__mouse_stat = 1;
 struct mouse e_mouse = {  0, 0, 0  };
@@ -276,15 +272,12 @@ int main(int argc, char **argv)
  int so = 0, sd = 1;
  char *tp;
 
-#ifdef UNIX
  e_ini_unix(&argc, argv);
  e_switch_screen(1);
-#endif
- if (WpeIsXwin()) fb = x_fb;
- else fb = u_fb;
- e_ini_farbe(fb, 0);
+ fb = e_ini_farbe();
  if ((cn = (ECNT *)MALLOC(sizeof(ECNT))) == NULL)
   e_error(e_msg[ERR_LOWMEM], 2, fb);
+ WpeEditor = cn;
  cn->mxedt = -1;
  cn->curedt = 0;
  cn->edt[0] = 0;
@@ -296,17 +289,16 @@ int main(int argc, char **argv)
  strcpy(cn->fd.search, "");
  strcpy(cn->fd.replace, "");
  strcpy(cn->fd.file, SUDIR);
- strcpy(cn->fd.dirct, cn->dirct);
+ cn->fd.dirct = WpeStrdup(cn->dirct);
 
  cn->fd.sw = 16;
  cn->fd.sn = 0;
  cn->fd.rn = 0;
 
  cn->sdf = cn->rdf = cn->fdf = cn->ddf = cn->wdf = cn->hdf = cn->shdf = NULL;
- uhr.cn = h_error.cn = cn;
 
  /*   standard adjustments    */
- cn->dtmd = 'n';
+ cn->dtmd = DTMD_NORMAL;
  cn->autosv = 0;
  cn->maxchg = 999;
  cn->numundo = 10;
@@ -323,10 +315,18 @@ int main(int argc, char **argv)
  e_read_help_str();
  e_hlp = e_hlp_str[0];
  if (!(user_shell = getenv("SHELL"))) user_shell = DEF_SHELL;
+#ifdef HAVE_TEMPNAM
+ e_tmp_dir = tempnam(NULL, "xwpe_");
+#else
  e_tmp_dir = MALLOC(128);
  sprintf(e_tmp_dir, "/tmp/we_%u", (unsigned) getpid());
  e_tmp_dir = REALLOC(e_tmp_dir, (strlen(e_tmp_dir)+1)*sizeof(char));
- mkdir(e_tmp_dir, 0700);
+#endif
+ if ((e_tmp_dir == NULL) || (mkdir(e_tmp_dir, 0700) != 0))
+ {
+  perror("Xwpe: ");
+  return 1;
+ }
 
  for (i = 1; i < argc; i++)
  {
@@ -408,8 +408,8 @@ int main(int argc, char **argv)
  if (cn->mxedt == 0) WpeManager(cn->f[cn->mxedt]);
  do
  {
-  if (cn->f[cn->mxedt]->dtmd == 'F') i = WpeHandleFileManager(cn);
-  else if (cn->f[cn->mxedt]->dtmd == 'D') i = e_data_eingabe(cn);
+  if (cn->f[cn->mxedt]->dtmd == DTMD_FILEMANAGER) i = WpeHandleFileManager(cn);
+  else if (cn->f[cn->mxedt]->dtmd == DTMD_DATA) i = e_data_eingabe(cn);
   else i = e_eingabe(cn);
   if (i == AltX) i = e_quit(cn->f[cn->mxedt]);
  }
@@ -495,10 +495,12 @@ void e_ini_desk(ECNT *cn)
  e_pr_uul(cn->fb);
 }
 
-void e_ini_farbe(FARBE *fb, int sw)
+FARBE *e_ini_farbe()
 {
- if (!sw || (WpeIsXwin()))
+ if (WpeIsXwin())
  {
+  if (!x_fb)
+   x_fb = WpeMalloc(sizeof(FARBE));
   x_fb->er = e_s_x_clr(15,1);   /*  Editor Frames         */
   x_fb->et = e_s_x_clr(14,1);   /*  Editor Text           */
   x_fb->ez = e_s_x_clr(1,7);    /*  Editors Text highlighted  */
@@ -538,9 +540,12 @@ void e_ini_farbe(FARBE *fb, int sw)
   x_fb->cc = e_s_x_clr( 7, 1);  /*  C-Prog. Comments                                             */
   x_fb->dc = 0x02;
   x_fb->ws = 7;
+  return x_fb;
  }
- if (!sw || !(WpeIsXwin()))
+ else
  {
+  if (!u_fb)
+   u_fb = WpeMalloc(sizeof(FARBE));
   if (col_num)
   {
    u_fb->er = e_s_x_clr(15,4);  /*  Editor Frames         */
@@ -625,6 +630,16 @@ void e_ini_farbe(FARBE *fb, int sw)
    u_fb->dc = 0x20;
    u_fb->ws = 0;
   }
+  return u_fb;
+ }
+}
+
+void e_free_find(FIND *fd)
+{
+ if (fd->dirct)
+ {
+  free(fd->dirct);
+  fd->dirct = NULL;
  }
 }
 
