@@ -132,93 +132,102 @@ int e_clear_desk(FENSTER *f)
 /*    redraw everything */
 int e_repaint_desk(FENSTER *f)
 {
-   /* int j; */
-   ECNT *cn = f->ed;
-   int i, g[4];
+ /* int j; */
+ ECNT *cn = f->ed;
+ int i, g[4];
 #ifndef NO_XWINDOWS
-   extern PIC *e_X_l_pic;
-   PIC *sv_pic = NULL, *nw_pic = NULL;
-   if (WpeIsXwin())
-   {  if(e_X_l_pic && e_X_l_pic != cn->f[cn->mxedt]->pic)
-      {  sv_pic = e_X_l_pic;
-         nw_pic = e_open_view(e_X_l_pic->a.x, e_X_l_pic->a.y,
-            e_X_l_pic->e.x, e_X_l_pic->e.y, 0, 2);
-      }
-      (*e_u_ini_size)();
-   }
+ extern PIC *e_X_l_pic;
+ PIC *sv_pic = NULL, *nw_pic = NULL;
+
+ if (WpeIsXwin())
+ {
+  if (e_X_l_pic && e_X_l_pic != cn->f[cn->mxedt]->pic)
+  {
+   sv_pic = e_X_l_pic;
+   nw_pic = e_open_view(e_X_l_pic->a.x, e_X_l_pic->a.y,
+     e_X_l_pic->e.x, e_X_l_pic->e.y, 0, 2);
+  }
+  (*e_u_ini_size)();
+ }
 #endif
-   if(cn->mxedt < 1)
-   {  e_cls(f->fb->df.fb, f->fb->dc);
-      e_ini_desk(f->ed);
+ if (cn->mxedt < 1)
+ {
+  e_cls(f->fb->df.fb, f->fb->dc);
+  e_ini_desk(f->ed);
 #ifndef NO_XWINDOWS
-      if((WpeIsXwin()) && nw_pic)
-      {  e_close_view(nw_pic, 1);
-	 e_X_l_pic = sv_pic;
-      }
+  if ((WpeIsXwin()) && nw_pic)
+  {
+   e_close_view(nw_pic, 1);
+   e_X_l_pic = sv_pic;
+  }
 #endif
-      return(0);
-   }
-   cn->curedt = cn->mxedt;
-   ini_repaint(cn);
-   e_abs_refr();
-   for ( i = 1; i < cn->mxedt; i++)
-   {  e_firstl(cn->f[i], 0);
-      e_schirm(cn->f[i], 0);
-   }
-   e_firstl(cn->f[i], 1);
-   e_schirm(cn->f[i], 1);
+  return(0);
+ }
+ cn->curedt = cn->mxedt;
+ ini_repaint(cn);
+ e_abs_refr();
+ for (i = 1; i < cn->mxedt; i++)
+ {
+  e_firstl(cn->f[i], 0);
+  e_schirm(cn->f[i], 0);
+ }
+ e_firstl(cn->f[i], 1);
+ e_schirm(cn->f[i], 1);
 #ifndef NO_XWINDOWS
-   if(WpeIsXwin() && nw_pic)
-   {  e_close_view(nw_pic, 1);
-      e_X_l_pic = sv_pic;
-   }
+ if (WpeIsXwin() && nw_pic)
+ {
+  e_close_view(nw_pic, 1);
+  e_X_l_pic = sv_pic;
+ }
 #endif
 #if  MOUSE
-   g[0] = 2; fk_mouse(g);
+ g[0] = 2; fk_mouse(g);
 #endif
-   end_repaint();
-   e_cursor(cn->f[i], 1);
+ end_repaint();
+ e_cursor(cn->f[i], 1);
 #if  MOUSE
-   g[0] = 0; fk_mouse(g);
-   g[0] = 1; fk_mouse(g);
+ g[0] = 0; fk_mouse(g);
+ g[0] = 1; fk_mouse(g);
 #endif
-   return(0);
+ return(0);
 }
 
 /*    write system information   */
 int e_sys_info(FENSTER *f)
 {
-   PIC *pic = NULL;
-   char tmp[80];
-   int xa = 10, ya = 5, xe = xa + 60, ye = ya + 8;
-   fk_cursor(0);
-   pic = e_std_kst(xa, ya, xe, ye, " Information ", 1, f->fb->nr.fb, f->fb->nt.fb, f->fb->ne.fb);
-   if(pic == NULL) {  e_error(e_msg[ERR_LOWMEM], 1, f->fb); return(WPE_ESC);  }
-   e_pr_str(xa+3, ya+2, " Current File: ", f->fb->nt.fb, 0, 0, 0, 0);
-   e_pr_str(xa+3, ya+4, " Current Directory: ", f->fb->nt.fb, 0, 0, 0, 0);
-   e_pr_str(xa+3, ya+6, " Number of Files: ", f->fb->nt.fb, 0, 0, 0, 0);
-   if(strcmp(f->datnam, "Clipboard") != 0)
-   if(strcmp(f->dirct, f->ed->dirct) == 0)
-   e_pr_str(xa+23, ya+2, f->datnam, f->fb->nt.fb, 0, 0, 0, 0);
-   else
-   {  strcpy(tmp, f->dirct);
-      strcat(tmp, DIRS);
-      strcat(tmp, f->datnam);
-      e_pr_str(xa+23, ya+2, tmp, f->fb->nt.fb, 0, 0, 0, 0);
-   }
-   e_pr_str(xa+23, ya+4, f->ed->dirct, f->fb->nt.fb, 0, 0, 0, 0);
-   e_pr_str(xa+23, ya+6,
-      WpeNumberToString(f->ed->mxedt, WpeNumberOfPlaces(f->ed->mxedt), tmp),
-      f->fb->nt.fb, 0, 0, 0, 0);
+ PIC *pic = NULL;
+ char tmp[80];
+ int xa = 10, ya = 5, xe = xa + 60, ye = ya + 8;
+
+ fk_cursor(0);
+ pic = e_std_kst(xa, ya, xe, ye, " Information ", 1, f->fb->nr.fb, f->fb->nt.fb, f->fb->ne.fb);
+ if (pic == NULL) {  e_error(e_msg[ERR_LOWMEM], 1, f->fb); return(WPE_ESC);  }
+ e_pr_str(xa+3, ya+2, " Current File: ", f->fb->nt.fb, 0, 0, 0, 0);
+ e_pr_str(xa+3, ya+4, " Current Directory: ", f->fb->nt.fb, 0, 0, 0, 0);
+ e_pr_str(xa+3, ya+6, " Number of Files: ", f->fb->nt.fb, 0, 0, 0, 0);
+ if(strcmp(f->datnam, "Clipboard") != 0)
+ if(strcmp(f->dirct, f->ed->dirct) == 0)
+  e_pr_str(xa+23, ya+2, f->datnam, f->fb->nt.fb, 0, 0, 0, 0);
+ else
+ {
+  strcpy(tmp, f->dirct);
+  strcat(tmp, DIRS);
+  strcat(tmp, f->datnam);
+  e_pr_str(xa+23, ya+2, tmp, f->fb->nt.fb, 0, 0, 0, 0);
+ }
+ e_pr_str(xa+23, ya+4, f->ed->dirct, f->fb->nt.fb, 0, 0, 0, 0);
+ e_pr_str(xa+23, ya+6,
+   WpeNumberToString(f->ed->mxedt, WpeNumberOfPlaces(f->ed->mxedt), tmp),
+   f->fb->nt.fb, 0, 0, 0, 0);
 #if  MOUSE
-   while(e_mshit() != 0);
-   e_getch();
-   while(e_mshit() != 0);
+ while(e_mshit() != 0);
+ e_getch();
+ while(e_mshit() != 0);
 #else
-   e_getch();
+ e_getch();
 #endif
-   e_close_view(pic, 1);
-   return(0);
+ e_close_view(pic, 1);
+ return(0);
 }
 
 /*   color adjustments  */
@@ -368,172 +377,193 @@ void e_pr_dif_colors(int sw, int xa, int ya, FENSTER *f, int sw2, int md)
 /*    color menu  */
 int e_frb_x_menue(int sw, int xa, int ya, FENSTER *f, int md)
 {
-   COLOR *frb = &(f->fb->er);
-   int c = 1, fsv = frb[sw].fb, x, y;
-   if(md == 1) sw += 11;
-   else if(md == 2) sw += 16;
-   else if(md == 3) sw += 32;
-   y = frb[sw].b;
-   x = frb[sw].f;
-   do
-   {  if(c == CRI && y < 7) y++;
-      else if(c == CLE && y > 0) y--;
-      else if(c == CUP && x > 0) x--;
-      else if(c == CDO && x < 15) x++;
-      else if(c >= 1000 && c < 1256)
-      {  x = (c-1000)/16;  y = (c-1000)%16;  }
-      e_pr_x_col_kasten(xa, ya, x, y, f, 1);
-      frb[sw] = e_s_clr(x, y);
-      e_pr_ed_beispiel(1, 2, f, sw, md);
+ COLOR *frb = &(f->fb->er);
+ int c = 1, fsv = frb[sw].fb, x, y;
+
+ if (md == 1) sw += 11;
+ else if (md == 2) sw += 16;
+ else if (md == 3) sw += 32;
+ y = frb[sw].b;
+ x = frb[sw].f;
+ do
+ {
+  if (c == CRI && y < 7) y++;
+  else if (c == CLE && y > 0) y--;
+  else if (c == CUP && x > 0) x--;
+  else if (c == CDO && x < 15) x++;
+  else if (c >= 1000 && c < 1256)
+  {
+   x = (c-1000)/16;
+   y = (c-1000)%16;
+  }
+  e_pr_x_col_kasten(xa, ya, x, y, f, 1);
+  frb[sw] = e_s_clr(x, y);
+  e_pr_ed_beispiel(1, 2, f, sw, md);
 #if  MOUSE
-      if((c=e_getch()) == -1) c = e_opt_ck_mouse(xa, ya, md);
+  if ((c=e_getch()) == -1) c = e_opt_ck_mouse(xa, ya, md);
 #else
-      c = e_getch();
+  c = e_getch();
 #endif
-   }
-   while (c != WPE_ESC && c != WPE_CR && c > -2);
-   if(c == WPE_ESC || c < -1) frb[sw] = e_n_clr(fsv);
-   return(frb[sw].fb);
+ } while (c != WPE_ESC && c != WPE_CR && c > -2);
+ if (c == WPE_ESC || c < -1)
+  frb[sw] = e_n_clr(fsv);
+ return(frb[sw].fb);
 }
 
 /*   draw color box  */
 void e_pr_x_col_kasten(int xa, int ya, int x, int y, FENSTER *f, int sw)
 {
-   int i, j, rfrb, ffrb, xe = xa + 25, ye = ya + 18;
-   rfrb = sw == 0 ? f->fb->nt.fb : f->fb->fs.fb;
-   ffrb = rfrb % 16;
-   e_std_rahmen(xa-2, ya-1, xe, ye, "Colors", 0, rfrb, 0);
+ int i, j, rfrb, ffrb, xe = xa + 25, ye = ya + 18;
+
+ rfrb = sw == 0 ? f->fb->nt.fb : f->fb->fs.fb;
+ ffrb = rfrb % 16;
+ e_std_rahmen(xa-2, ya-1, xe, ye, "Colors", 0, rfrb, 0);
 /*     e_pr_str((xa+xe-8)/2, ya-1, "Colors", rfrb, 0, 1, 
                                         f->fb->ms.f+16*(rfrb/16), 0);
 */
-   for (j = 0; j < 8; j++)
-   for (i = 0; i < 16; i++)
-   {  e_pr_char(3*j+xa, i+ya+1, ' ', 16*j+i);
-      e_pr_char(3*j+xa+1, i+ya+1, 'x', 16*j+i);
-      e_pr_char(3*j+xa+2, i+ya+1, ' ', 16*j+i);
-   }
-   for (i = 0; i < 18; i++)
-   {  e_pr_char(xa-1, i+ya, ' ', rfrb);
-      e_pr_char(xe-1, i+ya, ' ', rfrb);
-   }
-   for (j = 0; j < 25; j++)
-   {  e_pr_char(j+xa-1, ya, ' ', rfrb);
-      e_pr_char(j+xa-1, ye-1, ' ', rfrb);
-   }
+ for (j = 0; j < 8; j++)
+  for (i = 0; i < 16; i++)
+  {
+   e_pr_char(3*j+xa, i+ya+1, ' ', 16*j+i);
+   e_pr_char(3*j+xa+1, i+ya+1, 'x', 16*j+i);
+   e_pr_char(3*j+xa+2, i+ya+1, ' ', 16*j+i);
+  }
+ for (i = 0; i < 18; i++)
+ {
+  e_pr_char(xa-1, i+ya, ' ', rfrb);
+  e_pr_char(xe-1, i+ya, ' ', rfrb);
+ }
+ for (j = 0; j < 25; j++)
+ {
+  e_pr_char(j+xa-1, ya, ' ', rfrb);
+  e_pr_char(j+xa-1, ye-1, ' ', rfrb);
+ }
 #ifdef NEWSTYLE
-   if(!WpeIsXwin()) {
+ if (!WpeIsXwin())
+ {
 #endif
-      for (i = 0; i < 3; i++)
-      {  e_pr_char(3*y+xa+i, x+ya, RE5, x>0 ? 16*y+ffrb : rfrb );
-	 e_pr_char(3*y+xa+i, x+ya+2, RE5, x<15 ? 16*y+ffrb : rfrb );
-      }
-      e_pr_char(3*y+xa-1, x+ya+1, RE6, y<1 ? rfrb  : 16*(y-1)+ffrb);
-      e_pr_char(3*y+xa+3, x+ya+1, RE6, y>6 ? rfrb  : 16*(y+1)+ffrb);
-      e_pr_char(3*y+xa-1, x+ya, RE1, (y<1 || x<1) ? rfrb  : 16*(y-1)+ffrb);
-      e_pr_char(3*y+xa+3, x+ya, RE2, (y>6 || x<1) ? rfrb  : 16*(y+1)+ffrb);
-      e_pr_char(3*y+xa-1, x+ya+2, RE3, (y<1 || x>14) ? rfrb  : 16*(y-1)+ffrb);
-      e_pr_char(3*y+xa+3, x+ya+2, RE4, (y>6 || x>14) ? rfrb  : 16*(y+1)+ffrb);
+  for (i = 0; i < 3; i++)
+  {
+   e_pr_char(3*y+xa+i, x+ya, RE5, x>0 ? 16*y+ffrb : rfrb );
+   e_pr_char(3*y+xa+i, x+ya+2, RE5, x<15 ? 16*y+ffrb : rfrb );
+  }
+  e_pr_char(3*y+xa-1, x+ya+1, RE6, y<1 ? rfrb  : 16*(y-1)+ffrb);
+  e_pr_char(3*y+xa+3, x+ya+1, RE6, y>6 ? rfrb  : 16*(y+1)+ffrb);
+  e_pr_char(3*y+xa-1, x+ya, RE1, (y<1 || x<1) ? rfrb  : 16*(y-1)+ffrb);
+  e_pr_char(3*y+xa+3, x+ya, RE2, (y>6 || x<1) ? rfrb  : 16*(y+1)+ffrb);
+  e_pr_char(3*y+xa-1, x+ya+2, RE3, (y<1 || x>14) ? rfrb  : 16*(y-1)+ffrb);
+  e_pr_char(3*y+xa+3, x+ya+2, RE4, (y>6 || x>14) ? rfrb  : 16*(y+1)+ffrb);
 #ifdef NEWSTYLE
-   }
-   else e_make_xrect(3*y+xa, x+ya+1, 3*y+xa+2, x+ya+1, 1);
+ }
+ else e_make_xrect(3*y+xa, x+ya+1, 3*y+xa+2, x+ya+1, 1);
 #endif
 }
 
 /*    draw color example   */
 void e_pr_ed_beispiel(int xa, int ya, FENSTER *f, int sw, int md)
 {
-   COLOR *frb = &(f->fb->er);
-   FARBE *fb = f->fb;
-   int i, j, xe = xa+31, ye = ya+19;
+ COLOR *frb = &(f->fb->er);
+ FARBE *fb = f->fb;
+ int i, j, xe = xa+31, ye = ya+19;
 
-   frb[sw] = e_s_clr(frb[sw].f, frb[sw].b);
-   if(md == 1)
-   {  e_blk(xe-xa+1, xa+1, ya, fb->mt.fb);
-      e_pr_str_wsd(xa+5, ya,  "Edit", fb->mt.fb, 0, 1, f->fb->ms.fb,
-         xa+3, xa+11);
-      e_pr_str_wsd(xa+18, ya,  "Options", fb->mz.fb, 0, 0, f->fb->ms.fb,
-         xa+16, xa+27);
-      for(i = ya+1; i < ye; i++)
-         for(j = xa+1; j <= xe+1; j++)
-         {  e_pr_char(j, i, fb->dc, fb->df.fb);
-            e_pr_char(j, i, fb->dc, fb->df.fb);
-         }
-      e_std_rahmen(xa+17, ya+1, xa+26, ya+3, NULL, 0, fb->mr.fb, 0);
-      e_pr_str(xa+19, ya+2, "Colors", fb->mt.fb, 0, 1, fb->ms.fb, 0);
-      e_blk(xe-xa+1, xa+1, ye, fb->mt.fb);
-      e_pr_str_wsd(xa+4, ye, "Alt-F3 Close Window",
-         fb->mt.fb, 0, 6, fb->ms.fb, xa+2, xa+25);
+ frb[sw] = e_s_clr(frb[sw].f, frb[sw].b);
+ if (md == 1)
+ {
+  e_blk(xe-xa+1, xa+1, ya, fb->mt.fb);
+  e_pr_str_wsd(xa+5, ya,  "Edit", fb->mt.fb, 0, 1, f->fb->ms.fb,
+    xa+3, xa+11);
+  e_pr_str_wsd(xa+18, ya,  "Options", fb->mz.fb, 0, 0, f->fb->ms.fb,
+    xa+16, xa+27);
+  for (i = ya+1; i < ye; i++)
+   for (j = xa+1; j <= xe+1; j++)
+   {
+    e_pr_char(j, i, fb->dc, fb->df.fb);
+    e_pr_char(j, i, fb->dc, fb->df.fb);
    }
-   else if(md == 2)
-   {  e_std_rahmen(xa, ya, xe, ye, "Message", 1, fb->nr.fb, f->fb->ne.fb);
-      for(i = ya+1; i < ye; i++) e_blk(xe-xa-1, xa+1, i, fb->nt.fb);
-      e_pr_str(xa+4, ya+2, "Name:", f->fb->nt.fb, 0, 1,
-         f->fb->nsnt.fb, f->fb->nt.fb);
-      e_pr_str(xa+5, ya+3, "Active Write-Line ", fb->fa.fb, 0, 0, 0, 0);
-      e_pr_str(xa+4, ya+5, "Name:", f->fb->nt.fb, 0, 1,
-         f->fb->nsnt.fb, f->fb->nt.fb);
-      e_pr_str(xa+5, ya+6, "Passive Write-Line", fb->fr.fb, 0, 0, 0, 0);
-      e_pr_str(xa+4, ya+8, "Data:", f->fb->nt.fb, 0, 1,
-         f->fb->nsnt.fb, f->fb->nt.fb);
-      e_pr_str(xa+5, ya+9, "Active Marked ", f->fb->fz.fb, 0, 0, 0, 0);
-      e_pr_str(xa+5, ya+10, "Passive Marked", f->fb->frft.fb, 0, 0, 0, 0);
-      e_pr_str(xa+5, ya+11, "Data Text     ", f->fb->ft.fb, 0, 0, 0, 0);
-      e_pr_str(xa+4, ya+13, "Switches:", f->fb->nt.fb, 0, 1,
-         f->fb->nsnt.fb, f->fb->nt.fb);
-      e_pr_str(xa+5, ya+14, "[X] Active Switch ", f->fb->fsm.fb, 0, 0, 0, 0);
-      e_pr_str(xa+5, ya+15, "[ ] Passive Switch", f->fb->fs.fb, 4, 1,
-         f->fb->nsft.fb, f->fb->fs.fb);
-      e_pr_str(xa+6 , ye-2, "Button", f->fb->nz.fb, 0, -1,
-         f->fb->ns.fb, f->fb->nt.fb);
-      e_pr_str(xe-12 , ye-2, "Active", f->fb->nm.fb, 0, -1,
-         f->fb->nm.fb, f->fb->nt.fb);
+  e_std_rahmen(xa+17, ya+1, xa+26, ya+3, NULL, 0, fb->mr.fb, 0);
+  e_pr_str(xa+19, ya+2, "Colors", fb->mt.fb, 0, 1, fb->ms.fb, 0);
+  e_blk(xe-xa+1, xa+1, ye, fb->mt.fb);
+  e_pr_str_wsd(xa+4, ye, "Alt-F3 Close Window",
+    fb->mt.fb, 0, 6, fb->ms.fb, xa+2, xa+25);
+ }
+ else if (md == 2)
+ {
+  e_std_rahmen(xa, ya, xe, ye, "Message", 1, fb->nr.fb, f->fb->ne.fb);
+  for(i = ya+1; i < ye; i++) e_blk(xe-xa-1, xa+1, i, fb->nt.fb)
+   ;
+  e_pr_str(xa+4, ya+2, "Name:", f->fb->nt.fb, 0, 1,
+    f->fb->nsnt.fb, f->fb->nt.fb);
+  e_pr_str(xa+5, ya+3, "Active Write-Line ", fb->fa.fb, 0, 0, 0, 0);
+  e_pr_str(xa+4, ya+5, "Name:", f->fb->nt.fb, 0, 1,
+    f->fb->nsnt.fb, f->fb->nt.fb);
+  e_pr_str(xa+5, ya+6, "Passive Write-Line", fb->fr.fb, 0, 0, 0, 0);
+  e_pr_str(xa+4, ya+8, "Data:", f->fb->nt.fb, 0, 1,
+    f->fb->nsnt.fb, f->fb->nt.fb);
+  e_pr_str(xa+5, ya+9, "Active Marked ", f->fb->fz.fb, 0, 0, 0, 0);
+  e_pr_str(xa+5, ya+10, "Passive Marked", f->fb->frft.fb, 0, 0, 0, 0);
+  e_pr_str(xa+5, ya+11, "Data Text     ", f->fb->ft.fb, 0, 0, 0, 0);
+  e_pr_str(xa+4, ya+13, "Switches:", f->fb->nt.fb, 0, 1,
+    f->fb->nsnt.fb, f->fb->nt.fb);
+  e_pr_str(xa+5, ya+14, "[X] Active Switch ", f->fb->fsm.fb, 0, 0, 0, 0);
+  e_pr_str(xa+5, ya+15, "[ ] Passive Switch", f->fb->fs.fb, 4, 1,
+    f->fb->nsft.fb, f->fb->fs.fb);
+  e_pr_str(xa+6 , ye-2, "Button", f->fb->nz.fb, 0, -1,
+    f->fb->ns.fb, f->fb->nt.fb);
+  e_pr_str(xe-12 , ye-2, "Active", f->fb->nm.fb, 0, -1,
+    f->fb->nm.fb, f->fb->nt.fb);
 #ifdef NEWSTYLE
-      if (WpeIsXwin())
-      {  e_make_xrect(xa+4, ya+3, xa+23, ya+3, 1);
-         e_make_xrect(xa+4, ya+6, xa+23, ya+6, 1);
-         e_make_xrect(xa+4, ya+9, xa+19, ya+11, 1);
-         e_make_xrect_abs(xa+4, ya+9, xa+19, ya+9, 0);
-         e_make_xrect(xa+4, ya+14, xa+23, ya+15, 1);
-         e_make_xrect_abs(xa+4, ya+14, xa+23, ya+14, 0);
-      }
+  if (WpeIsXwin())
+  {
+   e_make_xrect(xa+4, ya+3, xa+23, ya+3, 1);
+   e_make_xrect(xa+4, ya+6, xa+23, ya+6, 1);
+   e_make_xrect(xa+4, ya+9, xa+19, ya+11, 1);
+   e_make_xrect_abs(xa+4, ya+9, xa+19, ya+9, 0);
+   e_make_xrect(xa+4, ya+14, xa+23, ya+15, 1);
+   e_make_xrect_abs(xa+4, ya+14, xa+23, ya+14, 0);
+  }
 #endif
-   }
-   else
-   {  e_std_rahmen(xa, ya, xe, ye, "Filename", 1, fb->er.fb, fb->es.fb);
-      e_mouse_leiste(xe, ya+1, ye-ya-1, 0, fb->em.fb);
-      e_mouse_leiste(xa+20, ye, 11, 1, fb->em.fb);
-      e_pr_char(xe-3, ya, WZN, fb->es.fb);
+ }
+ else
+ {
+  e_std_rahmen(xa, ya, xe, ye, "Filename", 1, fb->er.fb, fb->es.fb);
+  e_mouse_bar(xe, ya+1, ye-ya-1, 0, fb->em.fb);
+  e_mouse_bar(xa+20, ye, 11, 1, fb->em.fb);
+  e_pr_char(xe-3, ya, WZN, fb->es.fb);
 #ifdef NEWSTYLE
-      if(!WpeIsXwin()) {
+  if (!WpeIsXwin())
+  {
 #endif
-	 e_pr_char(xe-4, ya, '[', fb->er.fb);
-	 e_pr_char(xe-2, ya, ']', fb->er.fb);
+   e_pr_char(xe-4, ya, '[', fb->er.fb);
+   e_pr_char(xe-2, ya, ']', fb->er.fb);
 #ifdef NEWSTYLE
-      }  else e_make_xrect(xe-4, ya, xe-2, ya, 0);
+  }
+  else e_make_xrect(xe-4, ya, xe-2, ya, 0);
 #endif
-      for(i = ya+1; i < ye; i++) e_blk(xe-xa-1, xa+1, i, fb->et.fb);
-      if(md == 3)
-      {  e_pr_str(xa+4, ya+3, "#Preprozessor Comands", fb->cp.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+5, "This are C-Text Colors", fb->ct.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+7, "int char {} [] ; ,", fb->cr.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+9, "\"Constants\" 12 0x13", fb->ck.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+11, "/*   Comments    */", fb->cc.fb, 0, 0, 0, 0);
-      }
-      else
-      {  e_pr_str(xa+4, ya+3, "This are the Editor Colors", fb->et.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+4, "And this is a marked Line", fb->ez.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+5, "This is a found word", fb->et.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+14, ya+5, "found", fb->ek.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+8, "Help Header", fb->hh.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+9, "This is a marked Word", fb->et.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+14, ya+9, "marked", fb->hm.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+10, "in the Help File", fb->et.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+11, "Help Button", fb->hb.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+14, "This is a Breakpoint", fb->db.fb, 0, 0, 0, 0);
-	 e_pr_str(xa+4, ya+15, "Stop at Breakpoint", fb->dy.fb, 0, 0, 0, 0);
-      }
-   }
-   frb[sw] = e_s_clr(frb[sw].f, frb[sw].b);
+  for (i = ya+1; i < ye; i++) e_blk(xe-xa-1, xa+1, i, fb->et.fb);
+  if (md == 3)
+  {
+   e_pr_str(xa+4, ya+3, "#Preprozessor Comands", fb->cp.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+5, "This are C-Text Colors", fb->ct.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+7, "int char {} [] ; ,", fb->cr.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+9, "\"Constants\" 12 0x13", fb->ck.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+11, "/*   Comments    */", fb->cc.fb, 0, 0, 0, 0);
+  }
+  else
+  {
+   e_pr_str(xa+4, ya+3, "This are the Editor Colors", fb->et.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+4, "And this is a marked Line", fb->ez.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+5, "This is a found word", fb->et.fb, 0, 0, 0, 0);
+   e_pr_str(xa+14, ya+5, "found", fb->ek.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+8, "Help Header", fb->hh.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+9, "This is a marked Word", fb->et.fb, 0, 0, 0, 0);
+   e_pr_str(xa+14, ya+9, "marked", fb->hm.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+10, "in the Help File", fb->et.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+11, "Help Button", fb->hb.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+14, "This is a Breakpoint", fb->db.fb, 0, 0, 0, 0);
+   e_pr_str(xa+4, ya+15, "Stop at Breakpoint", fb->dy.fb, 0, 0, 0, 0);
+  }
+ }
+ frb[sw] = e_s_clr(frb[sw].f, frb[sw].b);
 }
 
 /*   Save - Options - Menu   */
@@ -648,9 +678,17 @@ int WpeReadColor(ECNT *cn, char *section, char *option, char *value)
  COLOR *c = NULL;
 
  if (WpeStrccmp("Term", section + strlen(OPT_SECTION_COLOR) + 1) == 0)
+ {
+  if (!u_fb)
+   u_fb = WpeMalloc(sizeof(FARBE));
   fb = u_fb;
+ }
  if (WpeStrccmp("X11", section + strlen(OPT_SECTION_COLOR) + 1) == 0)
+ {
+  if (!x_fb)
+   x_fb = WpeMalloc(sizeof(FARBE));
   fb = x_fb;
+ }
  if (WpeStrccmp("er", option) == 0)
   c = &fb->er;
  else if (WpeStrccmp("es", option) == 0)
@@ -939,12 +977,18 @@ int e_save_opt(FENSTER *f)
  WpeWriteGeneral(cn, str_line, fp);
  str_line = MALLOC(strlen(OPT_SECTION_COLOR) + 10);
  strcpy(str_line, OPT_SECTION_COLOR);
- strcat(str_line, "/Term");
- fprintf(fp, "[%s]\n", str_line);
- WpeWriteColor(cn, str_line, fp);
- strcpy(str_line + strlen(OPT_SECTION_COLOR), "/X11");
- fprintf(fp, "[%s]\n", str_line);
- WpeWriteColor(cn, str_line, fp);
+ if (u_fb)
+ {
+  strcat(str_line, "/Term");
+  fprintf(fp, "[%s]\n", str_line);
+  WpeWriteColor(cn, str_line, fp);
+ }
+ if (x_fb)
+ {
+  strcpy(str_line + strlen(OPT_SECTION_COLOR), "/X11");
+  fprintf(fp, "[%s]\n", str_line);
+  WpeWriteColor(cn, str_line, fp);
+ }
  FREE(str_line);
  str_line = OPT_SECTION_PROGRAMMING;
  fprintf(fp, "[%s]\n", str_line);
@@ -1709,7 +1753,7 @@ int e_opt_kst(W_OPTSTR *o)
 
 int e_edt_options(FENSTER *f)
 {
- int i, ret, mdsv = f->ed->dtmd, edopt = f->ed->edopt;
+ int i, ret, edopt = f->ed->edopt;
  W_OPTSTR *o = e_init_opt_kst(f);
 
  if (!o) return(-1);
@@ -1727,7 +1771,7 @@ int e_edt_options(FENSTER *f)
  e_add_numstr(3, 10, 19, 10, 3, 1000, 2, AltX, "MaX. Changes:", f->ed->maxchg, o);
  e_add_numstr(3, 11, 20, 11, 2, 100, 0, AltN, "Num. Undo:", f->ed->numundo, o);
  e_add_numstr(3, 12, 20, 12, 2, 100, 5, AltI, "Auto Ind. Col.:", f->ed->autoindent, o);
- e_add_sswstr(4, 3, 0, AltS, f->ed->dtmd == 's' ? 1 : 0, "Show Endmark ", o);
+ e_add_sswstr(4, 3, 0, AltS, f->ed->edopt & ED_SHOW_ENDMARKS ? 1 : 0, "Show Endmark ", o);
  e_add_sswstr(26, 3, 1, AltP, f->ed->autosv & 1, "OPtions         ", o);
  e_add_sswstr(26, 4, 1, AltH, f->ed->autosv & 2 ? 1 : 0, "CHanges         ", o);
  e_add_sswstr(4, 6, 2, AltD, f->ed->edopt & ED_OLD_TILE_METHOD ? 1 : 0, "OlD Style    ", o);
@@ -1745,18 +1789,17 @@ int e_edt_options(FENSTER *f)
  ret = e_opt_kst(o);
  if (ret != WPE_ESC)
  {
-  if (o->sstr[0]->num) f->ed->dtmd = 's';
-  else f->ed->dtmd = 'n';
   f->ed->autosv = o->sstr[1]->num + (o->sstr[2]->num << 1);
   f->ed->maxcol = o->nstr[0]->num;
   f->ed->tabn = o->nstr[1]->num;
   f->ed->maxchg = o->nstr[2]->num;
   f->ed->numundo = o->nstr[3]->num;
   f->ed->autoindent = o->nstr[4]->num;
-  f->ed->edopt = ((f->ed->edopt & ~389) + o->pstr[0]->num) +
+  f->ed->edopt = ((f->ed->edopt & ~ED_EDITOR_OPTIONS) + o->pstr[0]->num) +
     (o->pstr[1]->num == 0 ? ED_SOURCE_AUTO_INDENT : 0) +
     (o->pstr[1]->num == 1 ? ED_ALWAYS_AUTO_INDENT : 0) +
-    (o->sstr[3]->num ? ED_OLD_TILE_METHOD : 0);
+    (o->sstr[3]->num ? ED_OLD_TILE_METHOD : 0) +
+    (o->sstr[0]->num ? ED_SHOW_ENDMARKS : 0);
   if (f->ed->print_cmd)
    WpeFree(f->ed->print_cmd);
   f->ed->print_cmd = WpeStrdup(o->wstr[0]->txt);
@@ -1771,13 +1814,6 @@ int e_edt_options(FENSTER *f)
      f->ed->f[i]->flg = 0;
    e_repaint_desk(f);
   }
-  if (f->ed->dtmd != mdsv)
-  {
-   for (i = 0; i <= f->ed->mxedt; i++)
-    if (f->ed->f[i]->dtmd == mdsv)
-     f->ed->f[i]->dtmd = f->ed->dtmd;
-   e_repaint_desk(f);
-  }
  }
  freeostr(o);
  return(0);
