@@ -45,7 +45,6 @@ extern char *e_tmp_dir;
 #ifdef DEBUGGER
 extern int e_d_swtch;
 #endif
-extern int (*e_u_system)(char *exe);
 
 /********************************************************/   
 /**** defs for breakpoints and watches manipulations ****/
@@ -804,47 +803,64 @@ int e_check_c_file(char *name)
 
 int e_check_header(char *file, M_TIME otime, ECNT *cn, int sw)
 {
-   struct stat cbuf[1];
-   FILE *fp;
-   char *p, str[120], str2[120];
-   int i;
-   for(i = cn->mxedt; i > 0; i--)
-   {  if(file[0] == DIRC)
-      p = e_mkfilename(cn->f[i]->dirct, cn->f[i]->datnam);
-      else p = cn->f[i]->datnam;
-      if(!strcmp(p, file) && cn->f[i]->save)
-      {  e_save(cn->f[i]);  if(p != cn->f[i]->datnam) FREE(p);  break;  }
-      if(p != cn->f[i]->datnam) FREE(p);
+ struct stat cbuf[1];
+ FILE *fp;
+ char *p, str[120], str2[120];
+ int i;
+
+ for (i = cn->mxedt; i > 0; i--)
+ {
+  if (file[0] == DIRC)
+   p = e_mkfilename(cn->f[i]->dirct, cn->f[i]->datnam);
+  else
+   p = cn->f[i]->datnam;
+  if (!strcmp(p, file) && cn->f[i]->save)
+  {  e_save(cn->f[i]);  if(p != cn->f[i]->datnam) FREE(p);  break;  }
+  if (p != cn->f[i]->datnam)
+   FREE(p);
+ }
+ if ((fp = fopen(file, "r")) == NULL)
+  return(sw);
+ stat(file, cbuf);
+ if (otime < cbuf->st_mtime)
+  sw++;
+ while (fgets(str, 120, fp))
+ {
+  for (p = str; isspace(*p); p++);
+  if (*p == '/' && *(p+1) == '*')
+  {
+   p++;
+   do
+   {
+    for (p++; *p && *p != '*'; p++)
+     ;
+    if (!*p && !fgets((p = str), 120, fp))
+     break;
+   } while (p != NULL && (*p != '*' || *(p+1) != '/'));
+   if (!p) break;
+   for (p += 2; isspace(*p); p++)
+    ;
+  }
+  if (*p == '#')
+  {
+   for (p++; isspace(*p); p++)
+    ;
+   if (!strncmp(p, "include", 7))
+   {
+    for (p += 8; isspace(*p); p++)
+     ;
+    if (*p == '\"')
+    {
+     for (p++, i = 0; p[i] != '\"' && p[i] != '\0' && p[i] != '\n'; i++)
+      str2[i] = p[i];
+     str2[i] = '\0';
+     sw = e_check_header(str2, otime, cn, sw);
+    }
    }
-   if((fp = fopen(file, "r")) == NULL) return(sw);
-   stat(file, cbuf);
-   if(otime < cbuf->st_mtime) sw++;
-   while(fgets(str, 120, fp))
-   {  for(p = str; isspace(*p); p++);
-      if(*p == '/' && *(p+1) == '*')
-      {  p++;
-	 do
-	 {  for(p++; *p && *p != '*'; p++);
-	    if(!*p && !fgets((p = str), 120, fp)) break;
-	 } while (p != NULL && (*p != '*' || *(p+1) != '/'));
-	 if(!p) break;
-	 for(p += 2; isspace(*p); p++);
-      }
-      if(*p == '#')
-      {  for(p++; isspace(*p); p++);
-	 if(!strncmp(p, "include", 7))
-	 {  for(p += 8; isspace(*p); p++);
-	    if(*p == '\"')
-	    {  for(p++, i = 0; p[i] != '\"' && p[i] != '\0'
-				&& p[i] != '\n'; i++)  str2[i] = p[i];
-	       str2[i] = '\0';
-	       sw = e_check_header(str2, otime, cn, sw);
-	    }
-	 }
-      }
-   }
-   fclose(fp);
-   return(sw);
+  }
+ }
+ fclose(fp);
+ return(sw);
 }
 #endif
 
@@ -865,37 +881,38 @@ char *e_cat_string(char *p, char *str)
 
 int e_make_arg(char ***arg, char *str)
 {
-   int i, j;
-   char tmp[128], *p = tmp;
+ int i, j;
+ char tmp[128], *p = tmp;
 
-   if (!(*arg))
-    *arg = (char **) MALLOC(4*sizeof(char *));
-   else
-    *arg = (char **) REALLOC(*arg, 4*sizeof(char *));
-   (*arg)[0] = MALLOC(strlen(e_s_prog.compiler) + 1);
-   strcpy((*arg)[0], e_s_prog.compiler);
-   if (!str)
-   {
-    (*arg)[1] = NULL;
-    (*arg)[2] = NULL;
-    return(2);
-   }
-   strcpy(tmp, str);
-   for (j = 2, i = 0; p[i] != '\0'; j++)
-   {
-    for (; p[i] != '\0' && p[i] != ' '; i++);
-    (*arg)[j] = MALLOC(i + 1);
-    strncpy((*arg)[j], p, i);
-    (*arg)[j][i] = '\0';
-    *arg = (char **) REALLOC(*arg, (j + 3)*sizeof(char *));
-    if (p[i] != '\0')
-    {
-     p += (i + 1);
-     i = 0;
-    }
-   }
-   (*arg)[j] = NULL;
-   return(j);
+ if (!(*arg))
+  *arg = (char **) MALLOC(4*sizeof(char *));
+ else
+  *arg = (char **) REALLOC(*arg, 4*sizeof(char *));
+ (*arg)[0] = MALLOC(strlen(e_s_prog.compiler) + 1);
+ strcpy((*arg)[0], e_s_prog.compiler);
+ if (!str)
+ {
+  (*arg)[1] = NULL;
+  (*arg)[2] = NULL;
+  return(2);
+ }
+ strcpy(tmp, str);
+ for (j = 2, i = 0; p[i] != '\0'; j++)
+ {
+  for (; p[i] != '\0' && p[i] != ' '; i++)
+   ;
+  (*arg)[j] = MALLOC(i + 1);
+  strncpy((*arg)[j], p, i);
+  (*arg)[j][i] = '\0';
+  *arg = (char **) REALLOC(*arg, (j + 3)*sizeof(char *));
+  if (p[i] != '\0')
+  {
+   p += (i + 1);
+   i = 0;
+  }
+ }
+ (*arg)[j] = NULL;
+ return(j);
 }
 
 int e_add_arg(char ***arg, char *str, int n, int argc)
@@ -1273,7 +1290,7 @@ int e_project(FENSTER *f)
  int i;
  if (!e_project_name(f))
  {
-  for (i = cn->mxedt; i > 0 && (cn->f[i]->dtmd != 'D' || cn->f[i]->ins != 4);
+  for (i = cn->mxedt; i > 0 && (cn->f[i]->dtmd != DTMD_DATA || cn->f[i]->ins != 4);
     i--)
    ;
   if (i > 0)
@@ -1295,7 +1312,7 @@ int e_show_project(FENSTER *f)
  ECNT *cn = f->ed;
  int i;
 
- for (i = cn->mxedt; i > 0 && (cn->f[i]->dtmd != 'D' || cn->f[i]->ins != 4);
+ for (i = cn->mxedt; i > 0 && (cn->f[i]->dtmd != DTMD_DATA || cn->f[i]->ins != 4);
    i--)
   ;
  if (i > 0)
@@ -1318,7 +1335,7 @@ int e_cl_project(FENSTER *f)
  else
   e_prog.project = REALLOC(e_prog.project, sizeof(char));
  e_prog.project[0] = '\0';
- for (i = cn->mxedt; i > 0 && (cn->f[i]->dtmd != 'D' || cn->f[i]->ins != 4);
+ for (i = cn->mxedt; i > 0 && (cn->f[i]->dtmd != DTMD_DATA || cn->f[i]->ins != 4);
    i--)
   ;
  if (i > 0)
@@ -1334,7 +1351,7 @@ int e_p_add_item(FENSTER *f)
  ECNT *cn = f->ed;
  int i;
 
- for (i = cn->mxedt; i > 0 && (cn->f[i]->dtmd != 'D' || cn->f[i]->ins != 4);
+ for (i = cn->mxedt; i > 0 && (cn->f[i]->dtmd != DTMD_DATA || cn->f[i]->ins != 4);
    i--)
   ;
  if (i > 0)
@@ -1360,7 +1377,7 @@ int e_p_del_item(FENSTER *f)
  int i;
 
  for (i = cn->mxedt;
-   i > 0 && (cn->f[i]->dtmd != 'D' || cn->f[i]->ins != 4); i--)
+   i > 0 && (cn->f[i]->dtmd != DTMD_DATA || cn->f[i]->ins != 4); i--)
   ;
  if (i > 0)
   e_switch_window(cn->edt[i], cn->f[cn->mxedt]);
@@ -1677,140 +1694,163 @@ char *e_interpr_var(char *string)
 
 char *e_expand_var(char *string, FENSTER *f)
 {
-   int i, j = 0, k, len, kl = 0;
-   char *var = NULL, *v_string, *tmp;
+ int i, j = 0, k, len, kl = 0;
+ char *var = NULL, *v_string, *tmp;
 
-   for(i = 0; string[i]; i++)
-   {  if(string[i] == '\'')
-      {  kl = kl ? 0 : 1;
-	 for(j = i; (string[j] = string[j+1]) != '\0'; j++);
-	 i--;  continue;
-      }
-      if(string[i] == '\\' && (string[i+1] == 'n' || string[i+1] == 'r'))
-      {  string[i] = string[i+1] == 'n' ? '\n' : '\r';
-	 for(j = i+1; (string[j] = string[j+1]) != '\0'; j++);
-	 continue;
-      }
-      if(string[i] == '$' && !kl && (!i || string[i-1] != '\\'))
-      {  if(string[i+1] == '(')
-	 {  for(j = i+2; string[j] && string[j] != ')'; j++);
-	    if(!string[j]) continue;
-	 }
-	 else if(string[i+1] == '{')
-	 {  for(j = i+2; string[j] && string[j] != '}'; j++);
-	    if(!string[j]) continue;
-	 }
-	 if(string[i+1] == '(' || string[i+1] == '{')
-	 {  if(!(var = MALLOC((j-i-1) * sizeof(char))))
-	    {  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(string);  }
-	    for(k = i+2; k < j; k++) var[k-i-2] = string[k];
-	    var[k-i-2] = '\0';
-	 }
-	 else
-	 {  if(!(var = MALLOC(2 * sizeof(char))))
-	    {  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(string);  }
-	    var[0] = string[i+1];  var[1] = '\0';
-	 }
-	 if(!(v_string = getenv(var)))
-	 {  for(k = 0; k < p_v_n - 1; k++)
-	    {  if(!strcmp(p_v[k]->var, var))
-	       {  v_string = p_v[k]->string;  break;  }
-	    }
-	 }
-	 if(string[i+1] == '(' || string[i+1] == '{') len = (j-i+1);
-	 else len = 2;
-	 if(!v_string)
-	 {  for(k = i; (string[k] = string[k+len]) != '\0'; k++);
-	    if(!(string = REALLOC(tmp = string, (strlen(string) + 1)
-							 * sizeof(char))))
-	    {  FREE(var);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(tmp);  }
-	 }
-	 else
-	 {  len = strlen(v_string) - len;
-	    if(len >= 0)
-	    {  if(!(string = REALLOC(tmp = string,
-		      (k = strlen(string) + len + 1) * sizeof(char))))
-	       {  FREE(var);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(tmp);  }
-	       for(k--; k > j + len; k--)string[k] = string[k-len];
-	       for(k = i; v_string[k-i]; k++) string[k] = v_string[k-i];
-	    }
-	    else
-	    {  for(k = i; (string[k] = string[k-len]) != '\0'; k++);
-	       for(k = i; v_string[k-i]; k++) string[k] = v_string[k-i];
-	       if(!(string = REALLOC(tmp = string,
-		      (strlen(string) + 1) * sizeof(char))))
-	       {  FREE(var);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(tmp);  }
-	    }
-	 }
-	 FREE(var);
-      }
+ for (i = 0; string[i]; i++)
+ {
+  if (string[i] == '\'')
+  {
+   kl = kl ? 0 : 1;
+   for (j = i; (string[j] = string[j+1]) != '\0'; j++);
+   i--;
+   continue;
+  }
+  if (string[i] == '\\' && (string[i+1] == 'n' || string[i+1] == 'r'))
+  {
+   string[i] = string[i+1] == 'n' ? '\n' : '\r';
+   for (j = i+1; (string[j] = string[j+1]) != '\0'; j++);
+   continue;
+  }
+  if (string[i] == '$' && !kl && (!i || string[i-1] != '\\'))
+  {
+   if (string[i+1] == '(')
+   {
+    for (j = i+2; string[j] && string[j] != ')'; j++);
+    if (!string[j]) continue;
    }
-   return(string);
+   else if (string[i+1] == '{')
+   {
+    for (j = i+2; string[j] && string[j] != '}'; j++);
+    if (!string[j]) continue;
+   }
+   if (string[i+1] == '(' || string[i+1] == '{')
+   {
+    if (!(var = MALLOC((j-i-1) * sizeof(char))))
+    {
+     e_error(e_msg[ERR_LOWMEM], 0, f->fb);
+     return(string);
+    }
+    for (k = i+2; k < j; k++) var[k-i-2] = string[k];
+    var[k-i-2] = '\0';
+   }
+   else
+   {
+    if (!(var = MALLOC(2 * sizeof(char))))
+    {  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(string);  }
+    var[0] = string[i+1];  var[1] = '\0';
+   }
+   if (!(v_string = getenv(var)))
+   {
+    for (k = 0; k < p_v_n - 1; k++)
+    {
+     if (!strcmp(p_v[k]->var, var))
+     {  v_string = p_v[k]->string;  break;  }
+    }
+   }
+   if (string[i+1] == '(' || string[i+1] == '{') len = (j-i+1);
+   else len = 2;
+   if (!v_string)
+   {
+    for(k = i; (string[k] = string[k+len]) != '\0'; k++);
+    if (!(string = REALLOC(tmp = string, (strlen(string) + 1) * sizeof(char))))
+    {  FREE(var);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(tmp);  }
+   }
+   else
+   {
+    len = strlen(v_string) - len;
+    if (len >= 0)
+    {
+     if (!(string = REALLOC(tmp = string, (k = strlen(string) + len + 1) * sizeof(char))))
+     {  FREE(var);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(tmp);  }
+     for (k--; k > j + len; k--) string[k] = string[k-len];
+     for (k = i; v_string[k-i]; k++) string[k] = v_string[k-i];
+    }
+    else
+    {
+     for (k = i; (string[k] = string[k-len]) != '\0'; k++);
+     for (k = i; v_string[k-i]; k++) string[k] = v_string[k-i];
+     if (!(string = REALLOC(tmp = string, (strlen(string) + 1) * sizeof(char))))
+     {  FREE(var);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(tmp);  }
+    }
+   }
+   FREE(var);
+  }
+ }
+ return(string);
 }
 
 int e_read_var(FENSTER *f)
 {
-   struct proj_var **tmp;
-   FILE *fp;
-   char str[256], *sp1, *sp2, *stmp;
-   int i, j;
-   if((fp = fopen(e_prog.project, "r")) == NULL) return(-1);
-   if(p_v)
-   {  for(i = 0; i < p_v_n; i++)
-      {  if(p_v[i])
-	 {  if(p_v[i]->var) FREE(p_v[i]->var);
-	    if(p_v[i]->string) FREE(p_v[i]->string);
-	    FREE(p_v[i]);
-	 }
-      }
-      FREE(p_v);
+ struct proj_var **tmp;
+ FILE *fp;
+ char str[256], *sp1, *sp2, *stmp;
+ int i;
+
+ if ((fp = fopen(e_prog.project, "r")) == NULL) return(-1);
+ if (p_v)
+ {
+  for (i = 0; i < p_v_n; i++)
+  {
+   if (p_v[i])
+   {
+    if (p_v[i]->var) FREE(p_v[i]->var);
+    if (p_v[i]->string) FREE(p_v[i]->string);
+    FREE(p_v[i]);
    }
-   p_v_n = 0;
-   if(!(p_v = MALLOC(sizeof(struct proj_var *))))
-   {  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
-   while(fgets(str, 256, fp))
-   {  for(i = 0; isspace(str[i]); i++);
-      if(!str[i]) continue;
-      else if(str[i] == '#')
-      {  while (str[strlen(str)-1] != '\n')
-	 {  fgets(str, 256, fp);  }
-	 continue;
-      }
-      sp1 = str+i;
-      for(i = 0; isalnum(sp1[i]); i++);
-      for(j = i; isspace(sp1[j]); j++);
-      if(str[j] != '=') continue;
-      for(j++; isspace(sp1[j]) && sp1[j] != '\n'; j++);
-      sp2 = sp1 + j;
-      sp1[i] = '\0';
-      p_v_n++;
-      if(!(p_v = REALLOC(tmp = p_v, sizeof(struct proj_var *) * p_v_n)))
-      {  p_v = tmp;  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
-      if(!(p_v[p_v_n-1] = MALLOC(sizeof(struct proj_var))))
-      {  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
-      if(!(p_v[p_v_n-1]->var = MALLOC((i+1) * sizeof(char))))
-      {  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
-      strcpy(p_v[p_v_n-1]->var, sp1);
-      if(!(p_v[p_v_n-1]->string = MALLOC((strlen(sp2)+1) * sizeof(char))))
-      {  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
-      strcpy(p_v[p_v_n-1]->string, sp2);
-      while(p_v[p_v_n-1]->string[i = strlen(p_v[p_v_n-1]->string) - 1]
-			!= '\n' || (i && p_v[p_v_n-1]->string[i-1] == '\\'))
-      {  if(p_v[p_v_n-1]->string[i-1] == '\\') p_v[p_v_n-1]->string[i-1] = '\0';
-	 if(!fgets(str, 256, fp)) break;
-	 if(!(p_v[p_v_n-1]->string = REALLOC(stmp = p_v[p_v_n-1]->string,
+  }
+  FREE(p_v);
+ }
+ p_v_n = 0;
+ if (!(p_v = MALLOC(sizeof(struct proj_var *))))
+ {  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
+ while (fgets(str, 256, fp))
+ {
+  for (i = 0; isspace(str[i]); i++);
+  if (!str[i]) continue;
+  else if (str[i] == '#')
+  {
+   while (str[strlen(str)-1] != '\n')
+   {  fgets(str, 256, fp);  }
+   continue;
+  }
+  sp1 = str + i;
+  sp2 = strchr(sp1, '=');
+  if (sp2 == NULL) continue;
+  for (stmp = sp1; !((isspace(*stmp)) || (*stmp == '=')) && *stmp; stmp++)
+   ;
+  *stmp = 0;
+  for (sp2++; isspace(*sp2) && *sp2 != '\n'; sp2++);
+  p_v_n++;
+  if (!(p_v = REALLOC(tmp = p_v, sizeof(struct proj_var *) * p_v_n)))
+  {  p_v = tmp;  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
+  if (!(p_v[p_v_n-1] = MALLOC(sizeof(struct proj_var))))
+  {  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
+  if (!(p_v[p_v_n-1]->var = MALLOC((i+1) * sizeof(char))))
+  {  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
+  strcpy(p_v[p_v_n-1]->var, sp1);
+  if (!(p_v[p_v_n-1]->string = MALLOC((strlen(sp2)+1) * sizeof(char))))
+  {  fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
+  strcpy(p_v[p_v_n-1]->string, sp2);
+  while (p_v[p_v_n-1]->string[i = strlen(p_v[p_v_n-1]->string) - 1] != '\n' || (i && p_v[p_v_n-1]->string[i-1] == '\\'))
+  {
+   if (p_v[p_v_n-1]->string[i-1] == '\\') p_v[p_v_n-1]->string[i-1] = '\0';
+   if (!fgets(str, 256, fp)) break;
+   if (!(p_v[p_v_n-1]->string = REALLOC(stmp = p_v[p_v_n-1]->string,
 		(strlen(p_v[p_v_n-1]->string)+strlen(str)+1) * sizeof(char))))
-	 {  p_v[p_v_n-1]->string = stmp;
-	    fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);  }
-	 strcat(p_v[p_v_n-1]->string, str);
-      }
-      p_v[p_v_n-1]->string[strlen(p_v[p_v_n-1]->string) - 1] = '\0';
-      for(i = 0; p_v[p_v_n-1]->string[i]; i++)
-      if(p_v[p_v_n-1]->string[i] == '\t') p_v[p_v_n-1]->string[i] = ' ';
-      p_v[p_v_n-1]->string = e_expand_var(p_v[p_v_n-1]->string, f);
+   {
+    p_v[p_v_n-1]->string = stmp;
+    fclose(fp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(-1);
    }
-   fclose(fp);
-   return(0);
+   strcat(p_v[p_v_n-1]->string, str);
+  }
+  p_v[p_v_n-1]->string[strlen(p_v[p_v_n-1]->string) - 1] = '\0';
+  for (i = 0; p_v[p_v_n-1]->string[i]; i++)
+   if (p_v[p_v_n-1]->string[i] == '\t') p_v[p_v_n-1]->string[i] = ' ';
+  p_v[p_v_n-1]->string = e_expand_var(p_v[p_v_n-1]->string, f);
+ }
+ fclose(fp);
+ return(0);
 }
 
 int e_install(FENSTER *f)
@@ -1972,7 +2012,7 @@ int e_c_project(FENSTER *f)
  e_s_prog.comp_sw &= ~1;
  e_argc = 1;
  argc = 1;
- for(i = f->ed->mxedt; i > 0 && (f->ed->f[i]->dtmd != 'D' ||
+ for(i = f->ed->mxedt; i > 0 && (f->ed->f[i]->dtmd != DTMD_DATA ||
    f->ed->f[i]->ins != 4 || !f->ed->f[i]->save); i--)
   ;
  if (i > 0) e_p_update_prj_fl(f);
@@ -2027,16 +2067,12 @@ int e_c_project(FENSTER *f)
  }
  df = e_p_get_var("EXENAME");
  elen = strlen(e_prog.exedir)-1;
-#ifdef DJGPP
- if (e_prog.exedir[elen] == DIRC || e_prog.exedir[elen] == '/')
-#else
  if (e_prog.exedir[elen] == '/')
-#endif
   sprintf(ofile, "%s%s", e_prog.exedir,
-    (df && df->name[0][0]) ? df->name[0] : "a.out");
+    (df && df->anz > 0 && df->name[0][0]) ? df->name[0] : "a.out");
  else
   sprintf(ofile, "%s/%s", e_prog.exedir,
-    (df && df->name[0][0]) ? df->name[0] : "a.out");
+    (df && df->anz > 0 && df->name[0][0]) ? df->name[0] : "a.out");
  if (df) freedf(df);
  if (e_s_prog.exe_name) FREE(e_s_prog.exe_name);
  e_s_prog.exe_name = WpeStrdup(ofile);
@@ -2083,7 +2119,7 @@ int e_c_project(FENSTER *f)
  df = e_p_get_var("FILES");
  if (!df)
  {
-  e_error(e_p_msg[ERR_NOTHING], 0, f->fb);
+  e_error(e_p_msg[ERR_NOTHING], 0, cn->fb);
   e_free_arg(arg, argc);  e_free_arg(e_arg, e_argc);
   return(-1);
  }
@@ -2285,78 +2321,81 @@ int e_rel_brkwtch(FENSTER *f)
 ****/  
 struct dirfile **e_make_prj_opt(FENSTER *f)
 {
-   int i, j, ret;
-   char **tmp, *sp, *tp, text[256];
-   FILE *fp;
-   struct dirfile *save_df = NULL;
-   for(i = f->ed->mxedt; i > 0
-	&& (f->ed->f[i]->dtmd != 'D' || f->ed->f[i]->ins != 4
+ int i, j, ret;
+ char **tmp, *sp, *tp, text[256];
+ FILE *fp;
+ struct dirfile *save_df = NULL;
+
+ for (i = f->ed->mxedt; i > 0
+	&& (f->ed->f[i]->dtmd != DTMD_DATA || f->ed->f[i]->ins != 4
 				     || !f->ed->f[i]->save); i--);
-   if(i > 0) {  save_df = e_p_df[0];  e_p_df[0] = NULL;  }
-   if(e_p_df) freedfN(e_p_df, 3);
-   e_p_df = MALLOC(3 * sizeof(struct dirfile *));
-   if(!e_p_df) return(e_p_df);
-   for(i = 0; i < 3; i++) e_p_df[i] = NULL;
-   e_s_prog.comp_sw = 0;
-   ret = e_read_var(f);
-   if(ret)
-   {
-      if (e_s_prog.compiler) FREE(e_s_prog.compiler);
-      e_s_prog.compiler = WpeStrdup("gcc");
-      if (e_s_prog.comp_str) FREE(e_s_prog.comp_str);
-      e_s_prog.comp_str = WpeStrdup("-g");
-      if (e_s_prog.libraries) FREE(e_s_prog.libraries);
-      e_s_prog.libraries = WpeStrdup("");
-      if (e_s_prog.exe_name) FREE(e_s_prog.exe_name);
-      /* Project my_prog.prj defaults to an executable of my_prog BD */
-      strcpy(text, e_prog.project);
-      e_s_prog.exe_name = WpeStrdup(WpeStringCutChar(text, '.'));
-      /*e_s_prog.exe_name = WpeStrdup("a.out");*/
-      if (e_s_prog.intstr) FREE(e_s_prog.intstr);
-      e_s_prog.intstr = WpeStrdup(cc_intstr);
-      strcpy(library, "");
-      for(i = !save_df ? 0 : 1; i < 3; i++)
-      {  e_p_df[i] = MALLOC(sizeof(struct dirfile));
-	 e_p_df[i]->name = MALLOC(sizeof(char *));
-	 e_p_df[i]->name[0] = MALLOC(2 * sizeof(char));
-	 *e_p_df[i]->name[0] = ' '; *(e_p_df[i]->name[0] + 1) = '\0';
-	 e_p_df[i]->anz = 1;
-      }
-      if(save_df) e_p_df[0] = save_df;
-      return(e_p_df);
-   }
-   if(!(e_p_df[1] = MALLOC(sizeof(struct dirfile)))) return(e_p_df);
-   if(!(e_p_df[1]->name = MALLOC(sizeof(char *)))) return(e_p_df);
-   e_p_df[1]->anz = 0;
-   if(!(e_p_df[2] = MALLOC(sizeof(struct dirfile)))) return(e_p_df);
-   if(!(e_p_df[2]->name = MALLOC(sizeof(char *)))) return(e_p_df);
-   e_p_df[2]->anz = 0;
-   for(i = 0; i < p_v_n; i++)
-   {  if(!strcmp(p_v[i]->var, "CMP"))
-      {
-         if (e_s_prog.compiler) FREE(e_s_prog.compiler);
-         e_s_prog.compiler = WpeStrdup(p_v[i]->string);
-      }
-      else if(!strcmp(p_v[i]->var, "CMPFLAGS"))
-      {
-         if (e_s_prog.comp_str) FREE(e_s_prog.comp_str);
-         e_s_prog.comp_str = WpeStrdup(p_v[i]->string);
-      }
-      else if(!strcmp(p_v[i]->var, "LDFLAGS"))
-      {
-         if (e_s_prog.libraries) FREE(e_s_prog.libraries);
-          e_s_prog.libraries = WpeStrdup(p_v[i]->string);
-      }
-      else if(!strcmp(p_v[i]->var, "EXENAME"))
-      {
-         if (e_s_prog.exe_name) FREE(e_s_prog.exe_name);
-         e_s_prog.exe_name = WpeStrdup(p_v[i]->string);
-      }
-      else if(!strcmp(p_v[i]->var, "CMPMESSAGE"))
-      {
-         if (e_s_prog.intstr) FREE(e_s_prog.intstr);
-         e_s_prog.intstr = WpeStrdup(e_interpr_var(p_v[i]->string));
-      }
+ if (i > 0) {  save_df = e_p_df[0];  e_p_df[0] = NULL;  }
+ if (e_p_df) freedfN(e_p_df, 3);
+ e_p_df = MALLOC(3 * sizeof(struct dirfile *));
+ if (!e_p_df) return(e_p_df);
+ for (i = 0; i < 3; i++) e_p_df[i] = NULL;
+ e_s_prog.comp_sw = 0;
+ ret = e_read_var(f);
+ if (ret)
+ {
+  if (e_s_prog.compiler) FREE(e_s_prog.compiler);
+  e_s_prog.compiler = WpeStrdup("gcc");
+  if (e_s_prog.comp_str) FREE(e_s_prog.comp_str);
+  e_s_prog.comp_str = WpeStrdup("-g");
+  if (e_s_prog.libraries) FREE(e_s_prog.libraries);
+  e_s_prog.libraries = WpeStrdup("");
+  if (e_s_prog.exe_name) FREE(e_s_prog.exe_name);
+  /* Project my_prog.prj defaults to an executable of my_prog BD */
+  strcpy(text, e_prog.project);
+  e_s_prog.exe_name = WpeStrdup(WpeStringCutChar(text, '.'));
+  /*e_s_prog.exe_name = WpeStrdup("a.out");*/
+  if (e_s_prog.intstr) FREE(e_s_prog.intstr);
+  e_s_prog.intstr = WpeStrdup(cc_intstr);
+  strcpy(library, "");
+  for (i = !save_df ? 0 : 1; i < 3; i++)
+  {
+   e_p_df[i] = MALLOC(sizeof(struct dirfile));
+   e_p_df[i]->name = MALLOC(sizeof(char *));
+   e_p_df[i]->name[0] = MALLOC(2 * sizeof(char));
+   *e_p_df[i]->name[0] = ' '; *(e_p_df[i]->name[0] + 1) = '\0';
+   e_p_df[i]->anz = 1;
+  }
+  if (save_df) e_p_df[0] = save_df;
+  return(e_p_df);
+ }
+ if (!(e_p_df[1] = MALLOC(sizeof(struct dirfile)))) return(e_p_df);
+ if (!(e_p_df[1]->name = MALLOC(sizeof(char *)))) return(e_p_df);
+ e_p_df[1]->anz = 0;
+ if (!(e_p_df[2] = MALLOC(sizeof(struct dirfile)))) return(e_p_df);
+ if (!(e_p_df[2]->name = MALLOC(sizeof(char *)))) return(e_p_df);
+ e_p_df[2]->anz = 0;
+ for (i = 0; i < p_v_n; i++)
+ {
+  if (!strcmp(p_v[i]->var, "CMP"))
+  {
+   if (e_s_prog.compiler) FREE(e_s_prog.compiler);
+   e_s_prog.compiler = WpeStrdup(p_v[i]->string);
+  }
+  else if (!strcmp(p_v[i]->var, "CMPFLAGS"))
+  {
+   if (e_s_prog.comp_str) FREE(e_s_prog.comp_str);
+   e_s_prog.comp_str = WpeStrdup(p_v[i]->string);
+  }
+  else if (!strcmp(p_v[i]->var, "LDFLAGS"))
+  {
+   if (e_s_prog.libraries) FREE(e_s_prog.libraries);
+   e_s_prog.libraries = WpeStrdup(p_v[i]->string);
+  }
+  else if (!strcmp(p_v[i]->var, "EXENAME"))
+  {
+   if (e_s_prog.exe_name) FREE(e_s_prog.exe_name);
+   e_s_prog.exe_name = WpeStrdup(p_v[i]->string);
+  }
+  else if (!strcmp(p_v[i]->var, "CMPMESSAGE"))
+  {
+   if (e_s_prog.intstr) FREE(e_s_prog.intstr);
+   e_s_prog.intstr = WpeStrdup(e_interpr_var(p_v[i]->string));
+  }
 
 /**************************/
 /**** 
@@ -2365,123 +2404,132 @@ struct dirfile **e_make_prj_opt(FENSTER *f)
   These variables will be processed later on in e_rel_brkwtch
   function.
 ****/  
-      else if(!strcmp(p_v[i]->var, "BREAKPOINTS"))
-      {
-      }
-      else if(!strcmp(p_v[i]->var, "WATCHES"))
-      {
-      }
+  else if (!strcmp(p_v[i]->var, "BREAKPOINTS"))
+  {
+  }
+  else if (!strcmp(p_v[i]->var, "WATCHES"))
+  {
+  }
 /**************************/
 
-      else if(!strcmp(p_v[i]->var, "LIBNAME"))
-      strcpy(library, p_v[i]->string);
-      else if(!strcmp(p_v[i]->var, "CMPSWTCH"))
-      {  if(!strcmp(p_v[i]->string, "other")) e_s_prog.comp_sw = 1;
-      }
-      else if(!strcmp(p_v[i]->var, "FILES"))
-      e_p_df[0] = e_p_get_args(p_v[i]->string);
-      else
-      {  e_p_df[1]->anz++;
-	 if(!(e_p_df[1]->name = REALLOC(tmp =
+  else if (!strcmp(p_v[i]->var, "LIBNAME"))
+   strcpy(library, p_v[i]->string);
+  else if (!strcmp(p_v[i]->var, "CMPSWTCH"))
+  {
+   if (!strcmp(p_v[i]->string, "other")) e_s_prog.comp_sw = 1;
+  }
+  else if (!strcmp(p_v[i]->var, "FILES"))
+   e_p_df[0] = e_p_get_args(p_v[i]->string);
+  else
+  {
+   e_p_df[1]->anz++;
+   if (!(e_p_df[1]->name = REALLOC(tmp =
 				e_p_df[1]->name, e_p_df[1]->anz * sizeof(char *))))
-	 {  e_p_df[1]->anz--;  e_p_df[1]->name = tmp;  return(e_p_df);  }
-	 if(!(e_p_df[1]->name[e_p_df[1]->anz-1] = MALLOC((strlen(p_v[i]->var)
+   {  e_p_df[1]->anz--;  e_p_df[1]->name = tmp;  return(e_p_df);  }
+   if (!(e_p_df[1]->name[e_p_df[1]->anz-1] = MALLOC((strlen(p_v[i]->var)
 			+ strlen(p_v[i]->string) + 2)*sizeof(char))))
-	 {  e_p_df[1]->anz--;  return(e_p_df);  }
-	 sprintf(e_p_df[1]->name[e_p_df[1]->anz-1], "%s=%s",
+   {  e_p_df[1]->anz--;  return(e_p_df);  }
+   sprintf(e_p_df[1]->name[e_p_df[1]->anz-1], "%s=%s",
 					p_v[i]->var, p_v[i]->string);
-      }
-   }
-   if(!e_s_prog.compiler)
-   e_s_prog.compiler = WpeStrdup("gcc");
-   if(!e_s_prog.comp_str)
-   e_s_prog.comp_str = WpeStrdup("-g");
-   if(!e_s_prog.libraries)
-   e_s_prog.libraries = WpeStrdup("");
-   if(!e_s_prog.exe_name)
-   {
-    /* Project my_prog.prj defaults to an executable of my_prog BD */
-    strcpy(text, e_prog.project);
-    e_s_prog.exe_name = WpeStrdup(WpeStringCutChar(text, '.'));
-    /*e_s_prog.exe_name = WpeStrdup("a.out");*/
-   }
-   if(!e_s_prog.intstr)
-   e_s_prog.intstr = WpeStrdup(cc_intstr);
-   if(!e_p_df[0])
-   {  e_p_df[0] = MALLOC(sizeof(struct dirfile));
-      e_p_df[0]->anz = 0;
-   }
-   if((fp = fopen(e_prog.project, "r")) == NULL)
-   {  sprintf(text, e_msg[ERR_FOPEN], e_prog.project);
-      e_error(text, 0, f->fb);
-      return(e_p_df);
-   }
-   while((tp = fgets(text, 256, fp)))
-   {  if(text[0] == '\t') continue;
-      for(i = 0; isspace(text[i]); i++);
-      if(!strncmp(text+i, "install:", 8))
-      {  while(tp && (text[j = strlen(text)-1] != '\n'
-			|| text[j-1] == '\\'))
-	 tp = fgets(text, 256, fp);
-	 break;
-      }
-   }
-   if(!tp) {  fclose(fp);  return(e_p_df);  }
-   while(tp && (tp = fgets(text, 256, fp)))
-   {  for(i = 0; isspace(text[i]); i++);
-      sp = text+i;
-      if(sp[0] == '#')
-      {  while(tp && (text[j = strlen(text)-1] != '\n'
-			|| text[j-1] == '\\'))
-	 tp = fgets(text, 256, fp);
-	 continue;
-      }
-      if(text[0] != '\t') break;
-      if(sp[0] == '\0') continue;
-      e_p_df[2]->anz++;
-      if(!(e_p_df[2]->name = REALLOC(tmp =
+  }
+ }
+ if (!e_s_prog.compiler)
+  e_s_prog.compiler = WpeStrdup("gcc");
+ if (!e_s_prog.comp_str)
+  e_s_prog.comp_str = WpeStrdup("-g");
+ if (!e_s_prog.libraries)
+  e_s_prog.libraries = WpeStrdup("");
+ if (!e_s_prog.exe_name)
+ {
+  /* Project my_prog.prj defaults to an executable of my_prog BD */
+  strcpy(text, e_prog.project);
+  e_s_prog.exe_name = WpeStrdup(WpeStringCutChar(text, '.'));
+  /*e_s_prog.exe_name = WpeStrdup("a.out");*/
+ }
+ if (!e_s_prog.intstr)
+  e_s_prog.intstr = WpeStrdup(cc_intstr);
+ if (!e_p_df[0])
+ {
+  e_p_df[0] = MALLOC(sizeof(struct dirfile));
+  e_p_df[0]->anz = 0;
+ }
+ if ((fp = fopen(e_prog.project, "r")) == NULL)
+ {
+  sprintf(text, e_msg[ERR_FOPEN], e_prog.project);
+  e_error(text, 0, f->fb);
+  return(e_p_df);
+ }
+ while ((tp = fgets(text, 256, fp)))
+ {
+  if (text[0] == '\t') continue;
+  for (i = 0; isspace(text[i]); i++);
+  if (!strncmp(text+i, "install:", 8))
+  {
+   while(tp && (text[j = strlen(text)-1] != '\n' || text[j-1] == '\\'))
+    tp = fgets(text, 256, fp);
+   break;
+  }
+ }
+ if (!tp) {  fclose(fp);  return(e_p_df);  }
+ while(tp && (tp = fgets(text, 256, fp)))
+ {
+  for (i = 0; isspace(text[i]); i++);
+  sp = text+i;
+  if (sp[0] == '#')
+  {
+   while(tp && (text[j = strlen(text)-1] != '\n' || text[j-1] == '\\'))
+    tp = fgets(text, 256, fp);
+   continue;
+  }
+  if (text[0] != '\t') break;
+  if (sp[0] == '\0') continue;
+  e_p_df[2]->anz++;
+  if (!(e_p_df[2]->name = REALLOC(tmp =
 				e_p_df[2]->name, e_p_df[2]->anz * sizeof(char *))))
-      {  e_p_df[2]->anz--;  e_p_df[2]->name = tmp;  fclose(fp);  return(e_p_df);  }
-      if(!(e_p_df[2]->name[e_p_df[2]->anz-1] = MALLOC((strlen(sp) + 1))))
-      {  e_p_df[2]->anz--;  fclose(fp);  return(e_p_df);  }
-      
-      strcpy(e_p_df[2]->name[e_p_df[2]->anz-1], sp);
-      while(tp && (text[j = strlen(text)-1] != '\n'
-						|| text[j-1] == '\\'))
-      {  tp = fgets(text, 256, fp);
-	 if(tp)
-	 {  j = strlen(e_p_df[2]->name[e_p_df[2]->anz-1]);
-	    *(e_p_df[2]->name[e_p_df[2]->anz-1]+j-2) = '\0';
-	    if(!(e_p_df[2]->name[e_p_df[2]->anz-1] =
+  {  e_p_df[2]->anz--;  e_p_df[2]->name = tmp;  fclose(fp);  return(e_p_df);  }
+  if (!(e_p_df[2]->name[e_p_df[2]->anz-1] = MALLOC((strlen(sp) + 1))))
+  {  e_p_df[2]->anz--;  fclose(fp);  return(e_p_df);  }
+
+  strcpy(e_p_df[2]->name[e_p_df[2]->anz-1], sp);
+  while(tp && (text[j = strlen(text)-1] != '\n' || text[j-1] == '\\'))
+  {
+   tp = fgets(text, 256, fp);
+   if (tp)
+   {
+    j = strlen(e_p_df[2]->name[e_p_df[2]->anz-1]);
+    *(e_p_df[2]->name[e_p_df[2]->anz-1]+j-2) = '\0';
+    if (!(e_p_df[2]->name[e_p_df[2]->anz-1] =
 		    REALLOC(sp = e_p_df[2]->name[e_p_df[2]->anz-1],
 			strlen(e_p_df[2]->name[e_p_df[2]->anz-1])
 			+ strlen(text) + 1)))
-	    {  fclose(fp);  FREE(sp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);
+    {  fclose(fp);  FREE(sp);  e_error(e_msg[ERR_LOWMEM], 0, f->fb);
 	       return(e_p_df);
-	    }
-	    strcat(e_p_df[2]->name[e_p_df[2]->anz-1], text);
-	 }
-      }
-      j = strlen(e_p_df[2]->name[e_p_df[2]->anz-1]);
-      if(*(e_p_df[2]->name[e_p_df[2]->anz-1]+j-1) == '\n')
-		      *(e_p_df[2]->name[e_p_df[2]->anz-1]+j-1) = '\0';
+    }
+    strcat(e_p_df[2]->name[e_p_df[2]->anz-1], text);
    }
-   fclose(fp);
-   for(i = 0; i < 3; i++)
-   {  if(!e_p_df[i])
-      {  e_p_df[i] = MALLOC(sizeof(struct dirfile));
-	 e_p_df[i]->name = MALLOC(sizeof(char *));
-	 e_p_df[i]->anz = 0;
-      }
-      e_p_df[i]->name = REALLOC(e_p_df[i]->name,
+  }
+  j = strlen(e_p_df[2]->name[e_p_df[2]->anz-1]);
+  if (*(e_p_df[2]->name[e_p_df[2]->anz-1]+j-1) == '\n')
+   *(e_p_df[2]->name[e_p_df[2]->anz-1]+j-1) = '\0';
+ }
+ fclose(fp);
+ for (i = 0; i < 3; i++)
+ {
+  if (!e_p_df[i])
+  {
+   e_p_df[i] = MALLOC(sizeof(struct dirfile));
+   e_p_df[i]->name = MALLOC(sizeof(char *));
+   e_p_df[i]->anz = 0;
+  }
+  e_p_df[i]->name = REALLOC(e_p_df[i]->name,
 				(e_p_df[i]->anz + 1) * sizeof(char *));
-      e_p_df[i]->name[e_p_df[i]->anz] = MALLOC(2*sizeof(char));
-      *e_p_df[i]->name[e_p_df[i]->anz] = ' ';
-      *(e_p_df[i]->name[e_p_df[i]->anz] + 1) = '\0';
-      e_p_df[i]->anz++;
-   }
-   if(save_df) {  freedf(e_p_df[0]);  e_p_df[0] = save_df;  }
-   return(e_p_df);
+  e_p_df[i]->name[e_p_df[i]->anz] = MALLOC(2*sizeof(char));
+  *e_p_df[i]->name[e_p_df[i]->anz] = ' ';
+  *(e_p_df[i]->name[e_p_df[i]->anz] + 1) = '\0';
+  e_p_df[i]->anz++;
+ }
+ if (save_df) {  freedf(e_p_df[0]);  e_p_df[0] = save_df;  }
+ return(e_p_df);
 }
 
 int freedfN(struct dirfile **df, int n)
@@ -2502,7 +2550,7 @@ int e_wrt_prj_fl(FENSTER *f)
  char text[256];
 
  for (i = f->ed->mxedt;
-   i > 0 &&  (f->ed->f[i]->dtmd != 'D' || f->ed->f[i]->ins != 4); i--)
+   i > 0 &&  (f->ed->f[i]->dtmd != DTMD_DATA || f->ed->f[i]->ins != 4); i--)
   ;
  if (i == 0 || e_prog.project[0] == DIRC)
   strcpy(text, e_prog.project);
@@ -2747,53 +2795,71 @@ int e_p_show_messages(FENSTER *f)
 int e_p_konv_mess(char *var, char *str, char *txt, char *file, char *cmp,
   int *y, int *x)
 {
-   int i;
-   char *cp;
-   if(!strncmp(var, "FILE", 4) && !isalnum(var[4]))
-   {  for(i = strlen(str) - 1; i >= 0 && !isspace(str[i]); i--);
-      strcpy(file, str+i+1);
-   }
-   else if(!strncmp(var, "CMPTEXT", 7) && !isalnum(var[7])) strcpy(cmp, str);
-   else if(!strncmp(var, "LINE", 4) && !isalnum(var[4]))
-   {  if(!isdigit(str[0])) return(1);
-      *y = atoi(str);
-      if(var[4] == '+') *y += atoi(var+5);
-      else if(var[4] == '-') *y -= atoi(var+5);
-   }
-   else if(!strncmp(var, "COLUMN", 6) && !isalnum(var[6]))
-   {  if(!strncmp(var+6, "=BEFORE", 7))
-      {  txt[0] = 'B';
-	 strcpy(txt+1, str);
-	 *x = 0;
-	 var += 13;
-      }
-      else if(!strncmp(var+6, "=AFTER", 6))
-      {  txt[0] = 'A';
-	 strcpy(txt+1, str);
-	 *x = strlen(str);
-	 var += 12;
-      }
-      else if(!strncmp(var+6, "=PREVIOUS?", 10))
-      {  if(!str[0]) return(1);
-	 for(i = 0; (txt[i] = var[16+i])
-			&& txt[i] != '+' && txt[i] != '-'; i++);
-	 txt[i] = '\0';
-	 var += (16+i);
-	 cp = strstr(str, txt);
-	 for(i = 0; str+i < cp; i++);
-	 *x = i;
-	 txt[0] = 'P'; txt[1] = '\0';
-      }
-      else if(!isdigit(str[0])) return(1);
-      else
-      {  *x = atoi(str);
-	 txt[0] = '\0';
-	 var += 6;
-      }
-      if(var[0] == '+') *x += atoi(var+1);
-      else if(var[0] == '-') *x -= atoi(var+1);
-   }
-   return(0);
+ int i;
+ char *cp;
+
+ if (!strncmp(var, "FILE", 4) && !isalnum(var[4]))
+ {
+  for (i = strlen(str) - 1; i >= 0 && !isspace(str[i]); i--)
+   ;
+  strcpy(file, str+i+1);
+ }
+ else if (!strncmp(var, "CMPTEXT", 7) && !isalnum(var[7]))
+  strcpy(cmp, str);
+ else if (!strncmp(var, "LINE", 4) && !isalnum(var[4]))
+ {
+  if (!isdigit(str[0]))
+   return(1);
+  *y = atoi(str);
+  if (var[4] == '+')
+   *y += atoi(var+5);
+  else if (var[4] == '-')
+   *y -= atoi(var+5);
+ }
+ else if (!strncmp(var, "COLUMN", 6) && !isalnum(var[6]))
+ {
+  if (!strncmp(var+6, "=BEFORE", 7))
+  {
+   txt[0] = 'B';
+   strcpy(txt+1, str);
+   *x = 0;
+   var += 13;
+  }
+  else if (!strncmp(var+6, "=AFTER", 6))
+  {
+   txt[0] = 'A';
+   strcpy(txt+1, str);
+   *x = strlen(str);
+   var += 12;
+  }
+  else if (!strncmp(var+6, "=PREVIOUS?", 10))
+  {
+   if (!str[0])
+    return(1);
+   for (i = 0; (txt[i] = var[16+i]) && txt[i] != '+' && txt[i] != '-'; i++)
+    ;
+   txt[i] = '\0';
+   var += (16+i);
+   cp = strstr(str, txt);
+   for (i = 0; str+i < cp; i++)
+    ;
+   *x = i;
+   txt[0] = 'P'; txt[1] = '\0';
+  }
+  else if (!isdigit(str[0]))
+   return(1);
+  else
+  {
+   *x = atoi(str);
+   txt[0] = '\0';
+   var += 6;
+  }
+  if (var[0] == '+')
+   *x += atoi(var+1);
+  else if (var[0] == '-')
+   *x -= atoi(var+1);
+ }
+ return(0);
 }
 
 int e_p_comp_mess(char *a, char *b, char *c, char *txt, char *file, char *cmp,
