@@ -363,7 +363,7 @@ int e_program_opt(FENSTER *f)
   if (e_prog.sys_include)
    FREE(e_prog.sys_include);
   e_prog.sys_include = WpeStrdup(o->wstr[1]->txt);
-  f->ed->edopt = (f->ed->edopt & ~56) +
+  f->ed->edopt = (f->ed->edopt & ~ED_PROGRAMMING_OPTIONS) +
     (o->sstr[0]->num ? ED_ERRORS_STOP_AT : 0) +
     (o->sstr[1]->num ? ED_MESSAGES_STOP_AT : 0) +
     (o->sstr[2]->num ? ED_SYNTAX_HIGHLIGHT : 0);
@@ -443,7 +443,7 @@ int *e_sc_txt(int *c_sw, BUFFER *b)
  return(c_sw);
 }
 
-/*       Schreiben einer Zeile (Schirminhalt)      */
+/*       Writing of a line (content of a screen)      */
 void e_pr_c_line(int y, FENSTER *f)
 {
  BUFFER *b = f->b;
@@ -550,7 +550,8 @@ void e_pr_c_line(int y, FENSTER *f)
 
  e_mk_col(b->bf[y].s, i, frb, f->c_st, n_nd, n_bg);
 
- if (i == b->bf[y].len && f->dtmd == 's' && j < NUM_COLS_ON_SCREEN_SAFE + s->c.x - 1)
+ if ((i == b->bf[y].len) && (f->ed->edopt & ED_SHOW_ENDMARKS) &&
+   (DTMD_ISMARKABLE(f->dtmd)) && (j < NUM_COLS_ON_SCREEN_SAFE + s->c.x - 1))
  {
   if ((y < s->mark_end.y && ( y > s->mark_begin.y ||
     (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) ||
@@ -606,8 +607,7 @@ int e_add_synt_tl(char *filename, FENSTER *f)
 /*  Browser   */
 E_AFILE *e_aopen(char *name, char *path, int mode)
 {
- extern struct EXT h_error;
- ECNT *cn = h_error.cn;
+ ECNT *cn = WpeEditor;
  E_AFILE *ep = MALLOC(sizeof(E_AFILE));
  char str[256];
  int i, j;
@@ -1132,79 +1132,86 @@ int e_find_def(name, startfile, mode, file, num, xn, nold, oldfile, df, first)
    return(ret);
 }
 
-int e_show_nm_f(name, f, oldn, oldname)
-     char *name;
-     FENSTER *f;
-     int oldn;
-     char **oldname;
+int e_show_nm_f(char *name, FENSTER *f, int oldn, char **oldname)
 {
-   int i, j, len, ret, num, x, first = oldn < 0 ? 1 : 0;
-   char str[128], file[128], *filename;
-   struct dirfile *df, *fdf = NULL;
+ int i, j, len, ret, num, x, first = oldn < 0 ? 1 : 0;
+ char str[128], file[128], *filename;
+ struct dirfile *df, *fdf = NULL;
+
 #ifndef TESTSDEF
-   if(!access(e_prog.project, 0))
+ if (!access(e_prog.project, 0))
+ {
+  WpeMouseChangeShape(WpeWorkingShape);
+  if (e_read_var(f)) ret = -1;
+  else
+  {
+   df = e_p_get_var("FILES");
+   if (!df)
    {
-      WpeMouseChangeShape(WpeWorkingShape);
-      if(e_read_var(f)) ret = -1;
-      else
-      {  df = e_p_get_var("FILES");
-	 if(!df)
-	 {  e_error(e_p_msg[ERR_NOTHING], 0, f->fb);
-            WpeMouseRestoreShape();
-	    return(-1);
-	 }
-	 for(i = 0, ret = 1; i < df->anz && ret; i++)
-	 {  strcpy(str, df->name[i]);
-	    ret = e_find_def(name, str, 7, file, &num, &x, oldn,
-						*oldname, &fdf, &first);
-	 }
-	 freedf(df);
-      }
+    e_error(e_p_msg[ERR_NOTHING], 0, f->fb);
+    WpeMouseRestoreShape();
+    return(-1);
    }
-   else
+   for (i = 0, ret = 1; i < df->anz && ret; i++)
+   {
+    strcpy(str, df->name[i]);
+    ret = e_find_def(name, str, 7, file, &num, &x, oldn,
+      *oldname, &fdf, &first);
+   }
+   freedf(df);
+  }
+ }
+ else
 #endif
-   {  if(f->dtmd <= 'Z') return(-1);
-      WpeMouseChangeShape(WpeWorkingShape);
-      strcpy(str, f->datnam);
-      ret = e_find_def(name, str, 7, file, &num, &x, oldn,
-						*oldname, &fdf, &first);
-   }
-   freedf(fdf);
-   if(ret)
-   {  sprintf(str, "%s not found!", name);
-      e_error(str, 0, f->fb);
-      WpeMouseRestoreShape();
-      return(-1);
-   }
-   if(*oldname) FREE(*oldname);
-   *oldname = MALLOC((strlen(file)+1) * sizeof(char));
-   strcpy(*oldname, file);
-   for(i = strlen(file)-1; i >= 0 && file[i] != '/'; i--);
-   for(j = f->ed->mxedt; j > 0; j--)
-   {  if(i < 0) filename = f->ed->f[j]->datnam;
-      else filename = e_mkfilename(f->ed->f[j]->dirct, f->ed->f[j]->datnam);
-      if(!strcmp(filename, file)) break;
-   }
-   if(j > 0) e_switch_window(f->ed->edt[j], f->ed->f[f->ed->mxedt]);
-   else e_edit(f->ed, file);
-   f = f->ed->f[f->ed->mxedt];
-   for(i = num, j = x+1-(len = strlen(name)); i >= 0; )
-   {  for(len = strlen(name); j >= 0
-			&& strncmp(name, f->b->bf[i].s+j, len); j--);
-      if(j < 0 && i >= 0)
-      {  i--;  j = f->b->bf[i].len-len+1;  }
-      else break;
-   }
-   if(i >= 0) {  num = i;  x = j;  }
-   else len = 0;
-   f->s->fa.y = f->s->fe.y = f->b->b.y = num;
-   f->s->fe.x = f->b->b.x = x + len;
-   f->s->fa.x = x;
-   e_cursor(f, 1);
-   f->s->fa.y = num;
-   e_schirm(f, 1);
-   WpeMouseRestoreShape();
-   return(num);
+ {
+  if (!DTMD_ISTEXT(f->dtmd)) return(-1);
+  WpeMouseChangeShape(WpeWorkingShape);
+  strcpy(str, f->datnam);
+  ret = e_find_def(name, str, 7, file, &num, &x, oldn,
+    *oldname, &fdf, &first);
+ }
+ freedf(fdf);
+ if (ret)
+ {
+  sprintf(str, "%s not found!", name);
+  e_error(str, 0, f->fb);
+  WpeMouseRestoreShape();
+  return(-1);
+ }
+ if (*oldname) FREE(*oldname);
+ *oldname = MALLOC((strlen(file)+1) * sizeof(char));
+ strcpy(*oldname, file);
+ for (i = strlen(file)-1; i >= 0 && file[i] != '/'; i--)
+  ;
+ for (j = f->ed->mxedt; j > 0; j--)
+ {
+  if (i < 0) filename = f->ed->f[j]->datnam;
+  else filename = e_mkfilename(f->ed->f[j]->dirct, f->ed->f[j]->datnam);
+  if (!strcmp(filename, file)) break;
+ }
+ if (j > 0) e_switch_window(f->ed->edt[j], f->ed->f[f->ed->mxedt]);
+ else e_edit(f->ed, file);
+ f = f->ed->f[f->ed->mxedt];
+ for (i = num, j = x+1-(len = strlen(name)); i >= 0; )
+ {
+  for (len = strlen(name); j >= 0 && strncmp(name, f->b->bf[i].s+j, len);
+    j--)
+   ;
+  if (j < 0 && i >= 0)
+  {  i--;  j = f->b->bf[i].len-len+1;  }
+  else break;
+ }
+ if (i >= 0)
+ {  num = i;  x = j;  }
+ else len = 0;
+ f->s->fa.y = f->s->fe.y = f->b->b.y = num;
+ f->s->fe.x = f->b->b.x = x + len;
+ f->s->fa.x = x;
+ e_cursor(f, 1);
+ f->s->fa.y = num;
+ e_schirm(f, 1);
+ WpeMouseRestoreShape();
+ return(num);
 }
 
 struct {
@@ -1594,7 +1601,7 @@ int e_mk_beauty(int sw, int ndif, FENSTER *f)
  int *vkcb = MALLOC(sizeof(int));
  POINT sa, se, sb;
 
- for (i = f->ed->mxedt; i > 0 && f->ed->f[i]->dtmd <= 'Z'; i--)
+ for (i = f->ed->mxedt; i > 0 && !DTMD_ISTEXT(f->ed->f[i]->dtmd); i--)
   ;
  if (i <= 0)
  {
