@@ -80,8 +80,6 @@ char *tgoto();
 #define tparm2(a,b,c) tparm((a), (b), (c), 0, 0, 0, 0, 0, 0, 0)
 #endif
 
-extern int e_term_def;
-
 int WpeDllInit(int *argc, char **argv)
 {
  fk_u_cursor = fk_t_cursor;
@@ -96,8 +94,9 @@ int WpeDllInit(int *argc, char **argv)
  e_u_getch = e_t_getch;
  e_u_sys_ini = e_t_sys_ini;
  e_u_sys_end = e_t_sys_end;
+ e_u_system = system;
  fk_u_putchar = fk_t_putchar;
-#ifdef GPM
+#ifdef HAVE_LIBGPM
  if (WpeGpmMouseInit() == 0)
  {
   fk_mouse = WpeGpmMouse;
@@ -293,22 +292,12 @@ int init_cursor()
    key_f[11] = init_kkey("kcud1");
    key_f[12] = init_kkey("kcub1");
    key_f[13] = init_kkey("kcuf1");
-   if(e_term_def == 1)
-   {  key_f[14] = init_kkey("kfnd");
-      key_f[15] = init_kkey("kich1");
-      if(!(key_f[16] = init_kkey("kdch1"))) key_f[16] = "\33[3~";
-      key_f[17] = init_kkey("kslt");
-      key_f[18] = init_kkey("kpp");
-      key_f[19] = init_kkey("knp");
-   }
-   else
-   {  key_f[14] = init_kkey("kich1");
-      key_f[15] = init_kkey("khome");
-      key_f[16] = init_kkey("kpp");
-      if(!(key_f[17] = init_kkey("kdch1"))) key_f[17] = "\177";
-      key_f[18] = init_kkey("kend");
-      key_f[19] = init_kkey("knp");
-   }
+   key_f[14] = init_kkey("kich1");
+   key_f[15] = init_kkey("khome");
+   key_f[16] = init_kkey("kpp");
+   if(!(key_f[17] = init_kkey("kdch1"))) key_f[17] = "\177";
+   key_f[18] = init_kkey("kend");
+   key_f[19] = init_kkey("knp");
    key_f[20] = init_kkey("kbs");
    key_f[21] = init_kkey("khlp");
    key_f[22] = init_kkey("kll");
@@ -537,32 +526,36 @@ int fk_attrset(int a)
 
 void fk_colset(int c)
 {
-   int bg;
-   if(cur_attr == c) return;
-   cur_attr = c;
-   bg = c / 16;
+ int bg;
+ if (cur_attr == c) return;
+ cur_attr = c;
+ bg = c / 16;
 #ifdef TERMCAP
-   if((c %= 16) >= col_num)
-   {  fk_putp(att_bo);
-      e_putp(tgoto(col_fg, 0, c % col_num));
-      e_putp(tgoto(col_bg, 0, bg));
-   }
-   else
-   {  fk_putp(ratt_bo);
-      e_putp(tgoto(col_fg, 0, c));
-      e_putp(tgoto(col_bg, 0, bg));
-   }
+ if ((c %= 16) >= col_num)
+ {
+  fk_putp(att_bo);
+  e_putp(tgoto(col_fg, 0, c % col_num));
+  e_putp(tgoto(col_bg, 0, bg));
+ }
+ else
+ {
+  fk_putp(ratt_bo);
+  e_putp(tgoto(col_fg, 0, c));
+  e_putp(tgoto(col_bg, 0, bg));
+ }
 #else
-   if((c %= 16) >= col_num)
-   {  fk_putp(att_bo);
-      e_putp(tparm1(col_fg, c % col_num));
-      e_putp(tparm1(col_bg, bg));
-   }
-   else
-   {  fk_putp(ratt_bo);
-      e_putp(tparm1(col_fg, c));
-      e_putp(tparm1(col_bg, bg));
-   }
+ if ((c %= 16) >= col_num)
+ {
+  fk_putp(att_bo);
+  e_putp(tparm1(col_fg, c % col_num));
+  e_putp(tparm1(col_bg, bg));
+ }
+ else
+ {
+  fk_putp(ratt_bo);
+  e_putp(tparm1(col_fg, c));
+  e_putp(tparm1(col_bg, bg));
+ }
 #endif
 }
 
