@@ -60,106 +60,112 @@ char **StringToStringArray(char *str, int *maxLen, int minWidth, int *anzahl)
 }
 
 /*
-      Fehlernachricht ausgeben        */
+      Print error message        */
 int e_error(char *text, int sw, FARBE *f)
 {
-   extern struct EXT h_error;
-   PIC *pic = NULL;
-   int len, i, xa, xe, ya = 8, ye = 14;
-   char *header = NULL;
+ PIC *pic = NULL;
+ int len, i, xa, xe, ya = 8, ye = 14;
+ char *header = NULL;
 
-   fk_cursor(0);
-   WpeMouseChangeShape(WpeErrorShape);
-   if((len = strlen((char *)text)) < 20 ) len = 20;
-   xa = (80-len)/2 - 2;
-   xe = 82 - (80-len)/2;
-   if(sw == -1) header = "Message";
-   else if(sw == 0) header = "Error";
-   else if(sw == 1) header = "Serious Error";
-   else if(sw == 2) header = "Fatal Error";
-   
-   if(sw < 2) pic = e_std_kst(xa, ya, xe, ye, header, 1, f->nr.fb, f->nt.fb, f->ne.fb);
-   if(sw == 2 || pic == NULL)
-   {  pic = e_open_view(xa, ya, xe, ye, 0, 0);
-      e_std_rahmen(xa, ya, xe, ye, header, 1, 0, 0);
-   }
-   if(sw < 2)
-   {  e_pr_str((xe + xa - e_str_len((unsigned char *)text))/2,
-                  ya + 2, text, f->nt.fb, 0, 0, 0, 0);
-      e_pr_str((xe + xa - 4)/2, ya + 4, " OK ", f->nz.fb, 1, -1,
-                                              f->ns.fb, f->nt.fb);
-   }
-   else
-   {  e_pr_str((xe + xa - e_str_len((unsigned char *)text))/2,
-                  ya + 2, text, 112, 0, 0, 0, 0);
-      e_pr_str((xe + xa - 4)/2, ya + 4, " OK ", 32, 1, -1, 46, 112);
-   }
-   do
+ fk_cursor(0);
+ WpeMouseChangeShape(WpeErrorShape);
+ if ((len = strlen((char *)text)) < 20 ) len = 20;
+ xa = (80-len)/2 - 2;
+ xe = 82 - (80-len)/2;
+ if (sw == -1) header = "Message";
+ else if (sw == 0) header = "Error";
+ else if (sw == 1) header = "Serious Error";
+ else if (sw == 2) header = "Fatal Error";
+
+ if (sw < 2) pic = e_std_kst(xa, ya, xe, ye, header, 1, f->nr.fb, f->nt.fb, f->ne.fb);
+ if (sw == 2 || pic == NULL)
+ {
+  pic = e_open_view(xa, ya, xe, ye, 0, 0);
+  e_std_rahmen(xa, ya, xe, ye, header, 1, 0, 0);
+ }
+ if (sw < 2)
+ {
+  e_pr_str((xe + xa - e_str_len((unsigned char *)text))/2,
+    ya + 2, text, f->nt.fb, 0, 0, 0, 0);
+  e_pr_str((xe + xa - 4)/2, ya + 4, " OK ", f->nz.fb, 1, -1,
+    f->ns.fb, f->nt.fb);
+ }
+ else
+ {
+  e_pr_str((xe + xa - e_str_len((unsigned char *)text))/2,
+    ya + 2, text, 112, 0, 0, 0, 0);
+  e_pr_str((xe + xa - 4)/2, ya + 4, " OK ", 32, 1, -1, 46, 112);
+ }
+ do
+ {
 #if  MOUSE
-   {  if( (i = e_toupper(e_getch())) == -1)
-      i = e_er_mouse(xa+3, ya,(xe+xa-4)/2, ya+4);
+  if ((i = e_toupper(e_getch())) == -1)
+   i = e_er_mouse(xa+3, ya,(xe+xa-4)/2, ya+4);
 #else
-   {  i = e_toupper(e_getch());
+  i = e_toupper(e_getch());
 #endif
-   }  while( i != WPE_ESC && i != WPE_CR && i != 'O');
-   WpeMouseRestoreShape();
-   if(pic != NULL) e_close_view(pic, 1);
-   else e_cls(0, ' ');
-   fk_cursor(1);
-   if(sw == 1) e_quit(h_error.cn->f[h_error.cn->mxedt]);
-   if(sw > 0) WpeExit(sw);
-   return(sw);
+ } while (i != WPE_ESC && i != WPE_CR && i != 'O');
+ WpeMouseRestoreShape();
+ if (pic != NULL) e_close_view(pic, 1);
+ else e_cls(0, ' ');
+ fk_cursor(1);
+ if (sw == 1) e_quit(WpeEditor->f[WpeEditor->mxedt]);
+ if (sw > 0) WpeExit(sw);
+ return(sw);
 }
 
 /*   message with selection        */
 int e_message(int sw, char *str, FENSTER *f)
 {
-   int i, ret, mxlen = 0, anz = 0;
-   char **s;
-   W_OPTSTR *o = e_init_opt_kst(f);
+ int i, ret, mxlen = 0, anz = 0;
+ char **s;
+ W_OPTSTR *o = e_init_opt_kst(f);
 
-   if (!o)
-    return(-1);
+ if (!o)
+  return(-1);
 
-   s = StringToStringArray(str, &mxlen, 22, &anz);
+ s = StringToStringArray(str, &mxlen, 22, &anz);
 
-   o->ye = MAXSLNS - 6;
-   o->ya = o->ye - anz - 5;
-   o->xa = (MAXSCOL - mxlen - 6)/2;
-   o->xe = o->xa + mxlen + 6;
+ o->ye = MAXSLNS - 6;
+ o->ya = o->ye - anz - 5;
+ o->xa = (MAXSCOL - mxlen - 6)/2;
+ o->xe = o->xa + mxlen + 6;
 
-   o->bgsw = 0;
-   o->name = "Message";
-   for (i = 0; i < anz; i++)
-   {
-      e_add_txtstr((o->xe-o->xa-strlen(s[i]))/2, 2+i, s[i], o);
-      FREE(s[i]);
-   }
-   FREE(s);
-   if(!sw)
-   {  o->crsw = AltO;
-      e_add_bttstr((o->xe-o->xa-4)/2, o->ye-o->ya-2, 0, AltO, "Ok", NULL, o);
-   }
-   else
-   {  o->crsw = AltY;
-      e_add_bttstr(4, o->ye-o->ya-2, 0, AltY, "Yes", NULL, o);
-      e_add_bttstr((o->xe-o->xa-2)/2, o->ye-o->ya-2, 0, AltN, "No", NULL, o);
-      e_add_bttstr(o->xe-o->xa-9, o->ye-o->ya-2, -1, WPE_ESC, "Cancel", NULL, o);
-   }
-   ret = e_opt_kst(o);
-   freeostr(o);
-   return(ret == WPE_ESC ? WPE_ESC : (ret == AltN ? 'N' : 'Y'));
+ o->bgsw = 0;
+ o->name = "Message";
+ for (i = 0; i < anz; i++)
+ {
+  e_add_txtstr((o->xe-o->xa-strlen(s[i]))/2, 2+i, s[i], o);
+  FREE(s[i]);
+ }
+ FREE(s);
+ if (!sw)
+ {
+  o->crsw = AltO;
+  e_add_bttstr((o->xe-o->xa-4)/2, o->ye-o->ya-2, 0, AltO, "Ok", NULL, o);
+ }
+ else
+ {
+  o->crsw = AltY;
+  e_add_bttstr(4, o->ye-o->ya-2, 0, AltY, "Yes", NULL, o);
+  e_add_bttstr((o->xe-o->xa-2)/2, o->ye-o->ya-2, 0, AltN, "No", NULL, o);
+  e_add_bttstr(o->xe-o->xa-9, o->ye-o->ya-2, -1, WPE_ESC, "Cancel", NULL, o);
+ }
+ ret = e_opt_kst(o);
+ freeostr(o);
+ return(ret == WPE_ESC ? WPE_ESC : (ret == AltN ? 'N' : 'Y'));
 }
 
-/*         Erstes Eroeffnen eines Fensters                 */
+/*         First opening of a window                 */
 void e_firstl(FENSTER *f, int sw)
 {
-   f->pic = NULL;
-   f->pic = e_ed_kst(f, f->pic, sw);
-   if(f->pic == NULL)  e_error(e_msg[ERR_LOWMEM], 1, f->fb);
+ f->pic = NULL;
+ f->pic = e_ed_kst(f, f->pic, sw);
+ if (f->pic == NULL)
+  e_error(e_msg[ERR_LOWMEM], 1, f->fb);
 }
 
-/*         Schreiben des Filetypes    */
+/*         Writing of the file type    */
 int e_pr_filetype(FENSTER *f)
 {
  int frb = f->fb->es.fb;
@@ -269,18 +275,18 @@ int e_close_view(PIC *pic, int sw)
    return(sw);
 }
 
-/*    Rahmen fuer Edit-Fenster   */
+/*    Frame for edit window   */
 void e_ed_rahmen(FENSTER *f, int sw)
 {
  extern char *e_hlp;
  extern int nblst;
  extern WOPT *blst;
- char *header = NULL, tmp[80];
+ char *header = NULL;
 
- if (f->dtmd <= 'Z')
+ if (!DTMD_ISTEXT(f->dtmd))
  {
   if (f->datnam[0]) header = f->datnam;
-  if (f->dtmd == 'M')
+  if (f->dtmd == DTMD_FILEDROPDOWN)
    e_std_rahmen(f->a.x, f->a.y, f->e.x, f->e.y, header, sw, f->fb->er.fb,
      f->fb->es.fb);
   else
@@ -290,7 +296,7 @@ void e_ed_rahmen(FENSTER *f, int sw)
    e_pr_char(f->e.x-6, f->a.y, '0' + f->winnum, f->fb->nr.fb);
   else if (f->winnum >= 0)
    e_pr_char(f->e.x-6, f->a.y, 'A' - 10 + f->winnum, f->fb->nr.fb);
-  if (sw > 0 && (f->dtmd == 'F' || f->dtmd == 'D'))
+  if (sw > 0 && (f->dtmd == DTMD_FILEMANAGER || f->dtmd == DTMD_DATA))
   {
    if (f->zoom == 0)
     e_pr_char(f->e.x-3, f->a.y, WZN, f->fb->ne.fb);
@@ -320,23 +326,28 @@ void e_ed_rahmen(FENSTER *f, int sw)
  }
  if (f->datnam[0])
  {
-  if (strcmp(f->dirct, f->ed->dirct) == 0 || f->dtmd == 'm' ||
-    f->dtmd == 'h' || strcmp(f->datnam, BUFFER_NAME) == 0 ||
+  if (strcmp(f->dirct, f->ed->dirct) == 0 ||
+    f->dtmd == DTMD_HELP || strcmp(f->datnam, BUFFER_NAME) == 0 ||
     NUM_COLS_ON_SCREEN < 40)
-   strcpy(tmp, f->datnam);
+  {
+   header = (char *)WpeMalloc(strlen(f->datnam) + 1);
+   strcpy(header, f->datnam);
+  }
   else
   {
-   strcpy(tmp, f->dirct);
-   strcat(tmp, f->datnam);
+   header = (char *)WpeMalloc(strlen(f->dirct) + strlen(f->datnam) + 1);
+   strcpy(header, f->dirct);
+   strcat(header, f->datnam);
   }
-  header = tmp;
  }
  e_std_rahmen(f->a.x, f->a.y, f->e.x, f->e.y, header, sw, f->fb->er.fb,
    f->fb->es.fb);
+ if (header)
+  WpeFree(header);
  if (sw > 0)
  {
-  e_mouse_leiste(f->e.x, f->a.y+1, NUM_LINES_ON_SCREEN - 1, 0, f->fb->em.fb);
-  e_mouse_leiste(f->a.x+19, f->e.y, NUM_COLS_ON_SCREEN - 20, 1, f->fb->em.fb);
+  e_mouse_bar(f->e.x, f->a.y+1, NUM_LINES_ON_SCREEN - 1, 0, f->fb->em.fb);
+  e_mouse_bar(f->a.x+19, f->e.y, NUM_COLS_ON_SCREEN - 20, 1, f->fb->em.fb);
   if (f->zoom == 0)
    e_pr_char(f->e.x-3, f->a.y, WZN, f->fb->es.fb);
   else
@@ -401,11 +412,11 @@ int e_schirm(FENSTER *f, int sw)
 {
  int j;
 
- if (f->dtmd == 'F')
+ if (f->dtmd == DTMD_FILEMANAGER)
   return(WpeDrawFileManager(f));
- else if (f->dtmd == 'D')
+ else if (f->dtmd == DTMD_DATA)
   return(e_data_schirm(f));
- else if (f->dtmd == 'M')
+ else if (f->dtmd == DTMD_FILEDROPDOWN)
   return(e_pr_file_window((FLWND*)f->b, 1, sw, f->fb->er.fb, f->fb->ez.fb,
     f->fb->frft.fb));
  if (NUM_LINES_OFF_SCREEN_TOP < 0)
@@ -427,71 +438,77 @@ int e_schirm(FENSTER *f, int sw)
 /*   Move and modify window */
 int e_size_move(FENSTER *f)
 {
-   int xa = f->a.x, ya = f->a.y, xe = f->e.x, ye = f->e.y;
-   int c = 0, xmin = 26, ymin = 3;
-   e_ed_rahmen(f, 0);
-   if(f->dtmd == 'M') xmin = 15;
-   else if(f->dtmd <= 'Z') ymin = 9;
-   while ((c = e_getch()) != WPE_ESC && c != WPE_CR)
-   {  switch(c)
-      {  case CLE:
-	    if(xa > 0) {  xa--; xe--;  }
-	    break;
-	 case CRI:
-	    if(xe < MAXSCOL-1) {  xa++; xe++;  }
-	    break;
-	 case CUP:
-	    if(ya > 1) {  ya--; ye--;  }
-	    break;
-	 case CDO:
-	    if(ye < MAXSLNS-2) {  ya++; ye++;  }
-	    break;
-	 case SCLE:
-	 case CCLE:
-	    if((xe - xa) > xmin) xe--;
-	    break;
-	 case SCRI:
-	 case CCRI:
-	    if(xe < MAXSCOL-1) xe++;
-	    break;
-	 case SCUP:
-	 case BUP:
-	    if((ye - ya) > ymin) ye--;
-	    break;
-	 case SCDO:
-	 case BDO:
-	    if(ye < MAXSLNS-2) ye++;
-	    break;
-      }
-      if( xa != f->a.x || ya != f->a.y || xe != f->e.x || ye != f->e.y)
-      {
-	 f->a.x = xa;
-	 f->a.y = ya;
-	 f->e.x = xe;
-	 f->e.y = ye;
-	 f->pic = e_ed_kst(f, f->pic, 0);
-	 if(f->pic == NULL)  e_error(e_msg[ERR_LOWMEM], 1, f->fb);
-	 if(f->dtmd == 'M')
-	 {  FLWND *fw = (FLWND*) f->b;
-	    fw->xa = f->a.x+1; fw->xe = f->e.x;
-	    fw->ya = f->a.y+1; fw->ye = f->e.y;
-	 }
-	 e_cursor(f, 0);
-	 e_schirm(f, 0);
-      }
+ int xa = f->a.x, ya = f->a.y, xe = f->e.x, ye = f->e.y;
+ int c = 0, xmin = 26, ymin = 3;
+
+ e_ed_rahmen(f, 0);
+ if (f->dtmd == DTMD_FILEDROPDOWN)
+  xmin = 15;
+ else if (!DTMD_ISTEXT(f->dtmd))
+  ymin = 9;
+ while ((c = e_getch()) != WPE_ESC && c != WPE_CR)
+ {
+  switch (c)
+  {
+   case CLE:
+    if (xa > 0) {  xa--; xe--;  }
+    break;
+   case CRI:
+    if (xe < MAXSCOL-1) {  xa++; xe++;  }
+    break;
+   case CUP:
+    if (ya > 1) {  ya--; ye--;  }
+    break;
+   case CDO:
+    if (ye < MAXSLNS-2) {  ya++; ye++;  }
+    break;
+   case SCLE:
+   case CCLE:
+    if ((xe - xa) > xmin) xe--;
+    break;
+   case SCRI:
+   case CCRI:
+    if (xe < MAXSCOL-1) xe++;
+    break;
+   case SCUP:
+   case BUP:
+    if ((ye - ya) > ymin) ye--;
+    break;
+   case SCDO:
+   case BDO:
+    if (ye < MAXSLNS-2) ye++;
+    break;
+  }
+  if (xa != f->a.x || ya != f->a.y || xe != f->e.x || ye != f->e.y)
+  {
+   f->a.x = xa;
+   f->a.y = ya;
+   f->e.x = xe;
+   f->e.y = ye;
+   f->pic = e_ed_kst(f, f->pic, 0);
+   if (f->pic == NULL)  e_error(e_msg[ERR_LOWMEM], 1, f->fb);
+   if (f->dtmd == DTMD_FILEDROPDOWN)
+   {
+    FLWND *fw = (FLWND*) f->b;
+    fw->xa = f->a.x+1; fw->xe = f->e.x;
+    fw->ya = f->a.y+1; fw->ye = f->e.y;
    }
-   e_ed_rahmen(f, 1);
-   return(c);
+   e_cursor(f, 0);
+   e_schirm(f, 0);
+  }
+ }
+ e_ed_rahmen(f, 1);
+ return(c);
 }
 
 /*       Standard Box                                  */
 PIC *e_std_kst(int xa, int ya, int xe, int ye, char *name, int sw, int fr,
   int ft, int fes)
 {
-   PIC *pic = e_open_view(xa, ya, xe, ye, ft, 1);
-   if (pic == NULL) return (NULL);
-   e_std_rahmen(xa, ya, xe, ye, name, sw, fr, fes);
-   return(pic);
+ PIC *pic = e_open_view(xa, ya, xe, ye, ft, 1);
+ if (pic == NULL) return (NULL);
+ e_std_rahmen(xa, ya, xe, ye, name, sw, fr, fes);
+ return(pic);
 }
 
 PIC *e_change_pic(int xa, int ya, int xe, int ye, PIC *pic, int sw, int frb)
@@ -517,13 +534,11 @@ PIC *e_change_pic(int xa, int ya, int xe, int ye, PIC *pic, int sw, int frb)
       newpic->a.x = xa;
       newpic->a.y = ya;
 #ifndef NEWSTYLE
-#if !defined(DJGPP)
       if(!WpeIsXwin())
       {  newpic->e.x = xe;
 	 newpic->e.y = ye;
       }
       else
-#endif
       {  newpic->e.x = xe < MAXSCOL-2 ? xe + 2 : xe < MAXSCOL-1 ? xe + 1 : xe;
 	 newpic->e.y = ye < MAXSLNS-2 ? ye + 1 : ye;
       }
@@ -595,9 +610,7 @@ PIC *e_change_pic(int xa, int ya, int xe, int ye, PIC *pic, int sw, int frb)
       e_get_pic_xrect(xa, ya, xe, ye, newpic);
 #endif
 #ifndef NEWSTYLE
-#if !defined(DJGPP)
       if (WpeIsXwin())
-#endif
       {  if(xe < MAXSCOL-1) for(i = ya+1; i <= ye+1 && i < MAXSLNS-1; i++) 
 						e_pt_col(xe+1, i, SHDCOL);
          if(xe < MAXSCOL-2) for(i = ya+1; i <= ye+1 && i < MAXSLNS-1; i++)
@@ -656,7 +669,7 @@ int e_close_window(FENSTER *f)
  char text[256];
 
  f = cn->f[cn->mxedt];
- if (f->dtmd == 'F')
+ if (f->dtmd == DTMD_FILEMANAGER)
  {
   FLBFFR *b = (FLBFFR *)f->b;
 
@@ -671,7 +684,10 @@ int e_close_window(FENSTER *f)
   cn->curedt = cn->edt[cn->mxedt];
   e_close_view(f->pic, 1);
   if (f != f0 && f != NULL)
+  {
+   e_free_find(&f->fd);
    FREE(f);
+  }
   if (cn->mxedt > 0)
   {
    f = cn->f[cn->mxedt];
@@ -679,7 +695,7 @@ int e_close_window(FENSTER *f)
   }
   return(0);
  }
- if (f->dtmd == 'D')
+ if (f->dtmd == DTMD_DATA)
  {
   FLWND *fw = (FLWND *)f->b;
   int swt = f->ins;
@@ -697,7 +713,10 @@ int e_close_window(FENSTER *f)
   cn->curedt = cn->edt[cn->mxedt];
   e_close_view(f->pic, 1);
   if (f != f0 && f != NULL)
+  {
+   e_free_find(&f->fd);
    FREE(f);
+  }
   if (cn->mxedt > 0 && (swt < 5 || swt == 7))
   {
    f = cn->f[cn->mxedt];
@@ -724,13 +743,11 @@ int e_close_window(FENSTER *f)
     (maxname > 12))
   {
    remove(e_make_postf(text, f->datnam, ".ASV"));
-#ifdef UNIX
    remove(e_make_postf(text, f->datnam, ".ESV"));
-#endif
   }
   if (strcmp(f->datnam, "Messages") && strcmp(f->datnam, "Watches"))
    e_close_buffer(f->b);
-  if (f->dtmd == 'h' && f->ins == 8)
+  if (f->dtmd == DTMD_HELP && f->ins == 8)
    e_help_free(f);
   if (f->datnam != NULL)
    FREE(f->datnam);
@@ -743,7 +760,10 @@ int e_close_window(FENSTER *f)
  cn->curedt = cn->edt[cn->mxedt];
  e_close_view(f->pic, 1);
  if (f != f0 && f != NULL)
+ {
+  e_free_find(&f->fd);
   FREE(f);
+ }
  if (cn->mxedt > 0)
  {
   f = cn->f[cn->mxedt];
@@ -853,7 +873,7 @@ int e_ed_cascade(FENSTER *f)
  return(0);
 }
 
-/*   Fenster in Bausteinform bringen   */
+/*   Tile windows   */
 int e_ed_tile(FENSTER *f)
 {
  ECNT *cn = f->ed;
@@ -866,7 +886,7 @@ int e_ed_tile(FENSTER *f)
 
  for (i = cn->mxedt; i >= 1; i--)
  {
-  if ((!(cn->edopt & ED_OLD_TILE_METHOD)) && ((cn->f[i]->dtmd <= 'Z') ||
+  if ((!(cn->edopt & ED_OLD_TILE_METHOD)) && (!DTMD_ISTEXT(cn->f[i]->dtmd) ||
     ((WpeIsProg()) && ((strcmp(cn->f[i]->datnam, "Messages") == 0) ||
     (strcmp(cn->f[i]->datnam, "Watches") == 0)))))
   {
@@ -973,7 +993,7 @@ int e_ed_tile(FENSTER *f)
   while (!editorwin[j]) j++;
   cn->f[j]->a = e_set_pnt(atmp[i].x, atmp[i].y);
   cn->f[j]->e = e_set_pnt(etmp[i].x, etmp[i].y);
-  cn->f[i + 1]->zoom = 0; /* Make sure zoom is off */
+  cn->f[j]->zoom = 0; /* Make sure zoom is off */
  }
  ini_repaint(cn);
  for ( i = 1; i < cn->mxedt; i++)
@@ -998,37 +1018,42 @@ int e_ed_next(FENSTER *f)
 /*   write a line (screen content)     */
 void e_pr_line(int y, FENSTER *f)
 {
-   BUFFER *b = f->b;
-   SCHIRM *s = f->s;
-   int i, j, k, frb;
+ BUFFER *b = f->b;
+ SCHIRM *s = f->s;
+ int i, j, k, frb;
 #ifdef DEBUGGER
-   int fsw = 0;
+ int fsw = 0;
 #endif
-   for(i = j = 0; j < NUM_COLS_OFF_SCREEN_LEFT; j++, i++)
-   {  if(*(b->bf[y].s + i) == WPE_TAB) j += (f->ed->tabn - j % f->ed->tabn - 1);
-#ifdef UNIX
-      else if(((unsigned char) *(b->bf[y].s + i)) > 126)
-      {  j++;
-	 if(((unsigned char) *(b->bf[y].s + i)) < 128 + ' ') j++;
-      }
-      else if(*(b->bf[y].s + i) < ' ') j++;
-#endif
-   }
-   if(j > NUM_COLS_OFF_SCREEN_LEFT) i--;
+
+ for (i = j = 0; j < NUM_COLS_OFF_SCREEN_LEFT; j++, i++)
+ {
+  if (*(b->bf[y].s + i) == WPE_TAB)
+   j += (f->ed->tabn - j % f->ed->tabn - 1);
+  else if(((unsigned char) *(b->bf[y].s + i)) > 126)
+  {
+   j++;
+   if (((unsigned char) *(b->bf[y].s + i)) < 128 + ' ')
+    j++;
+  }
+  else if (*(b->bf[y].s + i) < ' ') j++;
+ }
+ if (j > NUM_COLS_OFF_SCREEN_LEFT) i--;
 #ifdef DEBUGGER
-   for (j = 1; j <= s->brp[0]; j++)
-   if(s->brp[j] == y) {  fsw = 1;  break;  }
-   for(j = NUM_COLS_OFF_SCREEN_LEFT; i < b->bf[y].len && j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
-   {  if( y == s->da.y && i >= s->da.x && i < s->de.x )
-      frb = s->fb->dy.fb;
-      else if(fsw) frb = s->fb->db.fb;
+ for (j = 1; j <= s->brp[0]; j++)
+  if (s->brp[j] == y) {  fsw = 1;  break;  }
+ for (j = NUM_COLS_OFF_SCREEN_LEFT; i < b->bf[y].len && j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
+ {
+  if (y == s->da.y && i >= s->da.x && i < s->de.x )
+   frb = s->fb->dy.fb;
+  else if (fsw) frb = s->fb->db.fb;
 /*	else if( (i == s->pt[0].x && y == s->pt[0].y) || (i == s->pt[1].x && y == s->pt[1].y)  */
-      else if( y == s->fa.y && i >= s->fa.x && i < s->fe.x )
-      frb = s->fb->ek.fb;
+  else if (y == s->fa.y && i >= s->fa.x && i < s->fe.x )
+   frb = s->fb->ek.fb;
 #else
-   for(j = NUM_COLS_OFF_SCREEN_LEFT; i < b->bf[y].len && j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
-   {  if( y == s->fa.y && i >= s->fa.x && i < s->fe.x )
-      frb = s->fb->ek.fb;
+ for (j = NUM_COLS_OFF_SCREEN_LEFT; i < b->bf[y].len && j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
+ {
+  if (y == s->fa.y && i >= s->fa.x && i < s->fe.x )
+   frb = s->fb->ek.fb;
 #endif
 /*	if( (i == s->pt[0].x && y == s->pt[0].y) || (i == s->pt[1].x && y == s->pt[1].y)  
          || (i == s->pt[2].x && y == s->pt[2].y) || (i == s->pt[3].x && y == s->pt[3].y)  
@@ -1037,143 +1062,147 @@ void e_pr_line(int y, FENSTER *f)
          || (i == s->pt[8].x && y == s->pt[8].y) || (i == s->pt[9].x && y == s->pt[9].y))
             frb = s->fb->ek.fb;
 */
-      else if( (y < s->mark_end.y && ( y > s->mark_begin.y ||
-               (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) ||
-            (y == s->mark_end.y && i < s->mark_end.x && ( y > s->mark_begin.y ||
-               (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) )
-      frb = s->fb->ez.fb;
-      else if(f->dtmd == 'b' && *(b->bf[y].s + i) == 0)
-      frb = s->fb->et.fb - 16;
-      else if(f->dtmd == 'b' && (unsigned char) *(b->bf[y].s + i) == 0xff)
-      frb = s->fb->et.fb + 16;
-      else
-      frb = s->fb->et.fb;
-      
-      if(f->dtmd == 'h')
-      {  if(*(b->bf[y].s + i) == HBG || *(b->bf[y].s + i) == HFB ||
-		*(b->bf[y].s + i) == HHD || *(b->bf[y].s + i) == HBB)
-	 {  if(*(b->bf[y].s + i) == HHD) frb = s->fb->hh.fb;
-	    else if(*(b->bf[y].s + i) == HBB) frb = s->fb->hm.fb;
-	    else frb = s->fb->hb.fb;
+  else if ((y < s->mark_end.y && ( y > s->mark_begin.y ||
+    (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) ||
+    (y == s->mark_end.y && i < s->mark_end.x && ( y > s->mark_begin.y ||
+    (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) )
+   frb = s->fb->ez.fb;
+  else
+   frb = s->fb->et.fb;
+
+  if (f->dtmd == DTMD_HELP)
+  {
+   if (*(b->bf[y].s + i) == HBG || *(b->bf[y].s + i) == HFB ||
+     *(b->bf[y].s + i) == HHD || *(b->bf[y].s + i) == HBB)
+   {
+    if (*(b->bf[y].s + i) == HHD) frb = s->fb->hh.fb;
+    else if(*(b->bf[y].s + i) == HBB) frb = s->fb->hm.fb;
+    else frb = s->fb->hb.fb;
 #ifdef NEWSTYLE
-	    if(*(b->bf[y].s + i) != HBB) k = j;
-	    else k = -1;
+    if (*(b->bf[y].s + i) != HBB) k = j;
+    else k = -1;
 #endif
-	    for(i++; b->bf[y].s[i] != HED && i < b->bf[y].len
-				&& j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
-               e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
-			  y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, *(b->bf[y].s + i), frb);
-	    j--;
+    for (i++; b->bf[y].s[i] != HED && i < b->bf[y].len &&
+      j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
+     e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
+       y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, *(b->bf[y].s + i), frb);
+    j--;
 #ifdef NEWSTYLE
-	    if(WpeIsXwin() && k >= 0)
-	    e_make_xrect(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + k + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-			f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, 0);
+    if (WpeIsXwin() && k >= 0)
+     e_make_xrect(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + k + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+       f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, 0);
 #endif
-	    continue;
-	 }
-	 else if(*(b->bf[y].s + i) == HFE)
-	 {  for(; j < COL_NUM_ON_SCREEN_RIGHT; j++)
-	    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                              ' ', s->fb->hh.fb);
-	    return;
-	 }
-	 else if(*(b->bf[y].s + i) == HNF)
-	 {  for(k = j, i++; b->bf[y].s[i] != ':' && i < b->bf[y].len
-				&& j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
-               e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
-                 y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, *(b->bf[y].s + i), s->fb->hb.fb);
-#ifdef NEWSTYLE
-	    if (WpeIsXwin())
-	    e_make_xrect(f->a.x-NUM_COLS_OFF_SCREEN_LEFT+k+1, y-NUM_LINES_OFF_SCREEN_TOP+f->a.y+1,
-			f->a.x-NUM_COLS_OFF_SCREEN_LEFT+j, y-NUM_LINES_OFF_SCREEN_TOP+f->a.y+1, 0);
-#endif
-	    for(; b->bf[y].s[i] != HED && i < b->bf[y].len
-				&& j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
-               e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
-                 y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, *(b->bf[y].s + i), frb);
-            for(i++; b->bf[y].s[i] != HED && i < b->bf[y].len
-				&& j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
-               e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
-                 y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, ' ', frb);
-            for(; b->bf[y].s[i] != '.' && i < b->bf[y].len
-				&& j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
-               e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
-                 y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, ' ', frb);
-            j--;
-            continue;
-	 }
-	 else if(*(b->bf[y].s + i) == HED) {  j--;  continue;  }
-      }
-      if(*(b->bf[y].s + i) == WPE_TAB)
-      for(k = f->ed->tabn - j % f->ed->tabn; k > 1 &&
-                         j < NUM_COLS_ON_SCREEN + NUM_COLS_OFF_SCREEN_LEFT - 2; k--, j++)
-      e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                                                 ' ', frb);
-#ifdef UNIX
-      else if(!WpeIsXwin() && ((unsigned char)*(b->bf[y].s + i)) > 126)
-      {  e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j +1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                                                 '@', frb);
-	 if(++j >= COL_NUM_ON_SCREEN_RIGHT) return;
-	 if(((unsigned char)*(b->bf[y].s + i)) < 128 + ' '
-                                   && j < COL_NUM_ON_SCREEN_RIGHT)
-	 {  e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j +1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                                                 '^', frb);
-	    if(++j >= COL_NUM_ON_SCREEN_RIGHT) return;
-	 }
-      }
-      else if(*(b->bf[y].s + i) < ' ')
-      {  e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j +1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                                                 '^', frb);
-	 if(++j >= COL_NUM_ON_SCREEN_RIGHT) return;
-      }
-#endif
-      if(*(b->bf[y].s + i) == WPE_TAB)
-      e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,                  ' ', frb);
-#ifdef UNIX
-      else if(!WpeIsXwin() && ((unsigned char)*(b->bf[y].s + i)) > 126
-                                   && j < COL_NUM_ON_SCREEN_RIGHT)
-      {  if(((unsigned char)*(b->bf[y].s + i)) < 128 + ' ')
-	 e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                      ((unsigned char) *(b->bf[y].s + i)) + 'A' - 129, frb);
-	 else
-	 e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-			((unsigned char) *(b->bf[y].s + i)) - 128, frb);
-      }
-      else if(*(b->bf[y].s + i) < ' ' && j < COL_NUM_ON_SCREEN_RIGHT)
-      e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                          *(b->bf[y].s + i) + 'A' - 1, frb);
-#endif
-      else
-      e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                                    *(b->bf[y].s + i), frb);
+    continue;
    }
-   
-   if (i == b->bf[y].len && f->dtmd == 's' && j < COL_NUM_ON_SCREEN_RIGHT)
-   {  if( (y < s->mark_end.y && ( y > s->mark_begin.y ||
-               (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) ||
-            (y == s->mark_end.y && i < s->mark_end.x && ( y > s->mark_begin.y ||
-               (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) )
-      {  if( *(b->bf[y].s + i) == WPE_WR)
-	 e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                              PWR, s->fb->ez.fb);
-	 else
-	 e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                              PNL, s->fb->ez.fb);
-      }
-      else
-      {  if( *(b->bf[y].s + i) == WPE_WR)
-	 e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                              PWR, s->fb->et.fb);
-	 else
-	 e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                              PNL, s->fb->et.fb);
-      }
-      j++;
+   else if (*(b->bf[y].s + i) == HFE)
+   {
+    for (; j < COL_NUM_ON_SCREEN_RIGHT; j++)
+     e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+       ' ', s->fb->hh.fb);
+    return;
    }
-   
-   for(; j < COL_NUM_ON_SCREEN_RIGHT; j++)
+   else if (*(b->bf[y].s + i) == HNF)
+   {
+    for (k = j, i++; b->bf[y].s[i] != ':' && i < b->bf[y].len &&
+      j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
+     e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
+       y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, *(b->bf[y].s + i), s->fb->hb.fb);
+#ifdef NEWSTYLE
+    if (WpeIsXwin())
+     e_make_xrect(f->a.x-NUM_COLS_OFF_SCREEN_LEFT+k+1, y-NUM_LINES_OFF_SCREEN_TOP+f->a.y+1,
+       f->a.x-NUM_COLS_OFF_SCREEN_LEFT+j, y-NUM_LINES_OFF_SCREEN_TOP+f->a.y+1, 0);
+#endif
+    for (; b->bf[y].s[i] != HED && i < b->bf[y].len &&
+      j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
+     e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
+       y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, *(b->bf[y].s + i), frb);
+    for (i++; b->bf[y].s[i] != HED && i < b->bf[y].len &&
+      j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
+     e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
+       y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, ' ', frb);
+    for (; b->bf[y].s[i] != '.' && i < b->bf[y].len &&
+      j < COL_NUM_ON_SCREEN_RIGHT; i++, j++)
+     e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1,
+       y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1, ' ', frb);
+    j--;
+    continue;
+   }
+   else if (*(b->bf[y].s + i) == HED) {  j--;  continue;  }
+  }
+  if (*(b->bf[y].s + i) == WPE_TAB)
+   for (k = f->ed->tabn - j % f->ed->tabn; k > 1 &&
+     j < NUM_COLS_ON_SCREEN + NUM_COLS_OFF_SCREEN_LEFT - 2; k--, j++)
+    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+      ' ', frb);
+  else if (!WpeIsXwin() && ((unsigned char)*(b->bf[y].s + i)) > 126)
+  {
+   e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j +1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+     '@', frb);
+   if (++j >= COL_NUM_ON_SCREEN_RIGHT) return;
+   if (((unsigned char)*(b->bf[y].s + i)) < 128 + ' ' &&
+     j < COL_NUM_ON_SCREEN_RIGHT)
+   {
+    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j +1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+      '^', frb);
+    if (++j >= COL_NUM_ON_SCREEN_RIGHT) return;
+   }
+  }
+  else if (*(b->bf[y].s + i) < ' ')
+  {
+   e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j +1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+     '^', frb);
+   if (++j >= COL_NUM_ON_SCREEN_RIGHT) return;
+  }
+  if (*(b->bf[y].s + i) == WPE_TAB)
+   e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,                  ' ', frb);
+  else if (!WpeIsXwin() && ((unsigned char)*(b->bf[y].s + i)) > 126 &&
+    j < COL_NUM_ON_SCREEN_RIGHT)
+  {
+   if (((unsigned char)*(b->bf[y].s + i)) < 128 + ' ')
+    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+      ((unsigned char) *(b->bf[y].s + i)) + 'A' - 129, frb);
+   else
+    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+      ((unsigned char) *(b->bf[y].s + i)) - 128, frb);
+  }
+  else if (*(b->bf[y].s + i) < ' ' && j < COL_NUM_ON_SCREEN_RIGHT)
    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
-                                              ' ', s->fb->et.fb);
+     *(b->bf[y].s + i) + 'A' - 1, frb);
+  else
+   e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+     *(b->bf[y].s + i), frb);
+ }
+
+ if ((i == b->bf[y].len) && (f->ed->edopt & ED_SHOW_ENDMARKS) &&
+   (DTMD_ISMARKABLE(f->dtmd)) && (j < COL_NUM_ON_SCREEN_RIGHT))
+ {
+  if ((y < s->mark_end.y && ( y > s->mark_begin.y ||
+    (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) ||
+    (y == s->mark_end.y && i < s->mark_end.x && ( y > s->mark_begin.y ||
+    (y == s->mark_begin.y && i >= s->mark_begin.x) ) ) )
+  {
+   if (*(b->bf[y].s + i) == WPE_WR)
+    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+      PWR, s->fb->ez.fb);
+   else
+    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+      PNL, s->fb->ez.fb);
+  }
+  else
+  {
+   if (*(b->bf[y].s + i) == WPE_WR)
+    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+      PWR, s->fb->et.fb);
+   else
+    e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+      PNL, s->fb->et.fb);
+  }
+  j++;
+ }
+
+ for (; j < COL_NUM_ON_SCREEN_RIGHT; j++)
+  e_pr_char(f->a.x - NUM_COLS_OFF_SCREEN_LEFT + j + 1, y - NUM_LINES_OFF_SCREEN_TOP + f->a.y + 1,
+    ' ', s->fb->et.fb);
 }
 
 /*   draw standard-box frame  */
@@ -1274,45 +1303,48 @@ struct dirfile *e_add_df(char *str, struct dirfile *df)
 
 int e_sv_window(int xa, int ya, int *n, struct dirfile *df, FENSTER *f)
 {
-   ECNT *cn = f->ed;
-   int ret, ye = ya + 6;
-   int xe = xa +21;
-   FLWND *fw = MALLOC(sizeof(FLWND));
+ ECNT *cn = f->ed;
+ int ret, ye = ya + 6;
+ int xe = xa +21;
+ FLWND *fw = MALLOC(sizeof(FLWND));
 
-   if( (f = (FENSTER *) MALLOC(sizeof(FENSTER))) == NULL)
-   e_error(e_msg[ERR_LOWMEM], 1, cn->fb);
-   if(xe > MAXSCOL-3) {  xe = MAXSCOL - 3;  xa = xe - 21;  }
-   if(ye > MAXSLNS-3) {  ye = MAXSLNS - 3;  ya = ye - 6;  }
-   f->fb = cn->fb;
-   f->a = e_set_pnt(xa, ya);
-   f->e = e_set_pnt(xe, ye);
-   f->dtmd = 'M';
-   f->zoom = 0;
-   f->ed = cn;
-   f->c_sw = NULL;
-   f->c_st = NULL;
-   f->winnum = -1;
-   f->datnam = "";
-   if(!(f->pic = e_ed_kst(f, NULL, 1)))
-   {  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(0);  }
-   f->b = (BUFFER *)fw;
-   fw->mxa = xa; fw->mxe = xe; fw->mya = ya; fw->mye = ye;
-   fw->xa = xa+1; fw->xe = xe; fw->ya = ya+1; fw->ye = ye;
-   fw->df = df; fw->srcha = 0; fw->f = f;
-   fw->nf = fw->ia = fw->ja = 0;
-   do
-   {  ret = e_file_window(0, fw, f->fb->er.fb, f->fb->ez.fb);
+ if ((f = (FENSTER *) MALLOC(sizeof(FENSTER))) == NULL)
+  e_error(e_msg[ERR_LOWMEM], 1, cn->fb);
+ if (xe > MAXSCOL-3) {  xe = MAXSCOL - 3;  xa = xe - 21;  }
+ if (ye > MAXSLNS-3) {  ye = MAXSLNS - 3;  ya = ye - 6;  }
+ f->fb = cn->fb;
+ f->a = e_set_pnt(xa, ya);
+ f->e = e_set_pnt(xe, ye);
+ f->dtmd = DTMD_FILEDROPDOWN;
+ f->zoom = 0;
+ f->ed = cn;
+ f->c_sw = NULL;
+ f->c_st = NULL;
+ f->fd.dirct = NULL;
+ f->winnum = -1;
+ f->datnam = "";
+ if (!(f->pic = e_ed_kst(f, NULL, 1)))
+ {  e_error(e_msg[ERR_LOWMEM], 0, f->fb);  return(0);  }
+ f->b = (BUFFER *)fw;
+ fw->mxa = xa; fw->mxe = xe; fw->mya = ya; fw->mye = ye;
+ fw->xa = xa+1; fw->xe = xe; fw->ya = ya+1; fw->ye = ye;
+ fw->df = df; fw->srcha = 0; fw->f = f;
+ fw->nf = fw->ia = fw->ja = 0;
+ do
+ {
+  ret = e_file_window(0, fw, f->fb->er.fb, f->fb->ez.fb);
 #if MOUSE
-      if(ret < 0) ret = e_rahmen_mouse(f);
+  if (ret < 0) ret = e_rahmen_mouse(f);
 #endif
-      if((ret == AF2 && !(f->ed->edopt & ED_CUA_STYLE))
-		|| (f->ed->edopt & ED_CUA_STYLE && ret == CtrlL)) e_size_move(f);
-   } while(ret != WPE_CR && ret != WPE_ESC);
-   *n = fw->nf;
-   e_close_view(f->pic, 1);
-   FREE(fw);
-   FREE(f);
-   return(ret);
+  if ((ret == AF2 && !(f->ed->edopt & ED_CUA_STYLE)) ||
+    (f->ed->edopt & ED_CUA_STYLE && ret == CtrlL))
+   e_size_move(f);
+ } while(ret != WPE_CR && ret != WPE_ESC);
+ *n = fw->nf;
+ e_close_view(f->pic, 1);
+ FREE(fw);
+ FREE(f);
+ return(ret);
 }
 
 int e_schr_lst_wsv(char *str, int xa, int ya, int n, int len, int ft,
@@ -1343,101 +1375,100 @@ int e_schr_nchar_wsv(char *str, int x, int y, int n, int max, int col,
   int csw)
 #ifdef NEWSTYLE
 {
-   e_pr_char(x+max-3, y, ' ', csw);
-   e_pr_char(x+max-2, y, WSW, csw);
-   e_pr_char(x+max-1, y, ' ', csw);
-   e_make_xrect(x+max-3, y, x+max-1, y, 0);
-   return(e_schr_nchar(str, x, y, n, max-3, col));
+ e_pr_char(x+max-3, y, ' ', csw);
+ e_pr_char(x+max-2, y, WSW, csw);
+ e_pr_char(x+max-1, y, ' ', csw);
+ e_make_xrect(x+max-3, y, x+max-1, y, 0);
+ return(e_schr_nchar(str, x, y, n, max-3, col));
 }
 #else
 {
-#if !defined(NO_XWINDOWS) || defined(DJGPP) 
-   int swcol = (e_gt_col(x+max, y) / 16) * 16;
-#ifndef DJGPP
-   if (WpeIsXwin())
+#if !defined(NO_XWINDOWS)
+ int swcol = (e_gt_col(x+max, y) / 16) * 16;
+ if (WpeIsXwin())
+ {
+  e_pr_char(x+max, y, SCR, swcol);
+  e_pr_char(x+max, y+1, SCD, swcol);
+  e_pr_char(x+max-1, y+1, SCD, swcol);
+  e_pr_char(x+max-2, y+1, SCD, swcol);
+ }
 #endif
-   {  e_pr_char(x+max, y, SCR, swcol);
-      e_pr_char(x+max, y+1, SCD, swcol);
-      e_pr_char(x+max-1, y+1, SCD, swcol);
-      e_pr_char(x+max-2, y+1, SCD, swcol);
-   }
-#endif
-   e_pr_char(x+max-3, y, ' ', csw);
-   e_pr_char(x+max-2, y, WSW, csw);
-   e_pr_char(x+max-1, y, ' ', csw);
-   return(e_schr_nchar(str, x, y, n, max-4, col));
+ e_pr_char(x+max-3, y, ' ', csw);
+ e_pr_char(x+max-2, y, WSW, csw);
+ e_pr_char(x+max-1, y, ' ', csw);
+ return(e_schr_nchar(str, x, y, n, max-4, col));
 }
 #endif
 
 int e_mess_win(char *header, char *str, PIC **pic, FENSTER *f)
 {
-   extern int (*e_u_kbhit)(void);
+ ECNT *cn = f->ed;
+ extern int (*e_u_kbhit)(void);
 #if MOUSE
-   extern struct mouse e_mouse;
+ extern struct mouse e_mouse;
 #endif
-   int xa, ya, xe, ye, num, anz = 0, mxlen = 0, i, j;
-   char **s;
+ int xa, ya, xe, ye, num, anz = 0, mxlen = 0, i, j;
+ char **s;
 
-   s = StringToStringArray(str, &mxlen, strlen(header) + 8, &anz);
+ s = StringToStringArray(str, &mxlen, strlen(header) + 8, &anz);
 
-   ya = (MAXSLNS - anz - 6)/2;
-   ye = ya + anz + 5;
-   xa = (MAXSCOL - mxlen - 6)/2;
-   xe = xa + mxlen + 6;
-   if(ya < 2) ya = 2;
-   if(ye > MAXSLNS-3) ye = MAXSLNS - 3;
-   num = anz;
-   if(num > ye - ya - 5)
-   {  num = ye - ya - 5;
-      strcpy(s[num-1], "...");
-   }
-   
-   if(!(*pic) || (*pic)->e.x != xe || (*pic)->a.x != xa || (*pic)->e.x < xe)
-   {  *pic = e_change_pic(xa, ya, xe, ye, *pic, 1, f->fb->nt.fb);
-      for(i = xa + 1; i < xe; i++)
-      {  e_pr_char(i, ye-2, ' ', f->fb->nt.fb);
-	 e_pr_char(i, ye-1, ' ', f->fb->nt.fb);
-      }
-      e_pr_str((xe + xa - 6)/2, ye-2, "Ctrl C", f->fb->nz.fb, -1, -1,
-                                              f->fb->ns.fb, f->fb->nt.fb);
-   }
-   e_std_rahmen(xa, ya, xe, ye, header, 1, f->fb->nr.fb, f->fb->ne.fb);
-   for(i = xa + 1; i < xe; i++)
-   e_pr_char(i, ya+1, ' ', f->fb->nr.fb);
-   for(j = 0; j < num; j++)
-   {  e_pr_char(xa+1, ya+2+j, ' ', f->fb->nt.fb);
-      e_pr_char(xa+2, ya+2+j, ' ', f->fb->nt.fb);
-      e_pr_str(xa+3, ya+2+j, s[j], f->fb->nt.fb, 0, 0, 0, 0);
-      for(i = xa+strlen(s[j])+3; i < xe; i++)
-      e_pr_char(i, ya+2+j, ' ', f->fb->nt.fb);
-   }
-   for(j += ya+2; j < ye-2; j++)
-   for(i = xa + 1; i < xe; i++)
-   e_pr_char(i, j, ' ', f->fb->nt.fb);
-   for(i = 0; i < anz; i++) FREE(s[i]);
-   FREE(s);
-#ifdef DJGPP
-   while(i = e_kbhit())
-   {  if(i == -1 && e_mouse.y == ye-2
-		&& e_mouse.x > (xe + xa - 10)/2
-		&& e_mouse.x < (xe + xa + 6)/2 ) i = CtrlC;
-      if(i == CtrlC) break;
-   }
-#else
+ ya = (MAXSLNS - anz - 6)/2;
+ ye = ya + anz + 5;
+ xa = (MAXSCOL - mxlen - 6)/2;
+ xe = xa + mxlen + 6;
+ if (ya < 2) ya = 2;
+ if (ye > MAXSLNS-3) ye = MAXSLNS - 3;
+ num = anz;
+ if (num > ye - ya - 5)
+ {
+  num = ye - ya - 5;
+  strcpy(s[num-1], "...");
+ }
+
+ if (!(*pic) || (*pic)->e.x != xe || (*pic)->a.x != xa || (*pic)->e.x < xe)
+ {
+  *pic = e_change_pic(xa, ya, xe, ye, *pic, 1, cn->fb->nt.fb);
+  for (i = xa + 1; i < xe; i++)
+  {
+   e_pr_char(i, ye-2, ' ', f->fb->nt.fb);
+   e_pr_char(i, ye-1, ' ', f->fb->nt.fb);
+  }
+  e_pr_str((xe + xa - 6)/2, ye-2, "Ctrl C", cn->fb->nz.fb, -1, -1,
+    cn->fb->ns.fb, cn->fb->nt.fb);
+ }
+ e_std_rahmen(xa, ya, xe, ye, header, 1, cn->fb->nr.fb, cn->fb->ne.fb);
+ for (i = xa + 1; i < xe; i++)
+  e_pr_char(i, ya+1, ' ', cn->fb->nr.fb);
+ for (j = 0; j < num; j++)
+ {
+  e_pr_char(xa+1, ya+2+j, ' ', cn->fb->nt.fb);
+  e_pr_char(xa+2, ya+2+j, ' ', cn->fb->nt.fb);
+  e_pr_str(xa+3, ya+2+j, s[j], cn->fb->nt.fb, 0, 0, 0, 0);
+  for (i = xa+strlen(s[j])+3; i < xe; i++)
+   e_pr_char(i, ya+2+j, ' ', cn->fb->nt.fb);
+ }
+ for (j += ya+2; j < ye-2; j++)
+  for (i = xa + 1; i < xe; i++)
+   e_pr_char(i, j, ' ', cn->fb->nt.fb);
+ for (i = 0; i < anz; i++)
+  FREE(s[i]);
+ FREE(s);
 #ifndef NO_XWINDOWS
-   if (WpeIsXwin())
-   {  while((i = (*e_u_kbhit)()))
-      {  if(i == -1 && e_mouse.y == ye-2
-		&& e_mouse.x > (xe + xa - 10)/2
-		&& e_mouse.x < (xe + xa + 6)/2 ) i = CtrlC;
-	 if(i == CtrlC) break;
-      }
-   }
-   else
+ if (WpeIsXwin())
+ {
+  while ((i = (*e_u_kbhit)()))
+  {
+   if (i == -1 && e_mouse.y == ye-2 && e_mouse.x > (xe + xa - 10)/2 &&
+     e_mouse.x < (xe + xa + 6)/2 )
+    i = CtrlC;
+   if (i == CtrlC) break;
+  }
+ }
+ else
 #endif
-   while((i = (*e_u_kbhit)()) && i != CtrlC);
-#endif
-   return(i == CtrlC ? 1 : 0);
+  while ((i = (*e_u_kbhit)()) && i != CtrlC)
+   ;
+ return(i == CtrlC ? 1 : 0);
 }
 
 int e_opt_sec_box(int xa, int ya, int num, OPTK *opt, FENSTER *f, int sw)
@@ -1526,7 +1557,7 @@ int e_list_all_win(FENSTER *f)
  int i;
 
  for (i = f->ed->mxedt; i > 0; i--)
-  if (f->ed->f[i]->dtmd == 'D' && f->ed->f[i]->ins == 7)
+  if (f->ed->f[i]->dtmd == DTMD_DATA && f->ed->f[i]->ins == 7)
   {
    e_switch_window(f->ed->edt[i], f);
    return(0);
