@@ -15,7 +15,7 @@
 #define WpeDllInit WpeXtermInit
 #endif
 
-int e_X_sw_color(FARBE *fb);
+int e_X_sw_color(void);
 int fk_show_cursor(void);
 int e_ini_size(void);
 int e_x_getch(void);
@@ -27,7 +27,7 @@ int e_x_sys_ini(void);
 int e_x_sys_end(void);
 int fk_x_putchar(int c);
 int x_bioskey(void);
-int e_x_system(char *exe);
+int e_x_system(const char *exe);
 int e_x_cp_X_to_buffer(FENSTER *f);
 int e_x_copy_X_buffer(FENSTER *f);
 int e_x_paste_X_buffer(FENSTER *f);
@@ -308,8 +308,9 @@ int e_ini_size()
 #define A_Underline	1
 #define A_Bold		16
 
-int e_X_sw_color(FARBE *fb)
+int e_X_sw_color()
 {
+ FARBE *fb = WpeEditor->fb;
  fb->er = e_n_clr(A_Normal);
  fb->et = e_n_clr(A_Normal);
  fb->ez = e_n_clr(A_Reverse);
@@ -459,10 +460,9 @@ int e_x_change(PIC *pic)
     if (size_hints.width != MAXSCOL * WpeXInfo.font_width ||
       size_hints.height != MAXSLNS * WpeXInfo.font_height)
     {
-     extern struct EXT h_error;
      MAXSCOL = size_hints.width / WpeXInfo.font_width;
      MAXSLNS = size_hints.height / WpeXInfo.font_height;
-     e_x_repaint_desk(h_error.cn->f[h_error.cn->mxedt]);
+     e_x_repaint_desk(WpeEditor->f[WpeEditor->mxedt]);
     }
     break;
    case KeyPress:
@@ -541,10 +541,9 @@ int e_x_getch()
     if (size_hints.width != MAXSCOL * WpeXInfo.font_width ||
       size_hints.height != MAXSLNS * WpeXInfo.font_height)
     {
-     extern struct EXT h_error;
      MAXSCOL = size_hints.width / WpeXInfo.font_width;
      MAXSLNS = size_hints.height / WpeXInfo.font_height;
-     e_x_repaint_desk(h_error.cn->f[h_error.cn->mxedt]);
+     e_x_repaint_desk(WpeEditor->f[WpeEditor->mxedt]);
     }
     break;
    case ClientMessage:
@@ -556,8 +555,7 @@ int e_x_getch()
       (report.xclient.format == 32 &&
         report.xclient.data.l[0] == WpeXInfo.delete_atom)))
     {
-     extern struct EXT h_error;
-     e_quit(h_error.cn->f[h_error.cn->mxedt]);
+     e_quit(WpeEditor->f[WpeEditor->mxedt]);
     }
     break;
    case KeyPress:
@@ -790,7 +788,7 @@ int x_bioskey()
  return(e_mouse.k);
 }
 
-int e_x_system(char *exe)
+int e_x_system(const char *exe)
 {
  FILE *fp;
  int ret;
@@ -858,9 +856,9 @@ int e_x_repaint_desk(FENSTER *f)
   if (cn->f[i]->e.y >= MAXSLNS-1) cn->f[i]->e.y = MAXSLNS-2;
   if (cn->f[i]->e.x - cn->f[i]->a.x < 26)
    cn->f[i]->a.x = cn->f[i]->e.x - 26;
-  if (cn->f[i]->dtmd <= 'Z' && cn->f[i]->e.y - cn->f[i]->a.y < 9)
+  if (!DTMD_ISTEXT(cn->f[i]->dtmd) && cn->f[i]->e.y - cn->f[i]->a.y < 9)
    cn->f[i]->a.y = cn->f[i]->e.y - 9;
-  else if (cn->f[i]->dtmd > 'Z' && cn->f[i]->e.y - cn->f[i]->a.y < 3)
+  else if (DTMD_ISTEXT(cn->f[i]->dtmd) && cn->f[i]->e.y - cn->f[i]->a.y < 3)
    cn->f[i]->a.y = cn->f[i]->e.y - 3;
  }
  for (i = 1; i < cn->mxedt; i++)
@@ -953,6 +951,11 @@ int e_x_cp_X_to_buffer(FENSTER *f)
   XGetWindowProperty(WpeXInfo.display, WpeXInfo.window, WpeXInfo.property_atom,
     0, 1000000, FALSE, WpeXInfo.text_atom, &type, &format, &nitems, &bytes_left,
     &str);
+  if (type == None)
+  {
+   /* Specified property does not exit*/
+   return 0;
+  }
   n = strlen(str);
  }
 #else
